/* $RCSfile: StixCursor.cxx,v $
 * $Revision: 1.6 $ $Date: 2003/05/01 23:49:20 $
 * Auth: Dave Loffredo (loffredo@steptools.com)
 * 
 * Copyright (c) 1991-2003 by STEP Tools Inc. 
 * All Rights Reserved.
 * 
 * This file is part of the STIX Library
 * 
 * This file may be distributed and/or modified under the terms of 
 * the GNU General Public License version 2 as published by the Free
 * Software Foundation and appearing in the file LICENSE.GPL included
 * with this file.
 * 
 * THIS FILE IS PROVIDED "AS IS" WITH NO WARRANTY OF ANY KIND,
 * INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * STEP Tools Inc. also offers licenses for commercial use of this
 * software in non-GPL applications.  Contact info@steptools.com for
 * terms and pricing.
 * 
 * 		----------------------------------------
 */


#include <rose.h>
#include <integrated_cnc_schema.h>
#include <integrated_cnc_schema_ROSE_LOAD.h>
#include "stix_imp.h"


/* ========================================
 * Main Concept Cursors - These things allow easy traversal over the
 * internal indicies within the StixNC interface object. Since all of
 * these behave similarly, define a macro to implement several of the
 * member functions. The next() function differs, so do that by hand.
 */

#define IMPLEMENT_STIXNC_CURSOR(cls,var) \
cls::cls(StixNC * nc) 	{ m_nc = nc; m_i = 0; } \
void cls::rewind() 	{ m_i = 0; } \
unsigned cls::size() 	{ return (m_nc? m_nc->var.size(): 0); }


IMPLEMENT_STIXNC_CURSOR(StixWorkpieceCursor,m_wkpcs);
product_definition * StixWorkpieceCursor::next() 
{
    StixWorkpiece * idx = (StixWorkpiece *)
  	((m_nc && (m_i<m_nc->m_wkpcs.size()))?
  	 (m_nc->m_wkpcs[m_i++]): 0);

    return (idx? idx-> m_pdef: 0);
}


IMPLEMENT_STIXNC_CURSOR(StixProjectCursor,m_projects);
product * StixProjectCursor::next() 
{
    StixProject * idx = (StixProject *)
  	((m_nc && (m_i<m_nc->m_projects.size()))?
  	 (m_nc->m_projects[m_i++]): 0);

    return (idx? idx-> m_product: 0);
}


IMPLEMENT_STIXNC_CURSOR(StixSetupCursor,m_setups);
product * StixSetupCursor::next() 
{
    StixSetup * idx = (StixSetup *)
  	((m_nc && (m_i<m_nc->m_setups.size()))?
  	 (m_nc->m_setups[m_i++]): 0);

    return (idx? idx-> m_product: 0);
}



IMPLEMENT_STIXNC_CURSOR(StixFeatureCursor,m_features);
shape_aspect * StixFeatureCursor::next() 
{
    StixFeature * idx = (StixFeature *)
  	((m_nc && (m_i<m_nc->m_features.size()))?
  	 (m_nc->m_features[m_i++]): 0);

    return (idx? idx-> m_sa: 0);
}


IMPLEMENT_STIXNC_CURSOR(StixComponentCursor,m_comps);
shape_aspect * StixComponentCursor::next() 
{
    StixComponent * idx = (StixComponent *)
  	((m_nc && (m_i<m_nc->m_comps.size()))?
  	 (m_nc->m_comps[m_i++]): 0);

    return (idx? idx-> m_sa: 0);
}


IMPLEMENT_STIXNC_CURSOR(StixExecutableCursor,m_execs);
machining_process_executable * StixExecutableCursor::next() 
{
    return (machining_process_executable *)
  	((m_nc && (m_i<m_nc->m_execs.size()))?
  	 (m_nc->m_execs[m_i++]): 0);
}


IMPLEMENT_STIXNC_CURSOR(StixToolCursor,m_tools);
machining_tool * StixToolCursor::next() 
{
    return (machining_tool *)
  	((m_nc && (m_i<m_nc->m_tools.size()))?
  	 (m_nc->m_tools[m_i++]): 0);
}




/* ========================================
 * Feature Boss Cursor - This traverses over the bosses associated
 * with certain types of features.
 */

StixFeatureBossCursor::StixFeatureBossCursor(
    shape_aspect * sa
    )
{
    traverse(sa);
}

void StixFeatureBossCursor::traverse(
    shape_aspect * sa
    )
{
    m_idx = get_feature_idx(sa);
    m_i = 0;    /* reset traversal to beginning */
}

void StixFeatureBossCursor::rewind()
{
    m_i = 0;    /* reset traversal to beginning */
}

unsigned StixFeatureBossCursor::size()
{
    unsigned orig_pos = m_i;
    unsigned sz=0;

    /* This is not for loop guards.  Use null return from next() to
     * signal no more objects.  This computes the size, it does not
     * just return a stored value. 
     */
    rewind();
    while (next()) sz++;
    m_i = orig_pos;  /* restore */

    return sz;
}


shape_aspect * StixFeatureBossCursor::next()
{
#ifdef BUG_BOSSES_RELATED_INCORRECTLY
    unsigned sz;
    shape_aspect * sa;

    /* Bosses are being written out incorrectly.  The ED side refers
     * directly to the feature rather than to an intermediate, and the
     * SAR does not have the required text.
     */
    if (m_idx) for (sz=m_idx->m_direct_sars.size(); m_i<sz;) 
    {
	shape_aspect_relationship * sar = 
	    (shape_aspect_relationship *) m_idx->m_direct_sars[m_i++];

	/* test by type */
	sa = sar-> relating_shape_aspect();
	if (sa-> isa(ROSE_TYPE(boss))) return sa;
    }
    return 0;
#else
    unsigned sz;
    char * s;

    /* Components are attached to the feature on the ING side of the
     * relationship.  On the ED side is an intermediary shape aspect
     * that refers to the feature shape aspect.
     * 
     * Often we can find the component by just looking at the desc of
     * the relationship, but sometimes we must look at the desc of the
     * intermediate shape aspect as well.  Usually this string has the
     * suffix "occurrence" tacked on.
     */

    if (m_idx) for (sz=m_idx->m_component_sars.size(); m_i<sz;) 
    {
	shape_aspect_relationship * sar = 
	    (shape_aspect_relationship *) m_idx->m_component_sars[m_i++];

	/* this should be a COMPOSITE_SHAPE_ASPECT */
	shape_aspect * ed = sar-> related_shape_aspect();

	s = sar-> description();
	if (!s || strcmp ("uncut area", s))
	    continue;

  	/* made it through, everything matched */
	return sar-> relating_shape_aspect();
    }
    return 0;
#endif
}



/* ========================================
 * Workplan Element Cursor - This traverses over the bosses
 * associated with certain types of features.  For convenience, this
 * has a traverse() method to switch the workplan that it looks at.
 * This can traverse using the next() function but since ordering is
 * particularly important, this also has a get() by index function.
 * 
 * When update capability is added, this will also house the insert,
 * append, remove, and other update functions.
 */
StixWorkplanElementCursor::StixWorkplanElementCursor(
    machining_process_executable * wp
    )
{
    traverse(wp);
}

void StixWorkplanElementCursor::traverse(
    machining_process_executable * mpe
    )
{
    if (!mpe || !mpe-> isa (ROSE_DOMAIN(machining_workplan)))
	m_wp = 0;
    else
	m_wp = ROSE_CAST(machining_workplan,mpe);

    m_i = 0;    /* reset traversal to beginning */
}

void StixWorkplanElementCursor::rewind()
{
    m_i = 0;    /* reset traversal to beginning */
}

unsigned StixWorkplanElementCursor::size()
{
    return (m_wp? m_wp->m_steps.size(): 0);
}

/* returns null when no more objects */
machining_process_executable * 
StixWorkplanElementCursor::next()
{
    return get (m_i++);
}

/* returns null if out of bounds */
machining_process_executable * 
StixWorkplanElementCursor::get (unsigned i)
{
    sequential_method * sm = (m_wp && i<m_wp->m_steps.size())? 
	((sequential_method *) m_wp->m_steps[i]) : 0;

    action_method * ed = (sm? sm->related_method(): 0);

    /* test probably not necessary, but just to be safe */
    return (ed && ed-> isa(ROSE_DOMAIN(machining_process_executable)))?
	(ROSE_CAST(machining_process_executable,ed)): 0;
}

double
StixWorkplanElementCursor::get_sequence_position (unsigned i)
{
    sequential_method * sm = (m_wp && i<m_wp->m_steps.size())? 
	((sequential_method *) m_wp->m_steps[i]) : 0;

    return (sm? sm-> sequence_position(): 0);
}




/* ========================================
 * Workplan Nested Steps Cursor - This does a depth first search down
 * through the workplan elements and any nested workplans.  It returns
 * only workingsteps.  THIS IS A COMPUTED VALUE.
 *
 * Has a traverse() method to switch the workplan that it looks at.
 * This can traverse using next() or get() by index so you can access
 * sequentially or randomly. 
 */
StixWorkplanAllNestedCursor::StixWorkplanAllNestedCursor(
    machining_process_executable * wp
    )
{
    traverse(wp);
}

void StixWorkplanAllNestedCursor::traverse(
    machining_process_executable * mpe
    )
{
    m_wp = 0;
    m_i  = 0;    /* reset traversal to beginning */

    if (mpe && mpe-> isa (ROSE_DOMAIN(machining_workplan))) {
	m_wp = ROSE_CAST(machining_workplan,mpe);
	if (!m_wp->m_calc_all_steps)
	    /* build DFS list of all steps first */ 
	    calc_exec_all_worksteps (m_wp);
    }
}

void StixWorkplanAllNestedCursor::rewind()
{
    m_i = 0;    /* reset traversal to beginning */
}

unsigned StixWorkplanAllNestedCursor::size()
{
    return (m_wp? m_wp->m_calc_all_steps->size(): 0);
}

/* returns null when no more objects */
machining_workingstep *
StixWorkplanAllNestedCursor::next()
{
    return get (m_i++);
}

machining_workingstep *
StixWorkplanAllNestedCursor::get (unsigned i)
{
    /* returns null if out of bounds */
    return (m_wp && i<m_wp->m_calc_all_steps->size())? 
	((machining_workingstep *) m_wp->m_calc_all_steps->get(i)) : 0;
}




/* ========================================
 * Workplan Unique Feature Cursor - This examines each workingstep in the
 * given worplan and any nested workplans.  It computes the set of unique
 * features associated with those steps.  THIS IS A COMPUTED VALUE.
 *
 * Has a traverse() method to switch the workplan that it looks at.
 * This can traverse using next() or get() by index so you can access
 * sequentially or randomly. 
 *
 * If this is passed a workingstep rather than a workplan, this cursor 
 * will do the right thing and return the one feature referenced by it.
 */
StixWorkplanAllFeatureCursor::StixWorkplanAllFeatureCursor(
    machining_process_executable * wp
    )
{
    traverse(wp);
}

void StixWorkplanAllFeatureCursor::traverse(
    machining_process_executable * mpe
    )
{
    m_wp = 0;
    m_ws = 0;
    m_i  = 0;    /* reset traversal to beginning */

    if (!mpe) return;

    if (mpe-> isa (ROSE_DOMAIN(machining_workplan))) {
	m_wp = ROSE_CAST(machining_workplan,mpe);
	if (!m_wp->m_calc_unique_feats)
	    calc_exec_unique_features (m_wp);
    }
    else if (mpe-> isa (ROSE_DOMAIN(machining_workingstep))) {
	m_ws = ROSE_CAST(machining_workingstep,mpe);
    }
}

void StixWorkplanAllFeatureCursor::rewind()
{
    m_i = 0;    /* reset traversal to beginning */
}

unsigned StixWorkplanAllFeatureCursor::size()
{
    if (m_wp) return (m_wp->m_calc_unique_feats->size());
    if (m_ws) return (get_workingstep_feature (m_ws)? 1: 0);
    return 0;
}

/* returns null when no more objects */
shape_aspect *
StixWorkplanAllFeatureCursor::next()
{
    return get (m_i++);
}

shape_aspect *
StixWorkplanAllFeatureCursor::get (unsigned i)
{
    /* returns null if out of bounds, this returns the in-process
     * feature if it is defined */
    if (m_wp) 
	return (i<m_wp->m_calc_unique_feats->size())? 
	    ((shape_aspect *) m_wp->m_calc_unique_feats->get(i)) : 0;

    if (m_ws) 
	return (i==0)? get_workingstep_feature (m_ws): 0;

    return 0;
}



/* ========================================
 * Workplan Unique Tool Cursor - This examines each workingstep in the
 * given worplan and any nested workplans.  It computes the set of unique
 * tools associated with those steps.  THIS IS A COMPUTED VALUE.
 *
 * Has a traverse() method to switch the workplan that it looks at.
 * This can traverse using next() or get() by index so you can access
 * sequentially or randomly. 
 *
 * If this is passed a workingstep rather than a workplan, this cursor 
 * will do the right thing and return the one tool referenced by it.
 */
StixWorkplanAllToolCursor::StixWorkplanAllToolCursor(
    machining_process_executable * wp
    )
{
    traverse(wp);
}

void StixWorkplanAllToolCursor::traverse(
    machining_process_executable * mpe
    )
{
    m_wp = 0;
    m_ws = 0;
    m_i  = 0;    /* reset traversal to beginning */

    if (!mpe) return;

    if (mpe-> isa (ROSE_DOMAIN(machining_workplan))) {
	m_wp = ROSE_CAST(machining_workplan,mpe);
	if (!m_wp->m_calc_unique_tools)
	    calc_exec_unique_tools (m_wp);
    }
    else if (mpe-> isa (ROSE_DOMAIN(machining_workingstep))) {
	m_ws = ROSE_CAST(machining_workingstep,mpe);
    }
}

void StixWorkplanAllToolCursor::rewind()
{
    m_i = 0;    /* reset traversal to beginning */
}

unsigned StixWorkplanAllToolCursor::size()
{
    if (m_wp) return (m_wp->m_calc_unique_tools->size());
    if (m_ws) return (m_ws-> m_op && m_ws-> m_op-> m_tool)? 1: 0;
    return 0;
}

machining_tool * 
StixWorkplanAllToolCursor::next()
{
    /* returns null when no more objects */
    return get (m_i++);
}

machining_tool * 
StixWorkplanAllToolCursor::get (unsigned i)
{
    /* returns null if out of bounds, this returns the in-process
     * feature if it is defined */
    if (m_wp) 
	return (i<m_wp->m_calc_unique_tools->size())? 
	    ((machining_tool *) m_wp->m_calc_unique_tools->get(i)) : 0;

    if (m_ws) 
	return (i==0 && m_ws-> m_op)? m_ws-> m_op-> m_tool: 0;

    return 0;
}




/* ========================================
 * Feature Face Cursor - This returns the advanced faces associated
 * with a particular feature.  If it is given an in-process feature,
 * it will return faces for the associated final feature instead, but
 * if none are found, it will return the faces for the in-process one.
 *
 * Has a traverse() method to switch the feature that it looks at.
 * This can traverse using next() or get() by index so you can access
 * sequentially or randomly.  The get() is not as efficient, so avoid
 * it if you can.
 */

StixFeatureFaceCursor::StixFeatureFaceCursor(
    shape_aspect * sa
    )
{
    traverse(sa);
}

void StixFeatureFaceCursor::traverse(
    shape_aspect * sa
    )
{
    unsigned 	   typ   = get_aspect_type(sa);
    shape_aspect * final = get_feature_final(sa); 
    shape_representation * rep;

    m_sa = sa;	/* not really used, just nice to have */
    m_its = 0;
    m_i  = 0;   /* reset traversal to beginning */

    if (!sa || !typ) return;

    /* append the faces for this feature to the list */
    if (typ == FEAT_EDGE_ROUND) {
	/* look for a face shape representation */
	rep = get_edge_round_face_shape (final);
	if (!rep) rep = get_edge_round_face_shape (sa);
    }
    else {
	StixFeature * idx = get_feature_idx (final);
	rep = (idx? idx-> m_explicit: 0);
	
	if (!rep) {
	    idx = get_feature_idx (sa);
	    rep = (idx? idx-> m_explicit: 0);
	}
    }

    m_its = (rep? rep-> items(): 0);
}

void StixFeatureFaceCursor::rewind()
{
    m_i = 0;    /* reset traversal to beginning */
}

unsigned StixFeatureFaceCursor::size()
{
    unsigned orig_pos = m_i;
    unsigned sz=0;

    /* This is not for loop guards.  Use null return from next() to
     * signal no more objects.  This computes the size, it does not
     * just return a stored value. 
     */
    rewind();
    while (next()) sz++;
    m_i = orig_pos;  /* restore */

    return sz;
}

advanced_face *
StixFeatureFaceCursor::next()
{
    if (m_its) for (unsigned sz=m_its->size(); m_i<sz;) 
    {
	representation_item * it = m_its-> get(m_i++);
	if (it-> isa(ROSE_DOMAIN(advanced_face)))
	    return ROSE_CAST(advanced_face,it);
    }
    return 0;
}

advanced_face *
StixFeatureFaceCursor::get (unsigned i)
{
    unsigned orig_pos = m_i;
    unsigned cnt = 0;
    advanced_face * obj;

    /* This is not for loops.  Use next() instead. */
    rewind();
    while ((obj=next()) && cnt++<i);
    m_i = orig_pos;  /* restore */

    return obj;
}

