/* $RCSfile: StixNC.h,v $
 * $Revision: 1.21 $ $Date: 2003/08/05 18:11:41 $
 * Auth: Dave Loffredo (loffredo@steptools.com)
 * 
 * Copyright (c) 1991-2003 by STEP Tools Inc. 
 * All Rights Reserved.
 * 
 * This file is part of the STIX Library
 * 
 * This file may be distributed and/or modified under the terms of 
 * the GNU General Public License version 2 as published by the Free
 * Software Foundation and appearing in the file LICENSE.GPL included
 * with this file.
 * 
 * THIS FILE IS PROVIDED "AS IS" WITH NO WARRANTY OF ANY KIND,
 * INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * STEP Tools Inc. also offers licenses for commercial use of this
 * software in non-GPL applications.  Contact info@steptools.com for
 * terms and pricing.
 * 
 * 		----------------------------------------
 *
 * STEP-NC Interface Class.  This class manages the import/export
 * of data as well as all of the indexing neccessary to provide a
 * simplified view of the AP-238 data.
 */

#include <roseHdefs.h>

class StixNC {
protected:
    int 		m_verbose;
    RoseDesign * 	m_design;

    /* Keep these arrays for quick access to the major items and
     * also to provide a safe external identifier for them.  The 
     * external identifier is only used to coordinate with the COM 
     * interface.  The ID will be the index into the vector with a
     * type-id ORed into the high bits.
     */
    rose_vector m_wkpcs;	/* index workpieces */
    rose_vector m_projects;	/* index projects */
    rose_vector m_setups;	/* index setups */
    rose_vector m_features;	/* index features */
    rose_vector m_comps;	/* index feature components */
    rose_vector m_execs;	/* index workingsteps */
    rose_vector m_tools;	/* index machining tools */

    void build_index();
    void classify_products();
    void classify_aspects();
    void classify_actions();
    void classify_tolerances();

    void link_property_reps();
    void link_action_reps();
    void link_resource_reps();
    void link_amrs();
    void link_pdrs();
    void link_sars();
    void link_direct_sar (shape_aspect_relationship *);
    void link_feature_sar (shape_aspect_relationship *);
    void link_feature_final (machining_workingstep * ws);
    void link_process_assocs();
    void link_materials();

    void workpiece_calc_extra();
    void workstep_calc_extra();
    void feature_calc_extra();

#if 0
    void link_feature_hole();
    void link_feature_pocket();
    void link_feature_plane();
#endif

public:
    friend class StixWorkpieceCursor;
    friend class StixProjectCursor;
    friend class StixSetupCursor;
    friend class StixFeatureCursor;
    friend class StixComponentCursor;
    friend class StixExecutableCursor;
    friend class StixToolCursor;

    StixNC();
    ~StixNC();

    /* public com methods */
    int load (const char * filename);
    void reset();

    product * 		default_project();
    product_definition * default_workpiece();

    /* have different functions for casting and typing */
    StixProject *   	exid_to_project_idx (unsigned exid);
    StixSetup *     	exid_to_setup_idx (unsigned exid);
    StixWorkpiece * 	exid_to_workpiece_idx (unsigned exid);
    StixFeature *   	exid_to_feature_idx (unsigned exid);
    StixComponent * 	exid_to_component_idx (unsigned exid);

    product * 		exid_to_project (unsigned exid);
    product * 		exid_to_setup (unsigned exid);
    product_definition * exid_to_workpiece (unsigned exid);
    shape_aspect * 	exid_to_feature (unsigned exid);
    shape_aspect * 	exid_to_component (unsigned exid);

    machining_process_executable * exid_to_workstep (unsigned exid);
    machining_tool * 	exid_to_tool (unsigned exid);

    RoseDesign * 	design() 	{ return m_design; }

    StixSchemaType 	src_schema();
    char *	   	src_schema_name();
};

