/* $RCSfile: component.cxx,v $
 * $Revision: 1.13 $ $Date: 2003/05/30 14:49:57 $
 * Auth: Dave Loffredo (loffredo@steptools.com)
 * 
 * Copyright (c) 1991-2003 by STEP Tools Inc. 
 * All Rights Reserved.
 * 
 * This file is part of the STIX Library
 * 
 * This file may be distributed and/or modified under the terms of 
 * the GNU General Public License version 2 as published by the Free
 * Software Foundation and appearing in the file LICENSE.GPL included
 * with this file.
 * 
 * THIS FILE IS PROVIDED "AS IS" WITH NO WARRANTY OF ANY KIND,
 * INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * STEP Tools Inc. also offers licenses for commercial use of this
 * software in non-GPL applications.  Contact info@steptools.com for
 * terms and pricing.
 * 
 * 		----------------------------------------
 */


#include <rose.h>
#include <math.h>

#include <integrated_cnc_schema.h>
#include "stix_imp.h"


/* ----------------------------------------
 * Path Functions
 */

axis2_placement_3d * get_path_placement (shape_aspect * sa)
{
    StixComponent * idx = get_component_idx(sa);
    return (idx? get_placement (idx-> m_implicit, "orientation"): 0);
}

double get_path_linear_distance (
    shape_aspect * sa, StixUnit u
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (PATH_LINEAR != sa->m_type) return 0;

    return get_length_value (idx-> m_implicit, "distance", u);
}


direction * get_path_linear_direction (shape_aspect * sa)
{
    if (!sa || (PATH_LINEAR != sa->m_type)) return 0;
    return get_direction (get_direction_shape_property(sa));
}



#ifdef BUG_PATH_SHAPE_REP_NOT_USED
shape_representation * 
#else
path_shape_representation * 
#endif
get_path_general_shape (
    shape_aspect * sa 	
    )
{
    if (!sa || (PATH_GENERAL != sa->m_type)) return 0;
    return get_path_shape_property (sa, "sweep path");
}



double get_path_circle_radius (
    shape_aspect * sa, StixUnit u
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if ((PATH_OPEN_CIRC != sa->m_type) &&
	(PATH_CLOSED_CIRC != sa->m_type)) return 0;

    return get_length_value (idx-> m_implicit, "radius", u);
}

/* partial circular path only */
double get_path_circle_sweep_angle (
    shape_aspect * sa, StixUnit u
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (PATH_OPEN_CIRC != sa->m_type) return 0;

    return get_angle_value (idx-> m_implicit, "sweep angle", u);
}





/* ----------------------------------------
 * Taper Functions
 */

double get_taper_angle (shape_aspect * sa, StixUnit u)
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (TAPER_ANGLE != sa->m_type) return 0;

    return get_angle_value (idx-> m_implicit, "taper angle", u);
}



double get_taper_final_diameter (shape_aspect * sa, StixUnit u)
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (TAPER_DIAMETER != sa->m_type) return 0;

    return get_length_value (idx-> m_implicit, "final diameter", u);
}



/* ----------------------------------------
 * Profile Functions
 */

axis2_placement_3d * get_profile_placement (shape_aspect * sa)
{
    axis2_placement_3d * ap;
    StixComponent * pidx  = get_component_idx (sa);

    if (!pidx) return 0;
    if (!IS_PROFILE(sa)) return 0;

    ap = get_placement (pidx-> m_implicit, "orientation");
    return (ap? ap: get_placement (pidx-> m_explicit));
}


double get_profile_circle_diameter (shape_aspect * sa, StixUnit u)
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (PROF_CLOSED_CIRC != sa->m_type) return 0;

    return get_length_value (idx-> m_implicit, "diameter", u);
}




double get_profile_rectangular_width (shape_aspect * sa, StixUnit u)
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (PROF_CLOSED_RECT != sa->m_type) return 0;

    return get_length_value (idx-> m_implicit, "width", u);
}

double get_profile_rectangular_length (shape_aspect * sa, StixUnit u)
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (PROF_CLOSED_RECT != sa->m_type) return 0;

    return get_length_value (idx-> m_implicit, "length", u);
}

double get_profile_rectangular_corner_radius (shape_aspect * sa, StixUnit u)
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (PROF_CLOSED_RECT != sa->m_type) return 0;

    // This is an AP-224 thing, but does not appear in 14649
    return get_length_value (idx-> m_implicit, "corner radius", u);
}


double get_profile_ngon_diameter (
    shape_aspect * sa,  StixUnit u
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (PROF_CLOSED_NGON != sa->m_type) return 0;
    
    /* the length measure is either 
     * representation_item.name = 'circumscribed diameter'
     * representation_item.name = 'diameter across flats'
     * depending on which is used, the meaning of the measured
     * diameter changes.
     */

    measure_with_unit *cd, *fd;

    cd = get_length_rep_item (idx-> m_implicit, "circumscribed diameter");
    fd = get_length_rep_item (idx-> m_implicit, "diameter across flats");

    if (cd && fd) {
	fprintf (stdout, "WARN: ngon #%d has duplicate diameters\n",
		 sa-> entity_id());
    }
    
    if (cd) return get_length_value (cd, u);
    if (fd) return get_length_value (fd, u);
    return 0;
}

int get_profile_ngon_circumscribed_or_flats (
    shape_aspect * sa
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (PROF_CLOSED_NGON != sa->m_type) return 0;
    
    /* the length measure is either 
     * representation_item.name = 'circumscribed diameter'
     * representation_item.name = 'diameter across flats'
     * depending on which is used, the meaning of the measured
     * diameter changes.
     */

    /* true=flats false=circumscribed */

    measure_with_unit *fd;
    fd = get_length_rep_item (idx-> m_implicit, "diameter across flats");

    return (fd != 0);
}


int get_profile_ngon_sides (
    shape_aspect * sa
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (PROF_CLOSED_NGON != sa->m_type) return 0;

    return (int) get_count_value (idx-> m_implicit, "number of sides");
}



#ifdef BUG_PATH_SHAPE_REP_NOT_USED
shape_representation * 
#else
path_shape_representation * 
#endif
get_profile_general_shape (
    shape_aspect * sa 	
    )
{
    if (!sa) return 0;
    if (PROF_OPEN_GENERAL   != sa->m_type &&
	PROF_CLOSED_GENERAL != sa->m_type) return 0;

    return get_path_shape_property (sa);
}





double get_profile_linear_length (
    shape_aspect * sa, StixUnit u
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (PROF_LINEAR != sa->m_type) return 0;

    return get_length_value (idx-> m_implicit, "profile length", u);
}




double get_profile_arc_radius (
    shape_aspect * sa, StixUnit u
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (PROF_OPEN_CIRC != sa->m_type) return 0;

    return get_length_value (idx-> m_implicit, "radius", u);
}

double get_profile_arc_sweep_angle (
    shape_aspect * sa, StixUnit u
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (PROF_OPEN_CIRC != sa->m_type) return 0;

    return get_angle_value (idx-> m_implicit, "sweep angle", u);
}




double get_profile_square_u_width (
    shape_aspect * sa, StixUnit u
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (PROF_SQUARE_U != sa->m_type) return 0;

    return get_length_value (idx-> m_implicit, "width", u);
}

double get_profile_square_u_first_radius (
    shape_aspect * sa, StixUnit u
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (PROF_SQUARE_U != sa->m_type) return 0;

    return get_length_value (idx-> m_implicit, "first radius", u);
}

double get_profile_square_u_first_angle (
    shape_aspect * sa, StixUnit u
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (PROF_SQUARE_U != sa->m_type) return 0;

    return get_angle_value (idx-> m_implicit, "first angle", u);
}

double get_profile_square_u_second_radius (
    shape_aspect * sa, StixUnit u
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (PROF_SQUARE_U != sa->m_type) return 0;

    return get_length_value (idx-> m_implicit, "second radius", u);
}

double get_profile_square_u_second_angle (
    shape_aspect * sa, StixUnit u
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (PROF_SQUARE_U != sa->m_type) return 0;

    return get_angle_value (idx-> m_implicit, "second angle", u);
}




double get_profile_rounded_u_width (
    shape_aspect * sa, StixUnit u
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (PROF_ROUNDED_U != sa->m_type) return 0;

    return get_length_value (idx-> m_implicit, "width", u);
}




double get_profile_tee_first_angle (
    shape_aspect * sa, StixUnit u
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (PROF_TEE != sa->m_type) return 0;

    return get_angle_value (idx-> m_implicit, "first angle", u);
}

double get_profile_tee_second_angle (
    shape_aspect * sa, StixUnit u
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (PROF_TEE != sa->m_type) return 0;

    return get_angle_value (idx-> m_implicit, "second angle", u);
}

double get_profile_tee_cross_bar_width (
    shape_aspect * sa, StixUnit u
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (PROF_TEE != sa->m_type) return 0;

    return get_length_value (idx-> m_implicit, "cross bar width", u);
}

double get_profile_tee_cross_bar_depth (
    shape_aspect * sa, StixUnit u
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (PROF_TEE != sa->m_type) return 0;

    return get_length_value (idx-> m_implicit, "cross bar depth", u);
}

double get_profile_tee_radius (
    shape_aspect * sa, StixUnit u
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (PROF_TEE != sa->m_type) return 0;

    return get_length_value (idx-> m_implicit, "radius", u);
}

double get_profile_tee_width (
    shape_aspect * sa, StixUnit u
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (PROF_TEE != sa->m_type) return 0;

    return get_length_value (idx-> m_implicit, "width", u);
}

double get_profile_tee_first_offset (
    shape_aspect * sa, StixUnit u
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (PROF_TEE != sa->m_type) return 0;

    return get_length_value (idx-> m_implicit, "first offset", u);
}

double get_profile_tee_second_offset (
    shape_aspect * sa, StixUnit u
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (PROF_TEE != sa->m_type) return 0;

    return get_length_value (idx-> m_implicit, "second offset", u);
}




double get_profile_vee_radius (
    shape_aspect * sa, StixUnit u
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (PROF_VEE != sa->m_type) return 0;

    return get_length_value (idx-> m_implicit, "profile radius", u);
}

double get_profile_vee_angle (
    shape_aspect * sa, StixUnit u
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (PROF_VEE != sa->m_type) return 0;

    return get_angle_value (idx-> m_implicit, "profile angle", u);
}


double get_profile_vee_tilt (
    shape_aspect * sa, StixUnit u
    )
{
    /* in the mappings, this is locked to 90 because we 
     * transform the orientation.  Not sure if this should
     * continue, so return the actual value for now.
     */

    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (PROF_VEE != sa->m_type) return 0;

    return get_angle_value (idx-> m_implicit, "tilt angle", u);
}



/* ----------------------------------------
 * Pocket Bottom Functions
 */
double get_pocketbottom_radius (
    shape_aspect * sa, StixUnit u
    )
{
    // floor radius with general condition
    // floor radius with planar pocket bottom 
    // not with through bottom
    //
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (!IS_POCKET_BOTTOM(sa)) return 0;

    return get_length_value (idx-> m_implicit, "radius", u);
}

double get_pocketbottom_radiused_bottom_radius (
    shape_aspect * sa, StixUnit u
    )
{
    // This does not exist in AP-224
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (PBOT_RADIUSED != sa->m_type) return 0;

    return get_length_value (idx-> m_implicit, "pocket bottom radius", u);
}


face_shape_representation * get_pocketbottom_general_face (
    shape_aspect * sa
    )
{
    if (!sa || (PBOT_GENERAL != sa->m_type)) return 0;
    return get_face_shape_property (sa, "floor face");
}





/* ----------------------------------------
 * Hole Bottom Functions
 */
double get_holebottom_conical_tip_angle (
    shape_aspect * sa, StixUnit u
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (HBOT_CONICAL != sa->m_type) return 0;

    return get_angle_value (idx-> m_implicit, "tip angle", u);
}

double get_holebottom_conical_tip_radius (
    shape_aspect * sa, StixUnit u
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (HBOT_CONICAL != sa->m_type) return 0;

    return get_length_value (idx-> m_implicit, "tip radius", u);
}




/* ----------------------------------------
 * Slot End Functions
 */
double get_slotend_flat_first_radius (
    shape_aspect * sa, StixUnit u
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (SLOTEND_FLAT != sa->m_type) return 0;

    return get_length_value (idx-> m_implicit, "first radius", u);
}


double get_slotend_flat_second_radius (
    shape_aspect * sa, StixUnit u
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (SLOTEND_FLAT != sa->m_type) return 0;

    return get_length_value (idx-> m_implicit, "second radius", u);
}

double get_slotend_woodruff_radius (
    shape_aspect * sa, StixUnit u
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (SLOTEND_WOODRUFF != sa->m_type) return 0;

    return get_length_value (idx-> m_implicit, "radius", u);
}




/* ----------------------------------------
 * Hole Bottom Functions
 */


/* profile floor conditions */
double get_profloor_radius (
    shape_aspect * sa, StixUnit u
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (!IS_PROFILE_FLOOR(sa)) return 0;

    return get_length_value (idx-> m_implicit, "radius", u);
}


/* profile floor conditions */
int get_profloor_start_or_end (
    shape_aspect * sa
    )
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (!IS_PROFILE_FLOOR(sa)) return 0;

    /* from the 14649 document: if true, the profile_floor is
     * positioned at the end of a shape_profile. If false, it 
     * is at the start of the shape_profile.
     */
    return get_boolean_value (
	"shape profile end", 
	"shape profile start",
	idx-> m_implicit, 
	"shape profile floor orientation"
	);
}

planar_shape_representation * get_profloor_planar_face (	
    shape_aspect * sa
    )
{
    if (!sa || (PROFLOOR_PLANAR != sa->m_type)) return 0;
    return get_planar_shape_property (sa, "floor");
}

face_shape_representation * get_profloor_general_face (	
    shape_aspect * sa
    )
{
    if (!sa || (PROFLOOR_GENERAL != sa->m_type)) return 0;
    return get_face_shape_property (sa, "floor");
}
