# $RCSfile: integrated_cnc.ws,v $
# $Revision: 1.4 $ $Date: 2003/08/12 22:18:21 $
# Auth: Dave Loffredo (loffredo@steptools.com)
# 
# AP-238 WORKING SET FILE
#
# This working set file describes the complex entities needed for
# working with AP-238.  There may be others that are not included
# here.  Extend this file as needed.
#
# To see how to use working sets with the EXPRESS compiler, consult
# the EXPRESS compiler section of the STEP Utilities reference manual.
# This file is specified using the -ws option to the compiler
#
# 	% expfront -ws workset.ws schema.exp
#

#
# Unit Descriptions
#
ANDOR( si_unit	length_unit )
ANDOR( si_unit 	plane_angle_unit )
ANDOR( si_unit 	solid_angle_unit )
ANDOR( si_unit 	ratio_unit )
ANDOR( si_unit 	time_unit )

# no conversion based solid angle, ratio or time
ANDOR( conversion_based_unit 	length_unit )
ANDOR( conversion_based_unit 	plane_angle_unit )

#
# Measure Descriptions -- The length and plane measures may also be
# combined with a qualified rep item, to handle plus minus ranges or
# precisions.  Qualified lengths are widely used but qualified angles
# may also appear in data imported from AP-224.
#
ANDOR( 	measure_representation_item length_measure_with_unit )
ANDOR( 	measure_representation_item length_measure_with_unit \
	qualified_representation_item )

ANDOR( 	measure_representation_item plane_angle_measure_with_unit )
ANDOR( 	measure_representation_item plane_angle_measure_with_unit \
	qualified_representation_item )

ANDOR( 	measure_representation_item ratio_measure_with_unit )
ANDOR( 	measure_representation_item time_measure_with_unit )


# TEMPORARY.  Should really use one of the subtypes
ANDOR( 	measure_representation_item qualified_representation_item )


#
# Contexts and relationships
#
ANDOR(	geometric_representation_context \
	global_uncertainty_assigned_context \
	global_unit_assigned_context )

ANDOR(	geometric_representation_context \
	global_unit_assigned_context )

# for using pcurves w/breps
ANDOR(	geometric_representation_context \
	parametric_representation_context )


# Needed for using pcurves w/breps
ANDOR(	definitional_representation \
	shape_representation )

ANDOR(	representation_relationship_with_transformation \
	shape_representation_relationship )

#  
# Spline curves and surfaces (NURBS)
#
ANDOR(	rational_b_spline_curve		b_spline_curve_with_knots )
ANDOR(	rational_b_spline_curve		bezier_curve )
ANDOR(	rational_b_spline_curve		quasi_uniform_curve )
ANDOR(	rational_b_spline_curve 	uniform_curve )

ANDOR(	rational_b_spline_surface	b_spline_surface_with_knots )
ANDOR(	rational_b_spline_surface	bezier_surface )
ANDOR(	rational_b_spline_surface	quasi_uniform_surface )
ANDOR(	rational_b_spline_surface 	uniform_surface )

ANDOR(	bounded_curve 	surface_curve )


#
# Features -- Everything except transition features are combined with
# instanced_feature.  Instanced feature is a subtype of shape_aspect,
# which gives us the means to hook the feature to a shape.
#
# Note that replicate feature is never created directly, so it is not
# here, but all of the subtypes are.  Turning features will eventually
# be added to this list.
# 
#
ANDOR( instanced_feature boss ) 
ANDOR( instanced_feature circular_pattern ) 
ANDOR( instanced_feature composite_hole ) 
ANDOR( instanced_feature compound_feature ) 
ANDOR( instanced_feature externally_defined_feature_definition )
ANDOR( instanced_feature feature_pattern ) 
ANDOR( instanced_feature flat_face ) 
ANDOR( instanced_feature outside_profile ) 
ANDOR( instanced_feature pocket ) 
ANDOR( instanced_feature rectangular_pattern )
ANDOR( instanced_feature removal_volume ) 
ANDOR( instanced_feature round_hole )
ANDOR( instanced_feature rounded_end ) 
ANDOR( instanced_feature slot ) 
ANDOR( instanced_feature spherical_cap )
ANDOR( instanced_feature step ) 
ANDOR( instanced_feature thread ) 

#
# Expression Representation Items -- These are all of the different
# types of expressions that can appear in the condition property for
# the different stepnc executable control flow things.
#
# This is similar to the way that measures are treated by andor
# combining the different length, angle, etc types with measure
# representation item.
# 
# DISABLED FOR NOW, since we don't do funky control flow
#
# ANDOR( expression_representation_item	comparison_equal )
# ANDOR( expression_representation_item	comparison_not_equal )
# ANDOR( expression_representation_item	comparison_greater )
# ANDOR( expression_representation_item	comparison_greater_equal )
# ANDOR( expression_representation_item	comparison_less )
# ANDOR( expression_representation_item	comparison_less_equal )
# ANDOR( expression_representation_item	and_expression )
# ANDOR( expression_representation_item	not_expression )
# ANDOR( expression_representation_item	or_expression )
# ANDOR( expression_representation_item	xor_expression )

# Will instances of literal_number or numeric_variable ever be
# directly created, or will it always be one of these subtypes.
#
# ANDOR( expression_representation_item	int_literal )
# ANDOR( expression_representation_item	int_numeric_variable )
# ANDOR( expression_representation_item	real_literal )
# ANDOR( expression_representation_item	real_numeric_variable )


# Tolerances -- any of the geometric tolerances can be and/ored
# modified geometric tolerance to add some conditions to things.
# geometric_tolerance_with_defined_unit does not seem to be used
# in any complex combinations.

ANDOR( modified_geometric_tolerance geometric_tolerance_with_datum_reference )
ANDOR( modified_geometric_tolerance angularity_tolerance )
ANDOR( modified_geometric_tolerance coaxiality_tolerance )
ANDOR( modified_geometric_tolerance concentricity_tolerance )
ANDOR( modified_geometric_tolerance cylindricity_tolerance )
ANDOR( modified_geometric_tolerance flatness_tolerance )
ANDOR( modified_geometric_tolerance line_profile_tolerance )
ANDOR( modified_geometric_tolerance parallelism_tolerance )
ANDOR( modified_geometric_tolerance perpendicularity_tolerance )
ANDOR( modified_geometric_tolerance position_tolerance )
ANDOR( modified_geometric_tolerance roundness_tolerance )
ANDOR( modified_geometric_tolerance straightness_tolerance )
ANDOR( modified_geometric_tolerance surface_profile_tolerance )
ANDOR( modified_geometric_tolerance symmetry_tolerance )
ANDOR( modified_geometric_tolerance total_runout_tolerance )

# The position, line profile, and surface profile tolerances can also
# be associated with a datum reference, so we need a complex instance
# that includes that GTWDR as well.  This is a bit unwieldy, but that
# is the way it was modeled.

ANDOR(  geometric_tolerance_with_datum_reference line_profile_tolerance )
ANDOR(  geometric_tolerance_with_datum_reference line_profile_tolerance \
	modified_geometric_tolerance )

ANDOR(  geometric_tolerance_with_datum_reference position_tolerance )
ANDOR(  geometric_tolerance_with_datum_reference position_tolerance \
	modified_geometric_tolerance )

ANDOR(  geometric_tolerance_with_datum_reference surface_profile_tolerance )
ANDOR(  geometric_tolerance_with_datum_reference surface_profile_tolerance \
	modified_geometric_tolerance )

