/* $RCSfile: feature.cxx,v $
 * $Revision: 1.19 $ $Date: 2003/05/30 14:49:57 $
 * Auth: Dave Loffredo (loffredo@steptools.com)
 * 
 * Copyright (c) 1991-2003 by STEP Tools Inc. 
 * All Rights Reserved.
 * 
 * This file is part of the STIX Library
 * 
 * This file may be distributed and/or modified under the terms of 
 * the GNU General Public License version 2 as published by the Free
 * Software Foundation and appearing in the file LICENSE.GPL included
 * with this file.
 * 
 * THIS FILE IS PROVIDED "AS IS" WITH NO WARRANTY OF ANY KIND,
 * INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * STEP Tools Inc. also offers licenses for commercial use of this
 * software in non-GPL applications.  Contact info@steptools.com for
 * terms and pricing.
 * 
 * 		----------------------------------------
 */

#include <rose.h>
#include <math.h>

#include <integrated_cnc_schema.h>
#include "stix_imp.h"

/* closeness to zero for comparison on axis transforms */
#define epsilon 1e-15



unsigned get_aspect_type (shape_aspect * sa)
{
    return (sa? sa-> m_type: FEAT_NONE);
}


char * get_aspect_typename (shape_aspect * sa)
{
    switch (get_aspect_type(sa)) {
    case FEAT_NONE: 		return "unrecongized aspect";

    case FEAT_UNKNOWN:		return "unknown feature";
    case FEAT_BOSS: 		return "boss";
    case FEAT_CBORE:		return "counterbore hole";
    case FEAT_CHAMFER:		return "chamfer";
    case FEAT_COMPOUND:		return "compound feature";
    case FEAT_CSUNK: 		return "countersunk hole";
    case FEAT_EDGE_ROUND:	return "edge round";
    case FEAT_HOLE:		return "round hole";
    case FEAT_PAT_CIRC:		return "circular pattern";
    case FEAT_PAT_GEN:		return "general pattern";
    case FEAT_PAT_RECT:		return "rectangular pattern";
    case FEAT_PLANE:		return "planar face";
    case FEAT_POCKET:		return "pocket";
    case FEAT_POCKET_OPEN:	return "open pocket";
    case FEAT_POCKET_CLOSED:	return "closed pocket";
    case FEAT_PROFILE:		return "profile";
    case FEAT_PROF_OUTSIDE:	return "outside profile";
    case FEAT_PROF_SHAPE:	return "shape profile";
    case FEAT_PROF_SHAPE_RECT:	return "rectangular shape profile";	
    case FEAT_REGION:		return "region";
    case FEAT_ROUNDED_END:	return "rounded end";
    case FEAT_SLOT:		return "slot";
    case FEAT_SPHERICAL_CAP:	return "spherical cap";
    case FEAT_STEP:		return "step";
    case FEAT_THREAD:		return "thread";
    case FEAT_TOOLPATH:		return "toolpath";

    case PROF_UNKNOWN:		return "unknown profile";
    case PROF_LINEAR:		return "linear profile";
    case PROF_OPEN_GENERAL:	return "open general profile";
    case PROF_OPEN_CIRC:	return "open circular profile";
    case PROF_ROUNDED_U:	return "rounded u profile";
    case PROF_SQUARE_U:		return "square u profile";
    case PROF_TEE:		return "tee profile";
    case PROF_VEE:		return "vee profile";
	
    case PROF_CLOSED_CIRC:	return "closed circular profile";
    case PROF_CLOSED_NGON:	return "closed ngon profile";
    case PROF_CLOSED_GENERAL:	return "closed general profile";
    case PROF_CLOSED_RECT:	return "closed rectangular profile";

    case PATH_UNKNOWN:		return "unknown path";
    case PATH_LINEAR:		return "linear path";
    case PATH_OPEN_CIRC:	return "open circular path";
    case PATH_CLOSED_CIRC:	return "closed circular path";
    case PATH_GENERAL:		return "general path";

    case COMP_UNKNOWN:		return "unknown feature component";
    case COMP_APPLIED_AREA:	return "applied area";
    case COMP_BOSS_TOP:		return "boss top";
    case COMP_CHAMFER_OFFSET:	return "chamfer offset";
    case COMP_PATTERN_MOD:	return "pattern modification";
    case COMP_RIB_TOP_FLOOR:	return "rib floor";

    case HBOT_UNKNOWN:		return "unknown hole bottom";
    case HBOT_FLAT:		return "flat hole bottom";
    case HBOT_FLAT_RADIUS:	return "flat with radius hole bottom";
    case HBOT_SPHERICAL:	return "spherical hole bottom";
    case HBOT_CONICAL:		return "conical hole bottom";
    case HBOT_THROUGH:		return "through hole bottom";

    case PBOT_UNKNOWN:		return "unknown pocket bottom";
    case PBOT_THROUGH:		return "through pocket bottom";
    case PBOT_FLAT:		return "flat pocket bottom";
    case PBOT_RADIUSED:		return "radiused pocket bottom";
    case PBOT_GENERAL:		return "general pocket bottom";

    case SLOTEND_UNKNOWN:	return "unknown slot end";
    case SLOTEND_WOODRUFF:	return "woodruff slot end";
    case SLOTEND_RADIUSED:	return "radiused slot end";
    case SLOTEND_FLAT:		return "flat slot end";
    case SLOTEND_OPEN:		return "open slot end";
    case SLOTEND_LOOP:		return "loop slot end";

    case TAPER_UNKNOWN:		return "unknown taper";
    case TAPER_DIAMETER:	return "diameter taper";
    case TAPER_ANGLE:		return "angle taper";

    case PROFLOOR_UNKNOWN:	return "unknown profile floor";
    case PROFLOOR_THROUGH:	return "through profile floor";
    case PROFLOOR_PLANAR:		return "flat profile floor";
    case PROFLOOR_GENERAL:	return "general profile floor";

    default:
	return "unknown feature thing";
    }
}

unsigned calc_aspect_base_type (shape_aspect * sa)
{
    /* Calculates an attribute based only on type information
     * immediately available.  Later on we refine this based on
     * additional information gained from traversing relationships 
     */

    /* features are handled by the characterized object 
     * description attribute.   
     */
    if (!sa) return FEAT_NONE;

    if (sa-> isa(ROSE_DOMAIN(instanced_feature)))
    {
	instanced_feature * f = ROSE_CAST(instanced_feature,sa);

	/* we need the description from the characterized object part
	 * of the object.  Since we are using classes, we have to be
	 * careful about a bug that shows up with multiple atts with
	 * the same name.
	 *
	 * The sa.desc and char_obj.desc fields may be mapped to the
	 * same location in memory for late-bound access, like the P21
	 * filer.  Use late bound get call to make sure we get what it
	 * uses.
	 */
	char * desc = f-> getString("description");

	if (sa-> isa(ROSE_DOMAIN(boss)))
	    return FEAT_BOSS;

	if (sa-> isa(ROSE_DOMAIN(flat_face))) 
	    return FEAT_PLANE;


	/* All profiles have extra boundary occurrence shape aspect
	 * properties attached, which tells what type of profile it is
	 *
	 * general outside = 'boundary occurrence'
	 * circ closed shape prof = 'closed circular boundary occurrence'
	 * partial circ shape prof = 'partial circular boundary occurrence'
	 * general shape prof = 'complex boundary occurrence'
	 * rect closed shape prof = 'closed rectangular boundary occurrence'
	 * rect open shape prof = 'open rectangular boundary occurrence'
	 */

	if (sa-> isa(ROSE_DOMAIN(outside_profile))) 
	    return FEAT_PROFILE;

	if (sa-> isa(ROSE_DOMAIN(pocket))) 
	    return FEAT_POCKET;

	if (sa-> isa(ROSE_DOMAIN(removal_volume))) 
	    return FEAT_REGION;

	if (sa-> isa(ROSE_DOMAIN(round_hole)))
	    return FEAT_HOLE;

	if (sa-> isa(ROSE_DOMAIN(rounded_end))) 
	    return FEAT_ROUNDED_END;

	if (sa-> isa(ROSE_DOMAIN(slot))) 
	    return FEAT_SLOT;

	if (sa-> isa(ROSE_DOMAIN(spherical_cap)))
	    return FEAT_SPHERICAL_CAP;

	if (sa-> isa(ROSE_DOMAIN(step))) 
	    return FEAT_STEP;

	/* An ARM thread can also be a catalog thread which is an
	 * externally defined feature with a description of 'thread'
	 */
	if ((sa-> isa(ROSE_DOMAIN(thread))) ||
	    (sa-> isa(ROSE_DOMAIN(externally_defined_feature_definition)) &&
	     !strcmp(desc, "thread")))
	    return FEAT_THREAD;

	/* compound features */
	if (sa-> isa(ROSE_DOMAIN(compound_feature))) 
	{
	    /* check for subtype for holes */
	    if (sa-> isa(ROSE_DOMAIN(composite_hole))) {
		if (!strcmp(desc,"counterbore")) return FEAT_CBORE;
		if (!strcmp(desc,"countersunk")) return FEAT_CSUNK;
		return FEAT_NONE;  /* ERROR */
	    }
	    return FEAT_COMPOUND;
	}


	/* pattern features */
	if (sa-> isa(ROSE_DOMAIN(circular_pattern))) 
	    return FEAT_PAT_CIRC;

	if (sa-> isa(ROSE_DOMAIN(feature_pattern)))
	    return FEAT_PAT_GEN;
 
	if (sa-> isa(ROSE_DOMAIN(rectangular_pattern)))
	    return FEAT_PAT_RECT;

 
	/* no other matches, check to see if it is a bare 
	 * toolpath feature 
	 */
	if (!strcmp(desc,"toolpath"))
	    return FEAT_TOOLPATH;
	

	/* strange one */
	return FEAT_UNKNOWN;
    }

    /* ================================================== */
    /* TRANSITION FEATURES -- aspect, but not instanced_feature */
    if (sa-> isa(ROSE_DOMAIN(transition_feature))) 
    {
	if (sa-> isa(ROSE_DOMAIN(edge_round))) 
	    return FEAT_EDGE_ROUND;

	if (sa-> isa(ROSE_DOMAIN(chamfer)))
	    return FEAT_CHAMFER;

	/* strange one */
	return FEAT_UNKNOWN;
    }

    /* ================================================== */
    /* PROFILES */

    if (sa-> isa(ROSE_DOMAIN(linear_profile))) 
	return PROF_LINEAR;

    if (sa-> isa(ROSE_DOMAIN(open_path_profile))) 
	return PROF_OPEN_GENERAL;

    if (sa-> isa(ROSE_DOMAIN(partial_circular_profile))) 
	return PROF_OPEN_CIRC;

    if (sa-> isa(ROSE_DOMAIN(rounded_u_profile))) 
	return PROF_ROUNDED_U;

    if (sa-> isa(ROSE_DOMAIN(square_u_profile))) 
	return PROF_SQUARE_U;

    if (sa-> isa(ROSE_DOMAIN(tee_profile))) 
	return PROF_TEE;

    if (sa-> isa(ROSE_DOMAIN(vee_profile)))
	return PROF_VEE;

    /* closed profiles */
    if (sa-> isa(ROSE_DOMAIN(circular_closed_profile))) 
	return PROF_CLOSED_CIRC;

    if (sa-> isa(ROSE_DOMAIN(ngon_closed_profile))) 
	return PROF_CLOSED_NGON;

    if (sa-> isa(ROSE_DOMAIN(closed_path_profile))) 
	return PROF_CLOSED_GENERAL;

    if (sa-> isa(ROSE_DOMAIN(rectangular_closed_profile))) 
	return PROF_CLOSED_RECT;



    /* ================================================== */
    /* PATHS */

    if (sa-> isa(ROSE_DOMAIN(path_feature_component))) 
    {
	char * desc = sa-> description();
	if (!strcmp(desc, "linear"))
	    return PATH_LINEAR;
	if (!strcmp(desc, "partial circular"))
	    return PATH_OPEN_CIRC;
	if (!strcmp(desc, "complete circular"))
	    return PATH_CLOSED_CIRC;
	if (!strcmp(desc, "complex"))
	    return PATH_GENERAL;

	/* strange one */
	return PATH_UNKNOWN;
    }

    /* ================================================== */
    /* COMPONENTS */

    if (sa-> isa(ROSE_DOMAIN(applied_area))) 
	return COMP_APPLIED_AREA;

    if (sa-> isa(ROSE_DOMAIN(boss_top))) 
	/* can have desc= planar or complex */
	return COMP_BOSS_TOP;

    if (sa-> isa(ROSE_DOMAIN(chamfer_offset))) 
	/* can have desc=first offset, second offset */
	return COMP_CHAMFER_OFFSET;

    if (sa-> isa(ROSE_DOMAIN(modified_pattern))) 
	return COMP_PATTERN_MOD;

    if (sa-> isa(ROSE_DOMAIN(rib_top_floor))) 
	/* can be desc='planar','complex' */
	return COMP_RIB_TOP_FLOOR;

    if (sa-> isa(ROSE_DOMAIN(hole_bottom))) 
    {
	char * desc = sa-> description();
	if (!strcmp(desc, "through"))
	    return HBOT_THROUGH;
	if (!strcmp(desc, "flat"))
	    return HBOT_FLAT;
	if (!strcmp(desc, "flat with radius"))
	    return HBOT_FLAT_RADIUS;
	if (!strcmp(desc, "spherical"))
	    return HBOT_SPHERICAL;
	if (!strcmp(desc, "conical"))
	    return HBOT_CONICAL;

	/* 'flat with taper' is an AP-224 bottom type that is 
	 * not used by AP-238.  Unclear what the difference is
	 * between this and conical */

 	/* strange one */
	return HBOT_UNKNOWN;
    }

    if (sa-> isa(ROSE_DOMAIN(pocket_bottom))) 
    {
	char * desc = sa-> description();
	if (!strcmp(desc, "through"))
	    return PBOT_THROUGH;
	if (!strcmp(desc, "planar"))
	    return PBOT_FLAT;
	if (!strcmp(desc, "complex"))
	    return PBOT_GENERAL;
	if (!strcmp(desc, "radiused"))
	    return PBOT_RADIUSED;

 	/* strange one */
	return PBOT_UNKNOWN;
    }

    if (sa-> isa(ROSE_DOMAIN(profile_floor))) 
    {
	char * desc = sa-> description();
	if (!strcmp(desc, "through"))
	    return PROFLOOR_THROUGH;
	if (!strcmp(desc, "planar"))
	    return PROFLOOR_PLANAR;
	if (!strcmp(desc, "complex"))
	    return PROFLOOR_GENERAL;

 	/* strange one */
	return PROFLOOR_UNKNOWN;
    }
    if (sa-> isa(ROSE_DOMAIN(slot_end))) 
    {
	char * desc = sa-> description();
	if (!strcmp(desc, "woodruff"))
	    return SLOTEND_WOODRUFF;
	if (!strcmp(desc, "radiused"))
	    return SLOTEND_RADIUSED;
	if (!strcmp(desc, "flat"))
	    return SLOTEND_FLAT;
	if (!strcmp(desc, "open"))
	    return SLOTEND_OPEN;

	/* SLOTEND_LOOP still in question */

 	/* strange one */
	return SLOTEND_UNKNOWN;
    }

    if (sa-> isa(ROSE_DOMAIN(taper))) 
    {
	char * desc = sa-> description();
	if (!strcmp(desc, "angle taper"))
	    return TAPER_ANGLE;
	if (!strcmp(desc, "diameter taper"))
	    return TAPER_DIAMETER;

 	/* strange one */
	return TAPER_UNKNOWN;
    }

    /* miss on all */
    return FEAT_NONE;
}


shape_aspect * get_feature_component(
    shape_aspect * feat,
    const char * sar_desc,	/* often 'foo usage' */
    const char * sa_ed_desc,	/* often 'something foo occurrence' */
    const char * sar_name	/* rare, used for slot ends, others */
    )
{
    StixFeature * idx = get_feature_idx (feat);
    unsigned i, sz;
    char * s;

    /* Components are attached to the feature on the ING side of the
     * relationship.  On the ED side is an intermediary shape aspect
     * that refers to the feature shape aspect.
     * 
     * Often we can find the component by just looking at the desc of
     * the relationship, but sometimes we must look at the desc of the
     * intermediate shape aspect as well.  Usually this string has the
     * suffix "occurrence" tacked on.
     */
    if (!idx) return 0;
    for (i=0, sz=idx->m_component_sars.size(); i<sz; i++) {
	shape_aspect_relationship * sar = 
	    (shape_aspect_relationship *) idx->m_component_sars[i];

	shape_aspect * ed = sar-> related_shape_aspect();

	/* bypass either check if passed in null */
	s = sar-> description();
	if (sar_desc && (!s || strcmp (sar_desc, s)))
	    continue;

	/* check the shape aspect desc if requested */
  	s = (ed? ed-> description(): 0);
  	if (sa_ed_desc && (!s || strcmp (sa_ed_desc, s))) 
  	    continue;

	/* check the relationship name, this is used to distinguish
	 * between multiple copies of the same type of thing, such as
	 * the starting & ending slot_end, or the two holes that form
	 * a countersunk feature.  
	 * 
	 * In retrospect, those mappings should probably have just
	 * used the ED shape aspect description, but we inherited this
	 * from AP-224, so it is beyond our power.  The reason the SAR
	 * name was used might be so the ED aspect could be shared by
	 * both slot ends.  Just conjecture.
	 */
	s = sar-> name();
	if (sar_name && (!s || strcmp (sar_name, s)))
	    continue;

  	/* made it through, everything matched */
	return sar-> relating_shape_aspect();
    }
    return 0;
}

axis2_placement_3d * get_feature_placement (shape_aspect * sa)
{
    axis2_placement_3d * ap;
    unsigned typ = get_aspect_type(sa);

    /* this may just be a big hack, but it was in the mappings */
    if (typ == FEAT_EDGE_ROUND) {
	shape_aspect * final = get_feature_final(sa); 
	shape_representation * rep;

	/* look for a face shape representation */
	rep = get_edge_round_face_shape (final);
	if (!rep) rep = get_edge_round_face_shape (sa);

	representation_item * it = (rep && rep-> items()? 
				    rep->items()->get(0): 0);
	RoseObject * geo = 0;
	RoseObject * pos = 0;

	if (it && it-> getAttribute("face_geometry")) {
	    geo = it-> getObject("face_geometry");
	}

	if (geo && geo-> getAttribute("position")) {
	    pos = geo-> getObject("position");
	}

	if (pos && pos-> isa(ROSE_DOMAIN(axis2_placement_3d)))
	    return ROSE_CAST(axis2_placement_3d,pos);
    }

    /* get the conventional orientation */
    StixFeature * idx = get_feature_idx (sa);

    ap = get_placement (idx-> m_implicit, "orientation");
    ap = (ap? ap: get_placement (idx-> m_explicit));

    /* The mapping got the placement of bosses from the profile.  I
     * don't think ST-Plan is currently writing a placement directly 
     * on the boss feature itself, although it should.  Rather than
     * always look to the profile, I return the real orientation if 
     * it exists and only look at the profile if it is missing.  
     */
    if ((typ == FEAT_BOSS) && !ap) {
	ap = get_profile_placement (get_boss_boundary (sa));
    }

    return ap;
}


StixMtrx calc_feature_orientation(shape_aspect * feat)
{
    unsigned typ = get_aspect_type(feat);
    axis2_placement_3d * ap = get_feature_placement(feat);

    StixMtrx orient; 
    StixMtrx_from_ap (ap,orient);  /* populate from axis placement */

    /* The original mapping returned the axis and ref direction
     * as a list [ Z, X ] from the placement, but it with some
     * customizations for different types of features
     */
    switch (typ) {
    case FEAT_STEP:
    {
	// Pivot the coordinates so that y is now z'. z becomes x'.
	// Since we only start with x and z, we need to do the cross
	// product first.  This assumes that the tilt angle is zero,
	// which is taken care of by the honeywell software.
	// adjust ref_direction to be (axis cross ref_direction)

	/* Previously honeywell had been placing the v profile
	 * with the Y axis in the middle of the profile angle (which
	 * is the enclosed angle of the profile.  What we want is a 
	 * placement with a tilt angle of zero.
	 */

	/* If we ever run into this again, can this be done with the
	 * genRotation function of the matrix class and then apply
	 * that to the placement.
	 */
	StixVctr oldx (orient.x());
	StixVctr oldy (orient.y());
	StixVctr oldz (orient.z());

	orient.x(oldz);  /* pivot the coordinate system */
	orient.y(oldx);
	orient.z(oldy);
	break;
    }

    case FEAT_POCKET_CLOSED:
    {
	/* For closed_general profiles, reset the orientation to the
	 * default.  Without this, general profiles end up pointing
	 * down into the part.  Unclear whos problem this is
	 */
	if (PROF_CLOSED_GENERAL == get_aspect_type(get_pocket_profile(feat)))
	{
	    /* Create a matrix with the default orientation an copy 
	     * only the 3x3 portion, leaving the placement intact.
	     */
	    StixMtrx dflt; 
	    orient.copy3(dflt);	/* change to the default orientation */
	}
	break;
    }
    case FEAT_POCKET_OPEN:
    {
	/* This is doing a similar thing to above for open general.
	 * Don't know why, but the mapping was using the Z axis from
	 * the file, then combining with a default ref dir along the
	 * global X axis.  Likely to make the axes non orthogonal.
	 */
	if (PROF_OPEN_GENERAL == get_aspect_type(get_pocket_profile(feat)))
	{
	    StixVctr refdir (1,0,0);
	    orient.x(refdir);  
	    orient.y(orient.z() * refdir);	/* calc Y = Z cross X */
	}
	break;
    }
    default:
	break; /* Just use the original ap3d values */
    }

    /* weed out tiny values */
    for (unsigned i=0; i<3; i++) {
	if (fabs(orient.get(0,i)) < epsilon) orient.put(0,0,i);
	if (fabs(orient.get(1,i)) < epsilon) orient.put(0,1,i);
	if (fabs(orient.get(2,i)) < epsilon) orient.put(0,2,i);
    }
    return orient;
}


double calc_feature_depth(shape_aspect * feat, StixUnit u)
{
    unsigned typ = get_aspect_type(feat);
    axis2_placement_3d * ap = get_feature_placement(feat);

    switch (typ) {

    case FEAT_HOLE:
	return get_path_linear_distance (get_hole_path(feat), u);

    case FEAT_PROF_OUTSIDE:
    {
	shape_aspect * lp = get_feature_component
	    (feat, "path feature component usage",
	     "outside profile depth occurrence");

	return get_path_linear_distance (lp, u);
    }

    case FEAT_POCKET:
    case FEAT_POCKET_OPEN:
    case FEAT_POCKET_CLOSED:
	return get_path_linear_distance (get_pocket_path(feat), u);


    case FEAT_STEP:
    case FEAT_PLANE:
    {
	/* these don't have a proper depth, so we measure from the
	 * feature origin to the stock along the positive Z direction
	 */
	product_definition * pd = get_feature_workpiece(feat);

	StixMtrx place = calc_feature_orientation(feat);
	StixVctr dir = place.z();
	StixVctr loc = place.w();
	double dir_comp, loc_comp;
	double axis_dist, length=-1;

	/* Since this is a unit vector, we can find out the depth by
	   scaling it out to the planes at x,y,z max and min.  Careful
	   about zero components and lengths less than one.
	 */
	
	/* check distance to max/min x */
	dir_comp = dir.get(0);
	loc_comp = loc.get(0);

	/* check for near zero */
	if (fabs(dir_comp) > epsilon) {
	    axis_dist = 
		(dir_comp < 0?
		 (loc_comp - get_workpiece_stock_min_x (pd)):
		 (get_workpiece_stock_max_x (pd) - loc_comp));

	    /* scale the vector and find the length, since it is a
	     * unit vector, we only need to divide by the component
	     */
	    length = axis_dist / fabs(dir_comp);
	}

	
	/* check distance to max/min y */
	dir_comp = dir.get(1);
	loc_comp = loc.get(1);

	/* check for near zero */
	if (fabs(dir_comp) > epsilon) {
	    axis_dist = 
		(dir_comp < 0?
		 (loc_comp - get_workpiece_stock_min_y (pd)):
		 (get_workpiece_stock_max_y (pd) - loc_comp));

	    /* scale the vector and find the length, since it is a
	     * unit vector, we only need to divide by the component
	     */
	    double newlen = axis_dist / fabs(dir_comp);
	    if (length < 0 || length > newlen)
		length = newlen;
	}


	/* check distance to max/min z */
	dir_comp = dir.get(2);
	loc_comp = loc.get(2);

	/* check for near zero */
	if (fabs(dir_comp) > epsilon) {
	    axis_dist = 
		(dir_comp < 0?
		 (loc_comp - get_workpiece_stock_min_z (pd)):
		 (get_workpiece_stock_max_z (pd) - loc_comp));

	    /* scale the vector and find the length, since it is a
	     * unit vector, we only need to divide by the component
	     */
	    double newlen = axis_dist / fabs(dir_comp);
	    if (length < 0 || length > newlen)
		length = newlen;
	}
	/* watch for crazy things and placements outside of the
           bounding box */
	if (length < 0) length = 0;
	return length;
    }

    case FEAT_CBORE:
    case FEAT_CHAMFER:
    case FEAT_COMPOUND:
    case FEAT_CSUNK:
    case FEAT_EDGE_ROUND:
    case FEAT_PAT_CIRC:
    case FEAT_PAT_GEN:
    case FEAT_PAT_RECT:
    case FEAT_PROFILE:
    case FEAT_PROF_SHAPE:
    case FEAT_PROF_SHAPE_RECT:
    case FEAT_REGION:
    case FEAT_ROUNDED_END:
    case FEAT_SLOT:
	/* this really needs to look at the profile */

    case FEAT_SPHERICAL_CAP:
    case FEAT_THREAD:
    {
	/* look for any sort of path component */
	shape_aspect * lp = get_feature_component 
	    (feat, "path feature component usage");

	return get_path_linear_distance (lp, u);
    }
    } /* end switch */
    return 0;
}


product_definition * get_feature_workpiece (shape_aspect * sa)
{
    product_definition_shape * pds = (sa? sa-> of_shape(): 0);
    RoseObject * pdo;

    if (!pds) return 0;
    pdo = get_select_object (pds->definition());
    if (pdo && pdo-> isa (ROSE_DOMAIN(product_definition))) {
	product_definition * pd = ROSE_CAST(product_definition,pdo);
	
	/* quick check to make sure it is a workpiece */
	return (get_workpiece_idx(pd)? pd: 0);
    }
    return 0;
}


shape_aspect * 	get_feature_final(shape_aspect * sa)
{
    StixFeature * idx = get_feature_idx(sa);
    return (idx? idx-> m_final_sa: 0);
}


/* ---------------------------------------- */
/* Edge Round */
face_shape_representation * get_edge_round_face_shape (
    shape_aspect * sa
    )
{
    if (!sa || (FEAT_EDGE_ROUND != sa->m_type)) return 0;
    return get_face_shape_property (sa, "edge round face");
}

face_shape_representation * get_edge_round_first_face (
    shape_aspect * sa
    )
{
    if (!sa || (FEAT_EDGE_ROUND != sa->m_type)) return 0;
    return get_face_shape_property (sa, "first face shape");
}

face_shape_representation * get_edge_round_second_face (
    shape_aspect * sa
    )
{
    if (!sa || (FEAT_EDGE_ROUND != sa->m_type)) return 0;
    return get_face_shape_property (sa, "second face shape");
}


double get_edge_round_radius (shape_aspect * sa, StixUnit u)
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (FEAT_EDGE_ROUND != sa->m_type) return 0;

    return get_length_value (idx-> m_implicit, "radius", u);
}

double get_edge_round_first_offset (shape_aspect * sa, StixUnit u)
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (FEAT_EDGE_ROUND != sa->m_type) return 0;

    return get_length_value (idx-> m_implicit, "first offset", u);
}

double get_edge_round_second_offset (shape_aspect * sa, StixUnit u)
{
    StixComponent * idx = get_component_idx(sa);

    if (!idx) return 0;
    if (FEAT_EDGE_ROUND != sa->m_type) return 0;

    return get_length_value (idx-> m_implicit, "second offset", u);
}





/* ---------------------------------------- */
/* Round Hole */
shape_aspect *  get_hole_profile (shape_aspect * sa)
{
    return get_feature_component (
	sa, "profile usage", "diameter occurrence"
	);
}

shape_aspect *  get_hole_path (shape_aspect * sa)
{
    return get_feature_component (
	sa, "path feature component usage", "hole depth occurrence"
	);
}

shape_aspect * 	get_hole_taper (shape_aspect * sa)
{
    /* holes, bosses, pockets */
    return get_feature_component (
	sa, "taper usage", "change in diameter occurrence"
	);

    /* type should = taper.  The sa.description tells what
     * taper ist is "angle taper"  or "diameter taper" 
     */

    // The SA -ed description changes depending on the type of the
    // feature.  For validation and creation, we should make sure to
    // set these properly.

    // hole --> "change in diameter occurrence" 
    // boss, pocket --> "change in boundary occurrence" 
}

shape_aspect * get_hole_bottom_condition (shape_aspect * sa)
{
    return get_feature_component (
	sa, "hole bottom usage", "bottom condition occurrence"
	);

    /* type should = hole_bottom. */
}

double get_hole_diameter (shape_aspect * sa, StixUnit u)
{
    shape_aspect * dp = get_hole_profile(sa);
    return get_profile_circle_diameter (dp, u);
}

double get_hole_final_diameter (shape_aspect * sa, StixUnit u)
{
    /* This is replicating an EXPRESS-X mapping.   You may
       want to just get the taper and figure it out for yourself */

    shape_aspect * dp = get_hole_profile(sa);
    shape_aspect * tp = get_hole_taper(sa);

    /* need to calculate with depth if we have an angle taper */
    if (tp) return get_taper_final_diameter (tp, u);

    return get_profile_circle_diameter (dp, u);
}



/* ---------------------------------------- */
/* Pocket */

shape_aspect *  get_pocket_profile (shape_aspect * sa)
{
    return get_feature_component (
	sa, "profile usage", "boundary occurrence"
	);
}

shape_aspect * 	get_pocket_path (shape_aspect * sa)
{
    return get_feature_component
	(sa, "path feature component usage",
	 "pocket depth occurrence");
}

shape_aspect *  get_pocket_open_wall_profile (shape_aspect * sa)
{
    return get_feature_component (
	sa, "wall boundary usage", "wall boundary occurrence"
	);
}


shape_aspect * 	get_pocket_taper (shape_aspect * sa)
{
    /* holes, bosses, pockets */
    return get_feature_component (
	sa, "taper usage", "change in boundary occurrence"
	);

    /* type should = taper.  The sa.description tells what
     * taper ist is "angle taper"  or "diameter taper" 
     */

    // The SA -ed description changes depending on the type of the
    // feature.  For validation and creation, we should make sure to
    // set these properly.

    // hole --> "change in diameter occurrence" 
    // boss, pocket --> "change in boundary occurrence" 
}


shape_aspect * get_pocket_bottom_condition (shape_aspect * sa)
{
    return get_feature_component (
	sa, "pocket bottom usage", "bottom condition occurrence"
	);

    /* type should = pocket_bottom */
}

double get_pocket_planar_radius (shape_aspect * sa, StixUnit u)
{
    shape_aspect * bc = get_pocket_bottom_condition (sa);
    return get_pocketbottom_radius (bc, u);
}

double get_pocket_orthogonal_radius (shape_aspect * sa, StixUnit u)
{
    StixComponent * idx = get_component_idx(sa);
    if (!idx) return 0;
    return get_length_value (
	idx-> m_implicit, "fillet radius", u

//  FIXME -- need to update ST-Plan
//	idx-> m_implicit, "orthogonal fillet radius", u
	);
}


double get_pocket_slope_angle (shape_aspect * sa, StixUnit u)
{
    // should return 0 if missing?
    shape_aspect * tp = get_pocket_taper (sa);
    return get_taper_angle (tp, u);
}



/* ---------------------------------------- */
/* Boss */

shape_aspect * 	get_boss_taper (shape_aspect * sa)
{
    /* holes, bosses, pockets */
    return get_feature_component (
	sa, "taper usage", "change in boundary occurrence"
	);

    /* type should = taper.  The sa.description tells what
     * taper ist is "angle taper"  or "diameter taper" 
     */

    // The SA -ed description changes depending on the type of the
    // feature.  For validation and creation, we should make sure to
    // set these properly.

    // hole --> "change in diameter occurrence" 
    // boss, pocket --> "change in boundary occurrence" 
}


shape_aspect * 	get_boss_boundary (shape_aspect * sa)
{
    return get_feature_component (sa, "profile usage");

    /* the sa_ed description should be one of the following,
       depending on the type of profile

        'circular profile occurrence'
        'enclosed profile occurrence'
        'rectangular profile occurrence'
    */
}



/* ---------------------------------------- */
/* Planar Face */
shape_aspect * 	get_planar_face_course_of_travel (shape_aspect * sa)
{
    // should be a linear path
    return get_feature_component (
	sa, "path usage", "course of travel occurrence"
	);
}

shape_aspect * 	get_planar_face_removal_boundary (shape_aspect * sa)
{
    // should be a linear profile
    return get_feature_component (
	sa, "profile usage", "removal boundary occurrence"
	);
}

shape_aspect * get_planar_face_face_boundary (shape_aspect * sa)
{
    // should be a linear profile
    return get_feature_component (
	sa, "profile usage", "enclosed boundary occurrence"
	);
}



/* ---------------------------------------- */
/* slots */
shape_aspect * get_slot_end_at_start (shape_aspect * sa)
{
    // should be a slot_end
    return get_feature_component (
	sa, "slot end usage", 
	"end condition occurrence",
	"course of travel start"
	);
}
shape_aspect * get_slot_end_at_end (shape_aspect * sa)
{
    // should be a slot_end
    return get_feature_component (
	sa, "slot end usage", 
	"end condition occurrence",
	"course of travel end"
	);
}

shape_aspect * get_slot_course_of_travel (shape_aspect * sa)
{
    return get_feature_component (
	sa, "path feature component usage", 
	"course of travel occurrence"
	/* course of travel -- in the mappings but superfluous */
	);
}

shape_aspect * get_slot_swept_shape (shape_aspect * sa)
{
    return get_feature_component (
	sa, "profile usage", 
	"swept shape occurrence"
	);
}



/* ---------------------------------------- */
/* outside profile */

shape_aspect * get_outside_profile_feature_boundary (shape_aspect * sa)
{
    return get_feature_component (
	sa, "profile usage", 
	"boundary occurrence"
	);

    /* this returned sa should have a description of "outside boundary" */

}

shape_aspect * get_outside_profile_sweep_path (shape_aspect * sa)
{
    /* This mapping needs to be cleaned up in both AP-238 and 
       in AP-224.  The type and ED-sa desc are wrong in 224, 
       AP-238 corrected the type, but replicated the desc problems
    */

    return get_feature_component (
	sa, "path feature component usage"
	/* no established value for ED desc */
	);
}




/* ---------------------------------------- */
/* shape profile */

shape_aspect * get_shape_profile_feature_boundary (shape_aspect * sa)
{
    if (!sa || !(FEAT_PROF_SHAPE == sa->m_type || 
		 FEAT_PROF_SHAPE_RECT == sa->m_type)) return 0;

    return get_feature_component (
	sa, "profile usage"
	);

    /* the sa_ed is one of 
       'complex boundary occurrence'
       'partial circular boundary occurrence'
       'closed circular boundary occurrence'
       'open rectangular boundary occurrence'
       'closed rectangular boundary occurrence'
    */

}

shape_aspect * get_shape_profile_sweep_path (shape_aspect * sa)
{
    if (!sa || !(FEAT_PROF_SHAPE == sa->m_type || 
		 FEAT_PROF_SHAPE_RECT == sa->m_type)) return 0;

    return get_feature_component (
	sa, "path feature component usage"
	/* no established value for ED desc */
	);
}

shape_aspect * get_shape_profile_floor (shape_aspect * sa)
{
    /* This original AP-224 mapping seems to indicate that this is
       connected to the same intermediate SA as the feature boundary.
       I don't know if that was intentional or a cut and paste error.
       If intentional, it is unusual.

    */
    if (!sa || !(FEAT_PROF_SHAPE == sa->m_type || 
		 FEAT_PROF_SHAPE_RECT == sa->m_type)) return 0;

    return get_feature_component (
	sa, "profile floor usage"
	/* see comment */
	);
}

direction * get_shape_profile_direction (shape_aspect * sa)
{
    /* The mappings have this on an intermediate SA, 
       but ST-Plan is currently putting it out this way
    */

    if (!sa || !(FEAT_PROF_SHAPE == sa->m_type || 
		 FEAT_PROF_SHAPE_RECT == sa->m_type)) return 0;

    return get_direction (get_direction_shape_property(sa));
}


/* ---------------------------------------- */
/* step feature */

shape_aspect * 	get_step_wall_boundary (shape_aspect * sa)
{
    if (!sa || (FEAT_STEP != sa->m_type)) return 0;
    return get_feature_component (
	sa, "profile usage",
	"removal boundary occurrence"
	);
}

shape_aspect * 	get_step_course_of_travel (shape_aspect * sa)
{
    if (!sa || (FEAT_STEP != sa->m_type)) return 0;
    return get_feature_component (
	sa, "path feature component usage",
	"course of travel occurrence"
	);
}
