/* $RCSfile: geometry.cxx,v $
 * $Revision: 1.8 $ $Date: 2003/05/27 18:30:35 $
 * Auth: Dave Loffredo (loffredo@steptools.com)
 * 
 * Copyright (c) 1991-2003 by STEP Tools Inc. 
 * All Rights Reserved.
 * 
 * This file is part of the STIX Library
 * 
 * This file may be distributed and/or modified under the terms of 
 * the GNU General Public License version 2 as published by the Free
 * Software Foundation and appearing in the file LICENSE.GPL included
 * with this file.
 * 
 * THIS FILE IS PROVIDED "AS IS" WITH NO WARRANTY OF ANY KIND,
 * INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * STEP Tools Inc. also offers licenses for commercial use of this
 * software in non-GPL applications.  Contact info@steptools.com for
 * terms and pricing.
 * 
 * 		----------------------------------------
 */

/* REMEMBER!!!    REMEMBER!!!    REMEMBER!!! */
/* express things index from 1, rose and c++ from 0 */
/* REMEMBER!!!    REMEMBER!!!    REMEMBER!!! */

/* Dealing with the transforms, AXIS is a unit vector along z,
   REFERENCE DIRECTION is a unit vector along x, so the y axis
   can be found as Z x X using the standard cross product
*/

#include <rose.h>
#include <integrated_cnc_schema.h>
#include "stix_imp.h"

#include "xform.h"

void apply_point_limits (
    StixShapeLimit * sl,
    StixMtrx * xform,
    double x, double y, double z
    )
{
    /* compare a given point against maxes and mins and push 
     * them out if needed. 
     */

    /* transform the point to the given axis placement */
    StixVctr cp(x,y,z);

//    printf ("applying point ");  cp.print (stdout);
    cp = cp * (*xform);
//    printf ("    as xformed ");  cp.print (stdout);

    x = cp.get(0);
    y = cp.get(1);
    z = cp.get(2);

    if (sl-> m_min.m_x == ROSE_NULL_REAL) {
	/* first point seen */
	sl-> m_min.m_x = x;
	sl-> m_max.m_x = x;
	sl-> m_min.m_y = y;
	sl-> m_max.m_y = y;
	sl-> m_min.m_z = z;
	sl-> m_max.m_z = z;
    }

    if (x < sl-> m_min.m_x)  sl-> m_min.m_x = x;
    if (x > sl-> m_max.m_x)  sl-> m_max.m_x = x;

    if (y < sl-> m_min.m_y)  sl-> m_min.m_y = y;
    if (y > sl-> m_max.m_y)  sl-> m_max.m_y = y;

    if (z < sl-> m_min.m_z)  sl-> m_min.m_z = z;
    if (z > sl-> m_max.m_z)  sl-> m_max.m_z = z;
}



void apply_vertex_limits (
    StixShapeLimit * sl,
    StixMtrx * xform,
    vertex * v
    )
{
    /* must be a vertex point */
    if (!v) return;
    if (!v || !v-> isa(ROSE_DOMAIN(vertex_point)))
	return;

    vertex_point * vp = ROSE_CAST(vertex_point, v);
    if (!vp->vertex_geometry() || 
	!vp->vertex_geometry()-> isa(ROSE_DOMAIN(cartesian_point)))
	return;

    cartesian_point * cp = ROSE_CAST(cartesian_point, vp->vertex_geometry());
    ListOfdouble * coords = cp-> coordinates();

    if (coords) apply_point_limits(
	sl, xform,
	coords-> get(0),
	coords-> get(1),
	coords-> get(2)
	);
}



void apply_edge_limits (
    StixShapeLimit * sl,
    StixMtrx * xform,
    edge * e
    )
{
    apply_vertex_limits (sl, xform, e-> edge_start());
    apply_vertex_limits (sl, xform, e-> edge_end());

    if (e-> isa(ROSE_DOMAIN(oriented_edge))) {
	oriented_edge * oe = ROSE_CAST(oriented_edge,e);
	apply_edge_limits (sl,xform,oe->edge_element());
    }
    if (e-> isa(ROSE_DOMAIN(edge_curve))) {
	edge_curve * ec = ROSE_CAST(edge_curve,e);
	/* do something with the edge geometry? */
    }
}



void apply_loop_limits (
    StixShapeLimit * sl,
    StixMtrx * xform,
    loop * lp
    )
{
    unsigned i, sz;

    if (!lp) return;
    if (lp-> isa (ROSE_DOMAIN(edge_loop))) 
    {
	edge_loop * el = ROSE_CAST (edge_loop,lp);
	ListOforiented_edge * edges = (el? el-> edge_list(): 0);

	if (!edges) return;
	for (i=0, sz=edges->size(); i<sz; i++) {
	    apply_edge_limits (sl, xform, edges-> get(i));
	}
    }
    else if (lp-> isa (ROSE_DOMAIN(vertex_loop))) 
    {
	vertex_loop * vl = ROSE_CAST (vertex_loop,lp);
	apply_vertex_limits (sl, xform, vl->loop_vertex());
    }
}


void apply_geometry_limits (
    StixShapeLimit * sl,
    StixMtrx * xform,
    manifold_solid_brep * mbr
    )
{
    unsigned i, sz;
    unsigned j, szz;
    SetOfface * faces = (mbr->outer()? mbr->outer()->cfs_faces() : 0);

    if (!faces) return;
    for (i=0, sz=faces->size(); i<sz; i++) {
	face * f = faces-> get(i);

	if (!f-> isa (ROSE_DOMAIN(advanced_face)))
	    continue;

	/* look at face_bound in af.bounds; */
	advanced_face * af = ROSE_CAST(advanced_face,f);
	
	/* loop over the bounds and add the points in edge loops and
	 * vertex loops 
	 */
	for (j=0, szz=af->bounds()->size(); j<szz; j++) {
	    apply_loop_limits (sl, xform, af->bounds()-> get(j)->bound());
	}

	/* Look at the face geometry */
	if (!af-> face_geometry()) continue;

	/* look at cylindrical surface face geometry */
	/* I don't know if this makes sense in the general case, or if
           it was something hacked specifically for the KCP data, but
           this is a straight transcription of the expx views */

	/* cylinder ymin and ymax */
	if (af-> face_geometry()-> isa (ROSE_DOMAIN(cylindrical_surface))) {
	    cylindrical_surface * cs = 
		ROSE_CAST (cylindrical_surface,af-> face_geometry());

	    /* in the XY plane */
	    apply_point_limits(sl, xform, cs->radius(), cs->radius(), 0);
	    apply_point_limits(sl, xform, -cs->radius(), -cs->radius(), 0);
	}
    }
}



void apply_brep_limits (
    StixShapeLimit * sl,
    StixMtrx * xform,
    shape_representation * rep
    )
{
    unsigned i, sz;
    SetOfrepresentation_item * its = (rep? rep-> items(): 0);
    axis2_placement_3d * place = get_placement(rep);

    if (!its) return;
    if (place) StixMtrx_from_ap (place, (*xform));

    /* go through the other geometry */
    for (i=0, sz=its->size(); i<sz; i++) {
	representation_item * it = its-> get(i);
	
	/* Look through all of the geometry in a brep to find the max
	 * and min points.  Do this on the untransformed geometry and
	 * then apply the placement later on.
	 */
	if (it-> isa (ROSE_DOMAIN(manifold_solid_brep))) {
	    /* what if we have multiples */
	    manifold_solid_brep * mbr = ROSE_CAST(manifold_solid_brep,it);
	    apply_geometry_limits (sl, xform, mbr);
	}

	/* what about other types of geometry? */
    }
}





void apply_bbox_limits (
    StixShapeLimit * sl,
    StixMtrx * xform,
    shape_representation * rep
    )
{
    if (!rep) return;

    axis2_placement_3d * place = get_placement(rep);

    /* FIXME -- this should understand geometry length units */
    double l = get_length_value(rep, "Length");		// should not be caps
    double w = get_length_value(rep, "Width");
    double h = get_length_value(rep, "Height");

    /* apply placement transform */
    if (place) StixMtrx_from_ap (place, (*xform));

    apply_point_limits(sl, xform,  0, 0, 0);
    apply_point_limits(sl, xform,  l, w, h);

// by the spec, should be 
//    apply_point_limits(sl, xform,  w, h, l);


//      apply_point_limits(sl, xform,  +(l/2), +(w/2), +(h/2));
//      apply_point_limits(sl, xform,  -(l/2), -(w/2), -(h/2));
}



StixShapeLimit * calc_shape_limits (
    product_definition * pdef
    )
{
    StixWorkpiece * idx = get_workpiece_idx(pdef);
    StixShapeLimit * sl = 0;
    StixMtrx xform;

    /* do we need to get the workpiece shape units? */

    if (idx && idx-> m_shape) {
	sl = new StixShapeLimit;
	sl-> m_min.m_x = ROSE_NULL_REAL;
	apply_brep_limits (sl, &xform, idx->m_shape);
//	xform.print(stdout);
    }

    return sl;
}

StixShapeLimit * calc_bbox_limits (
    product_definition * pdef
    )
{
    StixWorkpiece * idx = get_workpiece_idx(pdef);
    StixShapeLimit * sl = 0;
    StixMtrx xform;

    /* do we need to get the workpiece shape units? */

    if (idx && idx-> m_bbox) {
  	sl = new StixShapeLimit;
	sl-> m_min.m_x = ROSE_NULL_REAL;
  	apply_bbox_limits (sl, &xform, idx->m_bbox);
    }

    return sl;
}

