/* $RCSfile: measure.cxx,v $
 * $Revision: 1.12 $ $Date: 2003/03/24 23:48:43 $
 * Auth: Dave Loffredo (loffredo@steptools.com)
 * 
 * Copyright (c) 1991-2003 by STEP Tools Inc. 
 * All Rights Reserved.
 * 
 * This file is part of the STIX Library
 * 
 * This file may be distributed and/or modified under the terms of 
 * the GNU General Public License version 2 as published by the Free
 * Software Foundation and appearing in the file LICENSE.GPL included
 * with this file.
 * 
 * THIS FILE IS PROVIDED "AS IS" WITH NO WARRANTY OF ANY KIND,
 * INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * STEP Tools Inc. also offers licenses for commercial use of this
 * software in non-GPL applications.  Contact info@steptools.com for
 * terms and pricing.
 * 
 * 		----------------------------------------
 */

#include <rose.h>
#include <integrated_cnc_schema.h>
#include "stix_imp.h"



/* PRINT MEASURE VALUE -- STEP measures are stored as a select type of
 * many different types of value. This function examines a measure and
 * prints out the numeric or string value.  Measures are usually
 * associated with a unit.
 */
void print_measure_value (measure_value * m)
{
    if (!m) return;
    if (m-> is_context_dependent_measure()) { 
	printf ("%.15g", m-> _context_dependent_measure());	
	return;
    }
    if (m-> is_count_measure()) { 
	printf ("%.15g", m-> _count_measure());	
	return;
    }
    if (m-> is_descriptive_measure()) { 
	printf ("%s", m-> _descriptive_measure());	
	return;
    }
    if (m-> is_length_measure()) { 
	printf ("%.15g", m-> _length_measure());	
	return;
    }
    if (m-> is_numeric_measure()) { 
	printf ("%.15g", m-> _numeric_measure());	
	return;
    }
    if (m-> is_plane_angle_measure()) { 
	printf ("%.15g", m-> _plane_angle_measure());	
	return;
    }
    if (m-> is_parameter_value()) { 
	printf ("%.15g", m-> _parameter_value());	
	return;
    }
    if (m-> is_positive_length_measure()) { 
	printf ("%.15g", m-> _positive_length_measure());	
	return;
    }
    if (m-> is_positive_plane_angle_measure()) { 
	printf ("%.15g", m-> _positive_plane_angle_measure());	
	return;
    }
    if (m-> is_ratio_measure()) { 
	printf ("%.15g", m-> _ratio_measure());	
	return;
    }
    if (m-> is_solid_angle_measure()) { 
	printf ("%.15g", m-> _solid_angle_measure());	
	return;
    }
    if (m-> is_time_measure()) { 
	printf ("%.15g", m-> _time_measure());	
	return;
    }
}

unit * get_length_unit (shape_representation * rep)
{
    RoseObject * obj;
    SetOfunit * units;
    representation_context * rc = rep-> context_of_items();
    unsigned i,sz;

    if (!rc || !rc->isa(ROSE_DOMAIN(global_unit_assigned_context)))
	return 0;

    /* use late bound calls because it requires less casting twidling
     * and to avoid any possible and/or strangeness with extra contexts.
     */
    units = ROSE_CAST(SetOfunit,rc->getObject("units"));    
    if (!units) return 0;

    for (i=0, sz=units->size(); i<sz; i++) {
	obj = get_select_object(units->get(i));
	if (obj && obj-> isa(ROSE_DOMAIN(length_unit)))
	    return units->get(i);
    }
    return 0;
}




/* PRINT UNIT NAME -- STEP units are usually represented as a complex
 * instance composed of named unit types.  The usual set of units are
 * the SI units (meter, gram, second), although it is posible to define
 * conversion-based units such as inches, pounds, etc.  It is also
 * possible to define arbitrary contxt-dependant units such as scoops,
 * rolls, etc.
 */
char * get_unit_name (unit * u)
{ 
    static RoseStringObject name;

    if (!u) return "none";
    if (u-> is_named_unit()) {
	named_unit * nu = u-> _named_unit();

	if (nu-> isa (ROSE_TYPE(si_unit))) {
	    si_unit * su = ROSE_CAST (si_unit, nu);
	    switch (su->prefix()) {
	    case si_prefix_exa:		name = "exa";	break;
	    case si_prefix_pico:	name = "pico";	break;
	    case si_prefix_mega:	name = "mega";	break;
	    case si_prefix_femto:	name = "femto";	break;
	    case si_prefix_atto:	name = "atto";	break;
	    case si_prefix_centi:	name = "centi";	break;
	    case si_prefix_nano:	name = "nano";	break;
	    case si_prefix_hecto:	name = "hecto";	break;
	    case si_prefix_micro:	name = "micro";	break;
	    case si_prefix_tera:	name = "tera";	break;
	    case si_prefix_giga:	name = "giga";	break;
	    case si_prefix_milli:	name = "milli";	break;
	    case si_prefix_peta:	name = "peta";	break;
	    case si_prefix_deci:	name = "deci";	break;
	    case si_prefix_kilo:	name = "kilo";	break;
	    case si_prefix_deca:	name = "deca";	break;
	    case si_prefix_NULL:	
	    default:
		name = ""; /* nothing */ 
		break;
	    }

	    switch (su->name()) {
	    case si_unit_name_hertz:		name += "hertz";	break;
	    case si_unit_name_degree_celsius:	name += "degree_celsius"; break;
	    case si_unit_name_siemens:		name += "siemens";	break;
	    case si_unit_name_sievert:		name += "sievert";	break;
	    case si_unit_name_lux:		name += "lux";		break;
	    case si_unit_name_watt:		name += "watt";	break;
	    case si_unit_name_ohm:		name += "ohm";		break;
	    case si_unit_name_second:		name += "second";	break;
	    case si_unit_name_becquerel:	name += "becquerel";	break;
	    case si_unit_name_pascal:		name += "pascal";	break;
	    case si_unit_name_henry:		name += "henry";	break;
	    case si_unit_name_tesla:		name += "tesla";	break;
	    case si_unit_name_volt:		name += "volt";		break;
	    case si_unit_name_joule:		name += "joule";	break;
	    case si_unit_name_kelvin:		name += "kelvin";	break;
	    case si_unit_name_ampere:		name += "ampere";	break;
	    case si_unit_name_gram:		name += "gram";		break;
	    case si_unit_name_steradian:	name += "steradian";	break;
	    case si_unit_name_mole:		name += "mole";		break;
	    case si_unit_name_lumen:		name += "lumen";	break;
	    case si_unit_name_gray:		name += "gray";		break;
	    case si_unit_name_candela:		name += "candela";	break;
	    case si_unit_name_farad:		name += "farad";	break;
	    case si_unit_name_radian:		name += "radian";	break;
	    case si_unit_name_newton:		name += "newton";	break;
	    case si_unit_name_metre:		name += "metre";	break;
	    case si_unit_name_weber:		name += "weber";	break;
	    case si_unit_name_coulomb:		name += "coulomb";	break;
	    case si_unit_name_NULL:	
	    default:
		name = "none";
		break;
	    }

//  	    /* scan for common combinations */
//  	    if (strcmp (name,"metre"))		name = "m";
//  	    if (strcmp (name,"millimetre"))	name = "mm";
//  	    if (strcmp (name,"centimetre"))	name = "cm";
//  	    if (strcmp (name,"kilogram"))	name = "kg";

	    return name.ro_str();
	}

	if (nu-> isa (ROSE_TYPE(conversion_based_unit))) {
	    conversion_based_unit * cbu = 
		ROSE_CAST(conversion_based_unit,nu);

	    name = (cbu->name()? cbu->name(): "none");

//  	    strtolower (name);

//  	    /* scan for common combinations */
//  	    if (strcmp (name,"inch"))		name = "in";

	    return name.ro_str();

	}


	if (nu-> isa (ROSE_TYPE(context_dependent_unit))) {
	    context_dependent_unit * cdu = 
		ROSE_CAST(context_dependent_unit,nu);
	    name = (cdu->name()? cdu->name(): "none");
	    return name.ro_str();
	}
    }
    return "unknown";
}



char * get_unit_name (StixUnit u)
{
    switch (u) {
    case stixunit_default: 	return "default";
    case stixunit_mm:		return "mm";
    case stixunit_cm:		return "cm";
    case stixunit_m:		return "m";
    case stixunit_in:		return "in";
    case stixunit_ft:		return "ft";

    case stixunit_deg:		return "deg";
    case stixunit_rad:		return "radian";

    case stixunit_sec:		return "sec";

    case stixunit_ips:		return "in/s";
    case stixunit_fps:		return "ft/s";
    case stixunit_mmps:		return "mm/s";
    case stixunit_cmps:		return "cm/s";
    case stixunit_mps:		return "m/s";

    case stixunit_hertz:	return "hz";	
    case stixunit_rpm:		return "rpm";
    
    case stixunit_pa:		return "Pa";
    case stixunit_kpa:		return "kPa";
    case stixunit_mpa:		return "mPa";
    case stixunit_psi:		return "psi";

    default:			return "unknown";
    }
}


struct StixUnitConv { StixUnit as_is; StixUnit to_be; double conv; };

/* Most conversion constants from NIST guide to SI units.  Constants
 * for 180/pi and pi/180n from Dwight, Integrals and other Math Data,
 * 4th ed.
 */
static struct StixUnitConv  conv_consts[] = {

    /* length measures -- compute the conversion factors from metric
     * to english as an expression using the exact 2.54 factor so that 
     * we get the maximum precision from the representation.   
     */
    { stixunit_mm,	stixunit_cm,	0.1 },
    { stixunit_mm, 	stixunit_m, 	0.001 },
    { stixunit_mm, 	stixunit_in, 	1/25.4 },	/* ~0.03937008 */
    { stixunit_mm, 	stixunit_ft, 	1/(25.4*12) },	/* ~0.003280840 */

    { stixunit_cm,	stixunit_mm,	10 },
    { stixunit_cm, 	stixunit_m, 	0.01 },
    { stixunit_cm, 	stixunit_in, 	1/2.54 },	/* ~0.3937008 */
    { stixunit_cm, 	stixunit_ft, 	1/(2.54*12) },	/* ~0.03280840 */

    { stixunit_m,	stixunit_mm,	1000 },
    { stixunit_m, 	stixunit_cm, 	100 },
    { stixunit_m, 	stixunit_in, 	100/2.54 },	 
    { stixunit_m, 	stixunit_ft, 	100/(2.54*12) }, /* ~3.280840 */

    { stixunit_in,	stixunit_mm,	25.4 },
    { stixunit_in, 	stixunit_cm, 	2.54 },
    { stixunit_in,	stixunit_m,	0.0254 },
    { stixunit_in, 	stixunit_ft, 	1.0/12.0 },	/* ~0.08333333 */

    { stixunit_ft,	stixunit_mm,	304.8 },
    { stixunit_ft, 	stixunit_cm, 	30.48 },
    { stixunit_ft,	stixunit_m,	0.3048 },
    { stixunit_ft, 	stixunit_in, 	12 },

    /* plane angle */
    { stixunit_rad,	stixunit_deg,	57.29577951 },	 /* 180/pi */
    { stixunit_deg,	stixunit_rad,	0.01745329252 }, /* pi/180 */

    /* time -- only one unit */

    /* linear speed */
    { stixunit_mmps,	stixunit_cmps,	0.1 },
    { stixunit_mmps, 	stixunit_mps, 	0.001 },
    { stixunit_mmps, 	stixunit_ips, 	1/25.4 },	/* ~0.03937008 */
    { stixunit_mmps, 	stixunit_fps, 	1/(25.4*12) },	/* ~0.003280840 */

    { stixunit_cmps,	stixunit_mmps,	10 },
    { stixunit_cmps, 	stixunit_mps, 	0.01 },
    { stixunit_cmps, 	stixunit_ips, 	1/2.54 },	/* ~0.3937008 */
    { stixunit_cmps, 	stixunit_fps, 	1/(2.54*12) },	/* ~0.03280840 */

    { stixunit_mps,	stixunit_mmps,	1000 },
    { stixunit_mps, 	stixunit_cmps, 	100 },
    { stixunit_mps,	stixunit_ips,	100/2.54 },
    { stixunit_mps, 	stixunit_fps, 	100/(2.54*12) },

    /* rotational speed */
    { stixunit_hertz,	stixunit_rpm,	1.0/60.0 },
    { stixunit_rpm, 	stixunit_hertz,	60 },

    /* pressure */
    { stixunit_pa, 	stixunit_kpa,	0.001 },
    { stixunit_pa, 	stixunit_mpa,	0.000001 },
    { stixunit_pa, 	stixunit_psi,	0.0001450377 },

    { stixunit_kpa, 	stixunit_pa,	1000 },
    { stixunit_kpa, 	stixunit_mpa,	0.001 },
    { stixunit_kpa, 	stixunit_psi,	0.1450377 },

    { stixunit_mpa, 	stixunit_pa,	1000000 },
    { stixunit_mpa, 	stixunit_kpa,	1000 },
    { stixunit_mpa, 	stixunit_psi,	145.0377 },

    { stixunit_psi, 	stixunit_pa,	6894.757 },
    { stixunit_psi, 	stixunit_kpa,	6.894757 },
    { stixunit_psi, 	stixunit_mpa,	0.006894757 },
};


void print_conversion_factors()
{
    unsigned i;
    
    printf ("UNIT Conversion Constants\n");
    for (i=0; i< (sizeof(conv_consts)/sizeof(StixUnitConv)); i++)
    {
	printf ("%s to %-5s\t ==> %.15g\n",
		get_unit_name(conv_consts[i].as_is),
		get_unit_name(conv_consts[i].to_be),
		conv_consts[i].conv);
    }
}


double convert_measure_units (double val, StixUnit as_is, StixUnit to_be)
{
    unsigned i;

    /* passthru, no conversion */
    if (!to_be || to_be == as_is || to_be == stixunit_as_is) 
	return val;

    /* search table for a conversion factor */
    for (i=0; i< (sizeof(conv_consts)/sizeof(StixUnitConv)); i++)
    {
	if (as_is != conv_consts[i].as_is) continue;
	if (to_be != conv_consts[i].to_be) continue;
	return val * conv_consts[i].conv;
    }

    /* undefined conversion */
    return 0;
}


double convert_measure_units (double val, unit * as_obj, StixUnit to_be)
{
    /* convert unknown units */
    /* short circuit for now */
    return val;
}



unit * make_measure_unit (StixUnit to_be)
{
    return 0;
}


/* --------------------------------------------------
 * Default Units -- These specify application preference for
 * measure values.  Numeric values will be unit converted as 
 * needed on get and step units will be set properly on put.
 */
static StixUnit	dflt_unit_angle    = stixunit_deg;
static StixUnit	dflt_unit_length   = stixunit_as_is;  /* no coversion */
static StixUnit	dflt_unit_pressure = stixunit_pa;
static StixUnit	dflt_unit_rotspeed = stixunit_hertz;
static StixUnit	dflt_unit_speed    = stixunit_mps;
static StixUnit	dflt_unit_time     = stixunit_sec;

StixUnit get_angle_default_unit() 	{ return dflt_unit_angle; }
StixUnit get_length_default_unit()  	{ return dflt_unit_length; }
StixUnit get_pressure_default_unit()  	{ return dflt_unit_pressure; }
StixUnit get_rotspeed_default_unit() 	{ return dflt_unit_rotspeed; }
StixUnit get_speed_default_unit()  	{ return dflt_unit_speed; }
StixUnit get_time_default_unit() 	{ return dflt_unit_time; }

StixErr put_angle_default_unit (StixUnit u) 
{ 
    if (u == stixunit_deg || u == stixunit_rad) {
	dflt_unit_angle = u; 
	return 0;
    }
    return 1;
}

StixErr put_length_default_unit (StixUnit u) 
{
    if (u == stixunit_mm || u == stixunit_cm || u == stixunit_m ||
	u == stixunit_in || u == stixunit_ft) {
	dflt_unit_length = u; 
	return 0;
    }
    return 1;
}

StixErr put_pressure_default_unit (StixUnit u) 
{ 
    if (u == stixunit_pa || u == stixunit_psi) {
	dflt_unit_pressure = u; 
	return 0;
    }
    return 1;
}

StixErr put_rotspeed_default_unit (StixUnit u)
{ 
    if (u == stixunit_hertz || u == stixunit_rpm) {
	dflt_unit_rotspeed = u; 
	return 0;
    }
    return 1;
}

StixErr put_speed_default_unit (StixUnit u)
{
    if (u == stixunit_mmps || u == stixunit_cmps || u == stixunit_mps ||
	u == stixunit_ips  || u == stixunit_fps) {
	dflt_unit_speed = u; 
	return 0;
    }
    return 1;
}

StixErr put_time_default_unit (StixUnit u)
{
    if (u == stixunit_sec) {
	dflt_unit_time = u; 
	return 0;
    }
    return 1;
}

/* --------------------------------------------------
 * Measure Value Functions
 */

representation_item * get_representation_item (
    representation * rep, 
    RoseDomain * targ_type, 
    const char * targ_name
    )
{
    SetOfrepresentation_item * its = (rep? rep->items(): 0);
    unsigned i, sz; 

    if (its) for (i=0, sz=its-> size(); i<sz; i++) 
    {
	representation_item * it = its->get(i);
	char * it_name = it->name();
	
	/* match by name */
	if (targ_name && (!it_name || strcmp (it_name, targ_name)))
	    continue;

	/* match by type */
	if (targ_type && !it-> isa(targ_type))
	    continue;

	return it;
    }
    return 0;
}


descriptive_representation_item * get_descriptive_rep_item (
    representation * rep, 
    const char *     targ
    )
{
    representation_item * it = get_representation_item 
	(rep, ROSE_DOMAIN(descriptive_representation_item), targ);

    return (it? ROSE_CAST(descriptive_representation_item,it): 0);
}


/* ----------------------------------------
 * Ratio
 */
double get_ratio_value (measure_with_unit * mwu)
{
    /* just return any double value, dont fret units */
    measure_value * mv = (mwu? mwu-> value_component(): 0);
    return (mv? mv-> getDouble(): 0);
}

double get_ratio_value (
    representation * rep, 
    const char *     targ
    )
{
    representation_item * it = get_representation_item 
	(rep, ROSE_DOMAIN(ratio_measure_with_unit), targ);

    measure_with_unit * mwu = (it? ROSE_CAST(measure_with_unit,it): 0);
    measure_value *     mv  = (mwu? mwu-> value_component(): 0);
    return (mv? mv-> getDouble(): 0);
}



StixErr	 put_ratio_value (measure_with_unit *, double)
{
    return 1;
}

measure_with_unit * get_ratio_rep_item (
    representation * rep, 
    const char *     targ
    )
{
    representation_item * it = get_representation_item 
	(rep, ROSE_DOMAIN(ratio_measure_with_unit), targ);

    return (it? ROSE_CAST(measure_with_unit,it): 0);
}



/* ----------------------------------------
 * Counts
 */
double get_count_value (measure_with_unit * mwu)
{
    measure_value * mv = (mwu? mwu-> value_component(): 0);

    /* Validation code can make sure this is a count_measure */
    return (mv? mv-> getDouble(): 0);
}

double get_count_value (
    representation * rep, 
    const char *     targ
    )
{
    representation_item * it = get_representation_item 
	(rep, ROSE_DOMAIN(measure_with_unit), targ);

    measure_with_unit * mwu = (it? ROSE_CAST(measure_with_unit,it): 0);
    measure_value *     mv  = (mwu? mwu-> value_component(): 0);

    /* Validation code can make sure this is a count_measure */
    return (mv? mv-> getDouble(): 0);
}

StixErr	 put_count_value (measure_with_unit *, double)
{
    return 1;
}

measure_with_unit * get_count_rep_item (
    representation * rep, 
    const char *     targ
    )
{
    representation_item * it = get_representation_item 
	(rep, ROSE_DOMAIN(measure_with_unit), targ);

    return (it? ROSE_CAST(measure_with_unit,it): 0);
}



/* ----------------------------------------
 * Length
 */
double get_length_value (measure_with_unit * mwu, StixUnit u)
{
    /* just return any double value, dont fret units */
    measure_value * mv = (mwu? mwu-> value_component(): 0);
    unit * 	    uc = (mwu? mwu-> unit_component(): 0);

    if (!mv) return 0;

    /* Validation code could check to make sure that this is one of
     * the proper select types 
     */
    double val = mv-> getDouble();

    /* do conversions to the desired units. */
    return convert_measure_units (val, uc, (u? u: dflt_unit_length));
}

double get_length_value (representation *r, const char *t, StixUnit u)
{
    return get_length_value(get_length_rep_item (r, t), u);
}


StixErr	 put_length_value (measure_with_unit *, double, StixUnit u)
{
    return 1;
}

measure_with_unit * get_length_rep_item (
    representation * rep, 
    const char *     targ
    )
{
    representation_item * it = get_representation_item 
	(rep, ROSE_DOMAIN(length_measure_with_unit), targ);

    return (it? ROSE_CAST(measure_with_unit,it): 0);
}

/* ----------------------------------------
 * Angle
 */
double get_angle_value (measure_with_unit * mwu, StixUnit u)
{
    /* just return any double value, dont fret units */
    measure_value * mv = (mwu? mwu-> value_component(): 0);
    unit * 	    uc = (mwu? mwu-> unit_component(): 0);

    if (!mv) return 0;

    /* Validation code could check to make sure that this is one of
     * the proper select types 
     */
    double val = mv-> getDouble();

    /* do conversions to the desired units. */
//    return convert_measure_units (val, uc, (u? u: dflt_unit_angle));

//  FIXME for now just cvt rad to deg
    return convert_measure_units (val, stixunit_rad, (u? u: dflt_unit_angle));
}

double get_angle_value (representation *r, const char *t, StixUnit u)
{
    return get_angle_value(get_angle_rep_item (r, t), u);
}


StixErr	 put_angle_value (measure_with_unit *, double, StixUnit u)
{
    return 1;
}


measure_with_unit * get_angle_rep_item (
    representation * rep, 
    const char *     targ)
{
    representation_item * it = get_representation_item 
	(rep, ROSE_DOMAIN(plane_angle_measure_with_unit), targ);

    return (it? ROSE_CAST(measure_with_unit,it): 0);
}


/* ----------------------------------------
 * Time
 */
double get_time_value (measure_with_unit * mwu, StixUnit u)
{
    /* just return any double value, dont fret units */
    measure_value * mv = (mwu? mwu-> value_component(): 0);
    unit * 	    uc = (mwu? mwu-> unit_component(): 0);

    if (!mv) return 0;

    /* Validation code could check to make sure that this is one of
     * the proper select types 
     */
    double val = mv-> getDouble();

    /* Since we only have one time unit, this is not strictly
     * required, but we may as well put the infrastructure in 
     * place in case it is needed in the future
     */
    /* do conversions to the desired units. */
    return convert_measure_units (val, uc, (u? u: dflt_unit_time));
}

double get_time_value (representation *r, const char *t, StixUnit u)
{
    return get_time_value(get_time_rep_item (r, t), u);
}



StixErr	 put_time_value (measure_with_unit *, double, StixUnit u)
{
    return 1;
}

measure_with_unit * get_time_rep_item (
    representation * rep, 
    const char *     targ
    )
{
    representation_item * it = get_representation_item 
	(rep, ROSE_DOMAIN(time_measure_with_unit), targ);

    return (it? ROSE_CAST(measure_with_unit,it): 0);
}




/* ----------------------------------------
 * Speed
 */
double get_speed_value (measure_with_unit * mwu, StixUnit u)
{
    /* just return any double value, dont fret units */
    measure_value * mv = (mwu? mwu-> value_component(): 0);
    unit * 	    uc = (mwu? mwu-> unit_component(): 0);

    if (!mv) return 0;

    /* Validation code could check to make sure that this is one of
     * the proper select types 
     */
    double val = mv-> getDouble();

    /* do conversions to the desired units. */
    return convert_measure_units (val, uc, (u? u: dflt_unit_speed));
}

double get_speed_value (representation *r, const char *t, StixUnit u)
{
    return get_speed_value(get_speed_rep_item (r, t), u);
}




StixErr	 put_speed_value (measure_with_unit *, double, StixUnit u)
{
    return 1;
}

measure_with_unit * get_speed_rep_item (
    representation * rep, 
    const char *     targ)
{
    /* no specific measure type for speeds */
    representation_item * it = get_representation_item 
	(rep, ROSE_DOMAIN(measure_with_unit), targ);

    return (it? ROSE_CAST(measure_with_unit,it): 0);
}



/* ----------------------------------------
 * Rotational Speed
 */
double get_rotspeed_value (measure_with_unit * mwu, StixUnit u)
{
    /* just return any double value, dont fret units */
    measure_value * mv = (mwu? mwu-> value_component(): 0);
    unit * 	    uc = (mwu? mwu-> unit_component(): 0);

    if (!mv) return 0;

    /* Validation code could check to make sure that this is one of
     * the proper select types 
     */
    double val = mv-> getDouble();

    /* do conversions to the desired units. */
    return convert_measure_units (val, uc, (u? u: dflt_unit_rotspeed));
}

double get_rotspeed_value (representation *r, const char *t, StixUnit u)
{
    return get_rotspeed_value(get_rotspeed_rep_item (r, t), u);
}


StixErr	 put_rotspeed_value (measure_with_unit *, double, StixUnit u)
{
    return 1;
}

measure_with_unit * get_rotspeed_rep_item (
    representation * rep, 
    const char *     targ)
{
    /* no specific measure type for rotational  speeds */
    representation_item * it = get_representation_item 
	(rep, ROSE_DOMAIN(measure_with_unit), targ);

    return (it? ROSE_CAST(measure_with_unit,it): 0);
}


/* ----------------------------------------
 * Pressure 
 */
double get_pressure_value (measure_with_unit * mwu, StixUnit u)
{
    /* just return any double value, dont fret units */
    measure_value * mv = (mwu? mwu-> value_component(): 0);
    unit * 	    uc = (mwu? mwu-> unit_component(): 0);

    if (!mv) return 0;

    /* Validation code could check to make sure that this is one of
     * the proper select types 
     */
    double val = mv-> getDouble();

    /* do conversions to the desired units. */
    return convert_measure_units (val, uc, (u? u: dflt_unit_pressure));
}

double get_pressure_value (representation *r, const char *t, StixUnit u)
{
    return get_pressure_value(get_pressure_rep_item (r, t), u);
}


measure_with_unit * get_pressure_rep_item (
    representation * rep, 
    const char *     targ)
{
    /* no specific measure type for pressures */
    representation_item * it = get_representation_item 
	(rep, ROSE_DOMAIN(measure_with_unit), targ);

    return (it? ROSE_CAST(measure_with_unit,it): 0);
}


StixErr	 put_pressure_value (measure_with_unit *, double, StixUnit u)
{
    return 1;
}




/* ----------------------------------------
 * Pressure 
 */
int get_boolean_value (
    const char * true_sym,
    const char * false_sym,
    representation * rep, 
    const char * targ
    )
{
    representation_item * it = get_representation_item 
	(rep, ROSE_DOMAIN(descriptive_representation_item), targ);

    if (!it) return 0;

    descriptive_representation_item * dm = 
	ROSE_CAST(descriptive_representation_item,it);

    char * desc = dm->description();
    if (desc) {
	if (true_sym && !strcmp(desc, true_sym))
	    return 1;

	/* really just for validation */
	if (false_sym && !strcmp(desc, false_sym))
	    return 0;
    }

    return 0;     /* error case, null or unknown string */
}
