/* $RCSfile: report.cxx,v $
 * $Revision: 1.25 $ $Date: 2003/06/09 16:12:05 $
 * Auth: Dave Loffredo (loffredo@steptools.com)
 * 
 * Copyright (c) 1991-2003 by STEP Tools Inc. 
 * All Rights Reserved.
 * 
 * This file is part of the STIX Library
 * 
 * This file may be distributed and/or modified under the terms of 
 * the GNU General Public License version 2 as published by the Free
 * Software Foundation and appearing in the file LICENSE.GPL included
 * with this file.
 * 
 * THIS FILE IS PROVIDED "AS IS" WITH NO WARRANTY OF ANY KIND,
 * INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * STEP Tools Inc. also offers licenses for commercial use of this
 * software in non-GPL applications.  Contact info@steptools.com for
 * terms and pricing.
 * 
 * 		----------------------------------------
 *
 *	Report Writer
 */


#include <rose.h>
#include <integrated_cnc_schema.h>
#include <integrated_cnc_schema_ROSE_LOAD.h>

#include "stix_imp.h"

char* printable_name (action_method * am)
{
    if (!am) return "<null action>";

    char * name = am-> name();
    if (!name || !*name) name = am-> description();
    if (!name || !*name) name = "<no-name>";
    return name;
}    

char* printable_name (product * p)
{
    if (!p) return "<null product>";

    char * name = p-> id();
    if (!name || !*name) name = p-> name();
    if (!name || !*name) name = p-> description();
    if (!name || !*name) name = "<no-name>";
    return name;
}    

char* printable_name (product_definition_formation * pdf)
{
    if (!pdf) return "<null pdf>";

    char * name = pdf-> id();
    if (!name || !*name) name = pdf-> description();
    if (!name || !*name) name = "";
    return name;
}    

char* printable_name (product_definition * pd)
{
    if (!pd) return "<null pdef>";

    char * name = pd-> id();
    if (!name || !*name) name = pd-> description();
    if (!name || !*name) name = "";
    return name;
}


void dump_workpiece_detail (
    product_definition * pd, StixIndent* tabs
    )
{
    StixIndent dflt_tabs;
    if (!tabs) tabs = &dflt_tabs;

    StixWorkpiece * idx = get_workpiece_idx (pd);

    printf ("name = %s %s %s (id #%d) \n",
	    printable_name(pd->formation()->of_product()),
	    printable_name(pd->formation()),
	    printable_name(pd),
	    pd->entity_id());

    tabs->inc();

    shape_representation * shape = get_workpiece_shape (pd);
    if (shape) {
	tabs->print();
	printf ("shape = %s (id #%d)\n",
		shape->domain()->name(),
		shape->entity_id());

	tabs->print();
	printf ("shape length unit = %s\n",
		get_unit_name(get_length_unit(shape)));
    }


    if (idx-> m_bbox) {
	tabs->print();
	printf ("bounding box = %s (id #%d)\n",
		idx-> m_bbox->domain()->name(),
		idx-> m_bbox->entity_id());

	tabs->print();
	printf ("bounding box length unit = %s\n",
		get_unit_name(get_length_unit(idx-> m_bbox)));
    }


    product_definition * stk = get_workpiece_stock (pd);
    if (stk) {
	StixWorkpiece * sx = get_workpiece_idx (stk);

	tabs->print();
	printf ("stock = %s %s %s (id #%d)\n",
		printable_name(stk->formation()->of_product()),
		printable_name(stk->formation()),
		printable_name(stk),
		stk->entity_id());

	tabs->print();
	if (!sx || !sx-> m_shape)
	    printf ("stock not recognized as a workpiece\n");
	else {
	    printf ("stock length unit = %s\n",
		    get_unit_name(get_length_unit(sx-> m_shape)));
	}
		
	tabs->print();
	printf ("stock min/max X = %g / %g  (%g)\n",
		get_workpiece_stock_min_x (pd),
		get_workpiece_stock_max_x (pd),
		get_workpiece_stock_max_x (pd) -
		get_workpiece_stock_min_x (pd));
	tabs->print();
	printf ("stock min/max Y = %g / %g  (%g)\n",
		get_workpiece_stock_min_y (pd),
		get_workpiece_stock_max_y (pd),
		get_workpiece_stock_max_y (pd) -
		get_workpiece_stock_min_y (pd));
	tabs->print();
	printf ("stock min/max Z = %g / %g  (%g)\n",
		get_workpiece_stock_min_z (pd),
		get_workpiece_stock_max_z (pd),
		get_workpiece_stock_max_z (pd) - 
		get_workpiece_stock_min_z (pd));

    }


    if (idx->m_bblimits) {
	tabs->print();
	printf ("bounding box min/max X = %g / %g  (%g)\n",
		idx->m_bblimits->m_min.m_x,
		idx->m_bblimits->m_max.m_x,
		idx->m_bblimits->m_max.m_x -
		idx->m_bblimits->m_min.m_x);
	tabs->print();
	printf ("bounding box min/max Y = %g / %g  (%g)\n",
		idx->m_bblimits->m_min.m_y,
		idx->m_bblimits->m_max.m_y,
		idx->m_bblimits->m_max.m_y -
		idx->m_bblimits->m_min.m_y);
	tabs->print();
	printf ("bounding box min/max Z = %g / %g  (%g)\n",
		idx->m_bblimits->m_min.m_z,
		idx->m_bblimits->m_max.m_z,
		idx->m_bblimits->m_max.m_z -
		idx->m_bblimits->m_min.m_z);
    }

    tabs->dec();
}




void dump_workplan_detail (
    machining_workplan * wp, StixIndent* tabs
    )
{
    StixIndent dflt_tabs;
    if (!tabs) tabs = &dflt_tabs;

    unsigned i,sz;

    if (!wp) {
	printf ("No workplan\n");
	return;
    }

    printf ("workplan %s (id #%d) \n",
	    printable_name(wp),
	    wp->entity_id());

    tabs->inc();
    tabs->print();
    printf ("%d elements \n", wp->m_steps.size());

    tabs->print();
    if (wp->m_setup)
	printf ("setup = %s %s %s (id #%d)\n",
		printable_name(wp->m_setup->formation()->of_product()),
		printable_name(wp->m_setup->formation()),
		printable_name(wp->m_setup),
		wp->m_setup->entity_id());
    else
	printf ("setup = none\n");

    StixWorkplanElementCursor substeps(wp);
    for (i=0, sz=substeps.size(); i<sz; i++) {
	machining_process_executable * mpe = substeps.get(i);

	/* remember - sequence position is a float */
	tabs->print();
	printf ("[%d - %.15g] ", i, substeps.get_sequence_position(i));

	/* Eventually look at using the type code to detect what 
	 * type of executable it is, but for now just do it the old
	 * fashioned way with casts.  This was written before any
	 * of the type code stuff was put in.
	 */
	if (mpe-> isa(ROSE_DOMAIN(machining_workplan)))
	    dump_workplan_detail (ROSE_CAST(machining_workplan,mpe), tabs);

	else if (mpe-> isa(ROSE_DOMAIN(machining_workingstep)))
	    dump_workingstep_detail (ROSE_CAST(machining_workingstep,mpe), tabs);
	else {
	    printf ("%s (id #%d)\n", mpe->domain()->name(), 
		    mpe->entity_id());

	    tabs->print();
	    printf ("name = %s\n", 
		    (mpe->name()?mpe->name():"none"));

	    tabs->print();
	    printf ("desc = %s\n", 
		    (mpe->description()?mpe->description():"none"));
	}
    }

    tabs->dec();
}


void dump_workplan_all_tools (
    machining_workplan * wp, StixIndent* tabs
    )
{
    StixIndent dflt_tabs;
    StixWorkplanAllToolCursor objs (wp);
    machining_tool * mt;

    if (!tabs) tabs = &dflt_tabs;

    tabs->print();
    printf ("workplan tool requirements:\n");

    tabs->inc();
    if (!objs.size()) {
	tabs->print();
	printf ("NO TOOLS REQUIRED\n");
    }
    else while (mt = objs.next()) {
	printf ("tool #%d (type: %s)\n", mt->entity_id(),
		(mt->description()? mt->description(): "<none>"));
    }
    tabs->dec();
}


void dump_workplan_all_feats (
    machining_workplan * wp, StixIndent* tabs
    )
{
    StixIndent dflt_tabs;
    StixWorkplanAllFeatureCursor objs (wp);
    shape_aspect * sa;

    if (!tabs) tabs = &dflt_tabs;

    tabs->print();
    printf ("workplan complete feature set:\n");

    tabs->inc();
    if (!objs.size()) {
	tabs->print();
	printf ("NO FEATURES\n");
    }
    else while (sa = objs.next()) {
	printf ("feature #%d (type: %s)\n", sa->entity_id(),
		get_aspect_typename(sa));
    }
    tabs->dec();
}


void dump_action_props (
    action_method * am, StixIndent* tabs
    )
{
    StixIndent dflt_tabs;
    if (!tabs) tabs = &dflt_tabs;

    action_property_representation * apr;
    action_property * ap;

    if (!am || !am->m_props) return;
    apr = am->m_props;

    while (apr) {
	ap = apr-> property();
	if (ap && ap-> name() && *(ap-> name())) {
	    tabs->print();
	    printf ("property = %s\n", ap-> name());
	}
	apr = apr-> m_next_apr;
    }
}


void dump_operation_detail (
    machining_operation * op, StixIndent* tabs
    )
{
    StixIndent dflt_tabs;
    if (!tabs) tabs = &dflt_tabs;

    if (!op) {
	printf ("operation type = NONE!\n");
	return;
    }
    if (op->isa (ROSE_DOMAIN(drilling_type_operation)))
	printf ("operation type = Drilling (%s)\n", 
		op->domain()-> name());
    else if (op->isa (ROSE_DOMAIN(milling_type_operation)))
	printf ("operation type = Milling (%s)\n", 
		op->domain()-> name());
    else
	printf ("operation type = Other (%s)\n", 
		op->domain()-> name());
    tabs->inc();
    dump_action_props (op, tabs);
    tabs->dec();

    tabs->print();
    printf ("tool: ");
    dump_tool_detail (op->m_tool, tabs);


    machining_technology * tech = get_operation_technology (op);
    if (tech) {
	tabs->print();
	printf ("technology (#%d)\n", tech->entity_id());
	dump_action_props (tech, tabs);
    }

    machining_functions * funs = get_operation_functions (op);
    if (funs) {
	tabs->print();
	printf ("functions (#%d)\n", funs->entity_id());
	dump_action_props (funs, tabs);
    }

    machining_strategy * strat = get_operation_strategy (op);
    if (strat) {
	tabs->print();
	printf ("strategy (#%d)\n", strat->entity_id());
	dump_action_props (strat, tabs);
    }

    machining_approach_retract_strategy * astrat =
	get_milling_operation_approach_strategy (op);

    if (astrat) {
	tabs->print();
	printf ("approach (#%d)\n", astrat->entity_id());
	dump_action_props (astrat, tabs);
    }

    machining_approach_retract_strategy * rstrat =
	get_milling_operation_retract_strategy (op);

    if (rstrat) {
	tabs->print();
	printf ("retract (#%d)\n", rstrat->entity_id());
	dump_action_props (rstrat, tabs);
    }
}




void dump_workingstep_detail (
    machining_workingstep * ws, StixIndent* tabs
    )
{
    StixIndent dflt_tabs;
    if (!tabs) tabs = &dflt_tabs;

    if (!ws) 
	printf ("<null workingstep>\n");
    else
	/* id, name */
	printf ("workingstep %s (#%d)\n", 
		printable_name (ws), 
		ws-> entity_id());

    /* parent_name */
    tabs->inc();
    tabs->print();
    printf ("type = %s\n", get_exec_typename (ws));

    tabs->print();
    printf ("parent = %s (#%d)\n", 
	    printable_name (ws-> m_parent), 
	    (ws-> m_parent? ws-> m_parent-> entity_id(): 0));

    /* description */
    tabs->print();
    printf ("description = %s\n", 
	    (ws-> description()? ws-> description(): "none"));

    /* tipe */
    tabs->print();
    dump_operation_detail(ws->m_op, tabs);


    /* in process feature */
    tabs->print();
    if (!ws->m_inproc)
	printf ("in-process feature = none, use final feature\n");
    else {
	shape_aspect * sa = ws-> m_inproc->m_feature;
	if (!sa) 
	    printf ("in-process feature = DISCONNECTED!\n");
	else 
	    printf ("in-process feature = %s (#%d)\n", 
		    get_aspect_typename(sa), sa-> entity_id());
    }
	    

    /* final feature */
    tabs->print();
    if (!ws->m_final)
	printf ("final feature = none\n");
    else {
	shape_aspect * sa = ws-> m_final->m_feature;
	if (!sa) 
	    printf ("final feature = DISCONNECTED!\n");
	else 
	    printf ("final feature = %s (#%d)\n", 
		    get_aspect_typename(sa), sa-> entity_id());
    }
    tabs->dec();
}



void dump_feature_detail (
    shape_aspect * sa, StixIndent* tabs
    )
{
    StixIndent dflt_tabs;
    if (!tabs) tabs = &dflt_tabs;

    unsigned i, sz;
    StixFeature * idx = get_feature_idx (sa);

    if (!idx) {
	printf ("<null feature>\n");
	return;
    }

    printf ("%s (id #%d) \n", 
	    get_aspect_typename(sa),
	    sa->entity_id());

    tabs->inc();

    tabs->print();
    printf ("feature final: ");
    { 
	shape_aspect * ff = get_feature_final(sa);
	if (!ff) printf ("itself\n");
	else     printf ("#%d\n", ff-> entity_id());
    }

    tabs->print();
    printf ("feature faces: ");
    {
	StixFeatureFaceCursor faces (sa);
	advanced_face * f;

	if (!faces.size()) printf ("NONE\n");
	else while (f=faces.next()) {
	    printf ("#%d ", f-> entity_id());
	}
	printf ("\n");
    }
    
    for (i=0, sz=idx->m_component_sars.size(); i<sz; i++) {
	shape_aspect_relationship * sar = 
	    (shape_aspect_relationship *) idx->m_component_sars[i];

	shape_aspect * ing = sar-> relating_shape_aspect();
	shape_aspect * ed = sar-> related_shape_aspect();
	char * s;
	
	    tabs->print();
	    printf ("sar %d", i);

	    s = sar->name();
	    if (s && *s) printf (" name=[%s]", s);

	    s = sar->description();
	    if (s && *s) printf (" desc=[%s]", s);
	    printf ("\n");

	    tabs->print();
	    printf ("   %s", get_aspect_typename(ing));
	    s = ing->description();
	    if (s && *s) printf (" desc=[%s]", s);

	    s = ed->description();
	    if (s && *s) printf (" ed_desc=[%s]", s);
	    printf ("\n");
    }

    tabs->print();
    printf ("feature depth: %g \n", calc_feature_depth(sa));

    StixMtrx ori = calc_feature_orientation(sa);
    tabs->print();
    printf ("feature orientation: \n");
    ori.print_axes (stdout, tabs->string());

    tabs->dec();
}


void dump_tool_detail (
    machining_tool * mt, StixIndent* tabs
    )
{
    StixIndent dflt_tabs;
    if (!tabs) tabs = &dflt_tabs;

    char * lu_name = get_unit_name(get_length_default_unit());
    char * au_name = get_unit_name(get_angle_default_unit());

    if (!mt) {
	printf ("<missing tool>\n");
	return;
    }

    printf ("type: %s (id #%d) \n", 
	    (mt->description()? mt->description(): "<none>"),
	    mt->entity_id());

    tabs->inc();
    tabs->print();
    printf ("name: %s\n", (mt->name()? mt->name(): "<none>"));

    tabs->print();
    printf ("diameter: %g %s\n", 
	    get_tool_diameter(mt), lu_name);

    tabs->print();
    printf ("edge (flute) length: %g %s\n", 
	    get_tool_edge_length(mt), lu_name);

    tabs->print();
    printf ("edge radius: %g %s\n", 
	    get_tool_edge_radius(mt), lu_name);

    tabs->print();
    printf ("circumference angle: %g %s\n", 
	    get_tool_circumference_angle(mt), au_name);
    tabs->dec();
}
