/* $RCSfile: stix.h,v $
 * $Revision: 1.9 $ $Date: 2003/08/20 20:04:55 $
 * Auth: Dave Loffredo (loffredo@steptools.com)
 * 
 * Copyright (c) 1991-2003 by STEP Tools Inc. 
 * All Rights Reserved.
 * 
 * This file is part of the STIX Library
 * 
 * This file may be distributed and/or modified under the terms of 
 * the GNU General Public License version 2 as published by the Free
 * Software Foundation and appearing in the file LICENSE.GPL included
 * with this file.
 * 
 * THIS FILE IS PROVIDED "AS IS" WITH NO WARRANTY OF ANY KIND,
 * INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * STEP Tools Inc. also offers licenses for commercial use of this
 * software in non-GPL applications.  Contact info@steptools.com for
 * terms and pricing.
 * 
 * 		----------------------------------------
 */

#ifndef	STIX_H
#define	STIX_H

/* Use workarounds for some issues in the ST-Plan output.  Remove
 * these once the output code has been corrected.
 */
#define BUG_BOOLEAN_HAS_WRONG_VALUES
#define BUG_BOSSES_RELATED_INCORRECTLY
#define BUG_PATH_SHAPE_REP_NOT_USED

typedef unsigned StixID;

/* FORWARD DECLARATIONS - All of the API functions just pass around
 * pointers to AIM classes and internal structures.  We don't need to
 * have the full class declarations for most uses, so just forward
 * declare them.  Cxx files that do AIM things can pull in all of the
 * class headers, but other things do not need to.
 */
class StixNC;

/* forward declarations for internal index structures */
class StixProject;	/* defined in stix_imp.h */
class StixSetup;	/* defined in stix_imp.h */
class StixWorkpiece;	/* defined in stix_imp.h */
class StixFeature;	/* defined in stix_imp.h */
class StixComponent;	/* defined in stix_imp.h */
class StixShapeLimit;
class StixMaterial;
class StixProperty;


/* The schema manager and associated support functions let us read
 * designs with different schemas.  This makes it a little easier to 
 * import different schemas.
 */
enum StixSchemaType {
    StixSchema_other, 
    StixSchema_ap203, 
    StixSchema_ap214, 
    StixSchema_ap224, 
    StixSchema_ap238, 
};


/* forward declarations for AIM C++ Classes */
/* Not all of these are needed or used by the API functions, but I
 * just pulled out everything with a grep | uniq and didn't feel like
 * trimming things down.  In any case, it's still better than pulling
 * in headers for each of these!
 */
class Address;
class ListOfListOfDouble;
class ListOfListOfcartesian_point;
class ListOfListOfsurface_patch;
class ListOfcartesian_point;
class ListOfcomposite_curve_segment;
class ListOfgeneric_expression;
class ListOforiented_edge;
class ListOfpcurve_or_surface;
class ListOfrepresentation_item;
class ListOfreversible_topology_item;
class SetOfaction_resource;
class SetOfapproval_item;
class SetOfboundary_curve;
class SetOfcharacterized_action_definition;
class SetOfcharacterized_definition;
class SetOfdate_and_time_item;
class SetOfdatum_reference;
class SetOfderived_unit_element;
class SetOfdocument;
class SetOfdocument_reference_item;
class SetOfeffectivity_item;
class SetOfface;
class SetOfface_bound;
class SetOfgeometric_set_select;
class SetOfgeometric_tolerance;
class SetOfgroup_item;
class SetOforganization;
class SetOforiented_closed_shell;
class SetOfperson;
class SetOfperson_and_organization_item;
class SetOfproduct;
class SetOfproduct_context;
class SetOfproperty_definition_representation;
class SetOfrepresentation_item;
class SetOfreversible_topology_item;
class SetOfsecurity_classification_item;
class SetOfshape_aspect;
class SetOfshell;
class SetOfsupported_item;
class SetOftrimming_select;
class SetOfuncertainty_measure_with_unit;
class SetOfunit;
class SetOfvalue_qualifier;
class action;
class action_method;
class action_method_relationship;
class action_property;
class action_property_representation;
class action_resource;
class action_resource_relationship;
class action_resource_requirement;
class action_resource_type;
class advanced_brep_shape_representation;
class advanced_face;
class and_expression;
class angular_location;
class angular_size;
class angularity_tolerance;
class apex;
class application_context;
class application_context_element;
class application_protocol_definition;
class applied_approval_assignment;
class applied_area;
class applied_date_and_time_assignment;
class applied_document_reference;
class applied_document_usage_constraint_assignment;
class applied_effectivity_assignment;
class applied_group_assignment;
class applied_person_and_organization_assignment;
class applied_security_classification_assignment;
class approval;
class approval_assignment;
class approval_date_time;
class approval_item;
class approval_person_organization;
class approval_role;
class approval_status;
class assembly_component_usage;
class attribute_type;
class axis1_placement;
class axis2_placement;
class axis2_placement_2d;
class axis2_placement_3d;
class b_spline_curve;
class b_spline_curve_with_knots;
class b_spline_curve_with_knots_and_rational_b_spline_curve;
class b_spline_surface;
class b_spline_surface_with_knots;
class b_spline_surface_with_knots_and_rational_b_spline_surface;
class back_boring_operation;
class bezier_curve;
class bezier_curve_and_rational_b_spline_curve;
class bezier_surface;
class bezier_surface_and_rational_b_spline_surface;
class binary_boolean_expression;
class binary_generic_expression;
class block;
class block_shape_representation;
class boolean_expression;
class boolean_operand;
class boring_operation;
class boss;
class boss_and_instanced_feature;
class boss_top;
class bottom_and_side_milling_operation;
class boundary_curve;
class bounded_curve;
class bounded_curve_and_surface_curve;
class bounded_pcurve;
class bounded_surface;
class bounded_surface_curve;
class brep_with_voids;
class calendar_date;
class cartesian_point;
class cartesian_transformation_operator;
class cartesian_transformation_operator_3d;
class centre_of_symmetry;
class chamfer;
class chamfer_offset;
class characterized_action_definition;
class characterized_definition;
class characterized_material_property;
class characterized_object;
class characterized_product_definition;
class characterized_resource_definition;
class circle;
class circular_closed_profile;
class circular_pattern;
class circular_pattern_and_instanced_feature;
class circular_runout_tolerance;
class classification_role;
class closed_path_profile;
class closed_shell;
class coaxiality_tolerance;
class common_datum;
class comparison_equal;
class comparison_expression;
class comparison_greater;
class comparison_greater_equal;
class comparison_less;
class comparison_less_equal;
class comparison_not_equal;
class composite_curve;
class composite_curve_on_surface;
class composite_curve_segment;
class composite_hole;
class composite_hole_and_instanced_feature;
class composite_shape_aspect;
class compound_feature;
class compound_feature_and_instanced_feature;
class compound_item_definition;
class compound_representation_item;
class concentricity_tolerance;
class concurrent_action_method;
class configuration_design_item;
class conic;
class conical_surface;
class connected_face_set;
class context_dependent_shape_representation;
class context_dependent_unit;
class conversion_based_unit;
class conversion_based_unit_and_length_unit;
class conversion_based_unit_and_plane_angle_unit;
class coordinated_universal_time_offset;
class csg_primitive;
class csg_select;
class curve;
class curve_bounded_surface;
class curve_on_surface;
class curve_replica;
class cylindrical_shape_representation;
class cylindrical_surface;
class cylindricity_tolerance;
class data_environment;
class date;
class date_and_time;
class date_and_time_assignment;
class date_and_time_item;
class date_time_or_event_occurrence;
class date_time_role;
class date_time_select;
class dated_effectivity;
class datum;
class datum_feature;
class datum_reference;
class datum_target;
class definitional_representation;
class definitional_representation_and_shape_representation;
class degenerate_pcurve;
class degenerate_toroidal_surface;
class derived_property_select;
class derived_shape_aspect;
class derived_unit;
class derived_unit_element;
class description_attribute;
class description_attribute_select;
class descriptive_representation_item;
class dimension_related_tolerance_zone_element;
class dimensional_characteristic;
class dimensional_characteristic_representation;
class dimensional_exponents;
class dimensional_location;
class dimensional_location_with_path;
class dimensional_size;
class dimensional_size_with_path;
class direction;
class direction_shape_representation;
class document;
class document_file;
class document_reference;
class document_reference_item;
class document_representation_type;
class document_type;
class document_usage_constraint;
class document_usage_constraint_assignment;
class document_usage_role;
class drilling_operation;
class drilling_type_operation;
class drilling_type_strategy;
class edge;
class edge_curve;
class edge_loop;
class edge_round;
class effectivity;
class effectivity_assignment;
class effectivity_item;
class elementary_surface;
class ellipse;
class environment;
class evaluated_degenerate_pcurve;
class expanded_uncertainty;
class expression;
class expression_representation_item;
class extension;
class external_source;
class externally_defined_feature_definition;
class externally_defined_feature_definition_and_instanced_feature;
class externally_defined_item;
class face;
class face_bound;
class face_outer_bound;
class face_shape_representation;
class face_surface;
class feature_component_definition;
class feature_component_relationship;
class feature_definition;
class feature_pattern;
class feature_pattern_and_instanced_feature;
class fillet;
class flat_face;
class flat_face_and_instanced_feature;
class flatness_tolerance;
class founded_item;
class founded_item_select;
class freeform_milling_operation;
class freeform_milling_strategy;
class freeform_milling_tolerance_representation;
class functionally_defined_transformation;
class generic_expression;
class generic_literal;
class generic_variable;
class geometric_alignment;
class geometric_intersection;
class geometric_representation_context;
class geometric_representation_context_and_global_uncertainty_assigned_context_and_global_unit_assigned_context;
class geometric_representation_context_and_global_unit_assigned_context;
class geometric_representation_context_and_parametric_representation_context;
class geometric_representation_item;
class geometric_set;
class geometric_set_select;
class geometric_tolerance;
class geometric_tolerance_relationship;
class geometric_tolerance_with_datum_reference;
class geometric_tolerance_with_defined_unit;
class geometrically_bounded_surface_shape_representation;
class global_uncertainty_assigned_context;
class global_unit_assigned_context;
class group;
class group_assignment;
class group_item;
class group_relationship;
class hole_bottom;
class hyperbola;
class id_attribute;
class id_attribute_select;
class instanced_feature;
class instanced_feature_and_outside_profile;
class instanced_feature_and_pocket;
class instanced_feature_and_rectangular_pattern;
class instanced_feature_and_removal_volume;
class instanced_feature_and_round_hole;
class instanced_feature_and_rounded_end;
class instanced_feature_and_slot;
class instanced_feature_and_spherical_cap;
class instanced_feature_and_step;
class instanced_feature_and_thread;
class int_literal;
class int_numeric_variable;
class intersection_curve;
class item_defined_transformation;
class length_measure_with_unit;
class length_measure_with_unit_and_measure_representation_item;
class length_measure_with_unit_and_measure_representation_item_and_qualified_representation_item;
class length_unit;
class length_unit_and_si_unit;
class limits_and_fits;
class line;
class line_profile_tolerance;
class linear_profile;
class literal_number;
class local_time;
class location_shape_representation;
class loop;
class lot_effectivity;
class machining_adaptive_control_strategy;
class machining_approach_retract_strategy;
class machining_cutting_component;
class machining_feature_process;
class machining_functions;
class machining_nc_function;
class machining_offset_vector_representation;
class machining_operation;
class machining_operator_instruction;
class machining_process_executable;
class machining_process_model;
class machining_rapid_movement;
class machining_strategy;
class machining_technology;
class machining_tool;
class machining_tool_body_representation;
class machining_tool_dimension_representation;
class machining_tool_direction_representation;
class machining_toolpath;
class machining_toolpath_speed_profile_representation;
class machining_touch_probing;
class machining_workingstep;
class machining_workplan;
class make_from_usage_option;
class manifold_solid_brep;
class manifold_surface_shape_representation;
class mapped_item;
class marking;
class material_designation;
class material_designation_characterization;
class material_property;
class material_property_representation;
class measure_qualification;
class measure_representation_item;
class measure_representation_item_and_plane_angle_measure_with_unit;
class measure_representation_item_and_ratio_measure_with_unit;
class measure_representation_item_and_time_measure_with_unit;
class measure_value;
class measure_with_unit;
class milling_type_operation;
class milling_type_strategy;
class modified_geometric_tolerance;
class modified_pattern;
class multiple_arity_boolean_expression;
class multiple_arity_generic_expression;
class name_attribute;
class name_attribute_select;
class named_unit;
class next_assembly_usage_occurrence;
class ngon_closed_profile;
class ngon_shape_representation;
class not_expression;
class numeric_expression;
class numeric_variable;
class object_role;
class offset_curve_3d;
class offset_surface;
class open_path_profile;
class open_shell;
class or_expression;
class ordinal_date;
class organization;
class organizational_address;
class oriented_closed_shell;
class oriented_edge;
class oriented_face;
class oriented_open_shell;
class oriented_path;
class outer_boundary_curve;
class outer_round;
class outside_profile;
class parabola;
class parallel_offset;
class parallelism_tolerance;
class parametric_representation_context;
class partial_circular_profile;
class path;
class path_feature_component;
class path_shape_representation;
class pattern_offset_membership;
class pattern_omit_membership;
class pcurve;
class pcurve_or_surface;
class perpendicular_to;
class perpendicularity_tolerance;
class person;
class person_and_organization;
class person_and_organization_assignment;
class person_and_organization_item;
class person_and_organization_role;
class person_organization_select;
class personal_address;
class placed_datum_target_feature;
class placement;
class planar_shape_representation;
class plane;
class plane_angle_measure_with_unit;
class plane_angle_unit;
class plane_angle_unit_and_si_unit;
class plane_milling_operation;
class plus_minus_tolerance;
class pocket;
class pocket_bottom;
class point;
class point_on_curve;
class point_on_surface;
class polyline;
class position_tolerance;
class precision_qualifier;
class process_or_process_relationship;
class process_product_association;
class process_property_association;
class product;
class product_category;
class product_category_relationship;
class product_context;
class product_definition;
class product_definition_context;
class product_definition_formation;
class product_definition_process;
class product_definition_relationship;
class product_definition_shape;
class product_definition_usage;
class product_definition_with_associated_documents;
class product_or_formation_or_definition;
class product_related_product_category;
class profile_floor;
class projected_zone_definition;
class property_definition;
class property_definition_representation;
class property_or_shape_select;
class property_process;
class protrusion;
class qualified_representation_item;
class qualitative_uncertainty;
class quasi_uniform_curve;
class quasi_uniform_curve_and_rational_b_spline_curve;
class quasi_uniform_surface;
class quasi_uniform_surface_and_rational_b_spline_surface;
class ratio_measure_with_unit;
class ratio_unit;
class ratio_unit_and_si_unit;
class rational_b_spline_curve;
class rational_b_spline_curve_and_uniform_curve;
class rational_b_spline_surface;
class rational_b_spline_surface_and_uniform_surface;
class real_literal;
class real_numeric_variable;
class rectangular_closed_profile;
class rectangular_composite_surface;
class rectangular_pattern;
class rectangular_trimmed_surface;
class referenced_modified_datum;
class relationship_with_condition;
class removal_volume;
class reparametrised_composite_curve_segment;
class replicate_feature;
class representation;
class representation_context;
class representation_item;
class representation_item_relationship;
class representation_map;
class representation_relationship;
class representation_relationship_with_transformation;
class representation_relationship_with_transformation_and_shape_representation_relationship;
class represented_definition;
class requirement_for_action_resource;
class resource_property;
class resource_property_representation;
class resource_requirement_type;
class reversible_topology;
class reversible_topology_item;
class revolved_profile;
class rib_top;
class rib_top_floor;
class right_circular_cylinder;
class role_association;
class role_select;
class round_hole;
class rounded_end;
class rounded_u_profile;
class roundness_tolerance;
class runout_zone_definition;
class runout_zone_orientation;
class runout_zone_orientation_reference_direction;
class seam_curve;
class security_classification;
class security_classification_assignment;
class security_classification_item;
class security_classification_level;
class sequential_method;
class serial_action_method;
class serial_numbered_effectivity;
class shape_aspect;
class shape_aspect_deriving_relationship;
class shape_aspect_relationship;
class shape_defining_relationship;
class shape_definition;
class shape_definition_representation;
class shape_dimension_representation;
class shape_representation;
class shape_representation_relationship;
class shape_representation_with_parameters;
class shape_tolerance_select;
class shell;
class shell_based_surface_model;
class si_unit;
class si_unit_and_solid_angle_unit;
class si_unit_and_time_unit;
class side_milling_operation;
class simple_generic_expression;
class simple_numeric_expression;
class slot;
class slot_end;
class solid_angle_measure_with_unit;
class solid_angle_unit;
class solid_model;
class source_item;
class spherical_cap;
class spherical_surface;
class square_u_profile;
class standard_uncertainty;
class step;
class straightness_tolerance;
class supported_item;
class surface;
class surface_boundary;
class surface_curve;
class surface_model;
class surface_of_linear_extrusion;
class surface_of_revolution;
class surface_patch;
class surface_profile_tolerance;
class surface_replica;
class surface_texture_representation;
class swept_surface;
class symmetric_shape_aspect;
class symmetry_tolerance;
class tangent;
class taper;
class tapping_operation;
class tee_profile;
class thread;
class time_interval;
class time_interval_based_effectivity;
class time_measure_with_unit;
class time_unit;
class tolerance_method_definition;
class tolerance_value;
class tolerance_zone;
class tolerance_zone_definition;
class tolerance_zone_form;
class topological_representation_item;
class toroidal_surface;
class total_runout_tolerance;
class transformation;
class transition_feature;
class trimmed_curve;
class trimming_select;
class turned_knurl;
class type_qualifier;
class unary_boolean_expression;
class unary_generic_expression;
class uncertainty_measure_with_unit;
class uncertainty_qualifier;
class uniform_curve;
class uniform_surface;
class unit;
class value_qualifier;
class value_range;
class variable;
class variable_semantics;
class vector;
class vector_or_direction;
class vee_profile;
class vertex;
class vertex_loop;
class vertex_point;
class week_of_year_and_day_date;
class xor_expression;

/* enum defs needed for functions */
#include "limit_condition.h"	// for tolerance.h

#include "StixCursor.h"
#include "StixNC.h"
#include "stix_utils.h"
#include "tolerance.h"

#endif /* STIX_H */

