/* $RCSfile: stix_utils.h,v $
 * $Revision: 1.29 $ $Date: 2003/05/30 14:49:57 $
 * Auth: Dave Loffredo (loffredo@steptools.com)
 * 
 * Copyright (c) 1991-2003 by STEP Tools Inc. 
 * All Rights Reserved.
 * 
 * This file is part of the STIX Library
 * 
 * This file may be distributed and/or modified under the terms of 
 * the GNU General Public License version 2 as published by the Free
 * Software Foundation and appearing in the file LICENSE.GPL included
 * with this file.
 * 
 * THIS FILE IS PROVIDED "AS IS" WITH NO WARRANTY OF ANY KIND,
 * INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * STEP Tools Inc. also offers licenses for commercial use of this
 * software in non-GPL applications.  Contact info@steptools.com for
 * terms and pricing.
 * 
 * 		----------------------------------------
 */

/* External Identifiers */
#define EXID_TYPEMASK	0xf0000000
#define EXID_BODYMASK	0x0fffffff
#define EXID_SETUP	0x10000000
#define EXID_PROJECT	0x20000000
#define EXID_WORKPIECE	0x30000000
#define EXID_WORKSTEP	0x40000000
#define EXID_FEATURE	0x50000000
#define EXID_COMPONENT	0x60000000
#define EXID_TOOL	0x70000000

/* ----------------------------------------
 * Shape Aspect Types -- Define symbols for each type of shape
 * aspect using bit flags and such to allow for fast tests.  For
 * better function type checking, we may also define some enums
 * for these things.
 */

/* feature categories */
#define FEAT_NONE		0

/* control flow */
#define ASPECT_CAT_MASK		0xff00	/* type mask */

#define FEAT_BASE		0x0100	/* no more than 255 features */
#define FEAT_UNKNOWN		(FEAT_BASE)
#define FEAT_BOSS		(FEAT_BASE+1)
#define FEAT_CBORE		(FEAT_BASE+2)	/* compound */
#define FEAT_CHAMFER		(FEAT_BASE+3)	/* transition */
#define FEAT_COMPOUND		(FEAT_BASE+4)	/* compound */
#define FEAT_CSUNK		(FEAT_BASE+5)	/* compound */
#define FEAT_EDGE_ROUND		(FEAT_BASE+6)	/* transition */
#define FEAT_HOLE		(FEAT_BASE+7)
#define FEAT_PAT_CIRC		(FEAT_BASE+8)
#define FEAT_PAT_GEN		(FEAT_BASE+9)
#define FEAT_PAT_RECT		(FEAT_BASE+10)
#define FEAT_PLANE		(FEAT_BASE+11)
#define FEAT_POCKET		(FEAT_BASE+12)
#define FEAT_POCKET_OPEN	(FEAT_BASE+13)
#define FEAT_POCKET_CLOSED	(FEAT_BASE+14)
#define FEAT_PROFILE		(FEAT_BASE+15)
#define FEAT_PROF_OUTSIDE	(FEAT_BASE+24)
#define FEAT_PROF_SHAPE		(FEAT_BASE+25)
#define FEAT_PROF_SHAPE_RECT	(FEAT_BASE+26)
#define FEAT_REGION		(FEAT_BASE+16)
#define FEAT_ROUNDED_END	(FEAT_BASE+17)
#define FEAT_SLOT		(FEAT_BASE+18)
#define FEAT_SPHERICAL_CAP	(FEAT_BASE+19)
#define FEAT_STEP		(FEAT_BASE+20)
#define FEAT_THREAD		(FEAT_BASE+21)
#define FEAT_TOOLPATH		(FEAT_BASE+22)

/* profiles */
#define PROF_BASE		0x0200	/* no more than 255 profiles */
#define PROF_OC_MASK		0xff80	/* open/closed */
#define PROF_OPEN_BASE		0x0200	/* 8's bit set means closed */
#define PROF_CLOSE_BASE		0x0280
#define PROF_UNKNOWN		(PROF_BASE)

/* open profiles */
#define PROF_LINEAR		(PROF_OPEN_BASE+1)
#define PROF_OPEN_GENERAL	(PROF_OPEN_BASE+2)
#define PROF_OPEN_CIRC		(PROF_OPEN_BASE+3)
#define PROF_ROUNDED_U		(PROF_OPEN_BASE+4)
#define PROF_SQUARE_U		(PROF_OPEN_BASE+5)
#define PROF_TEE		(PROF_OPEN_BASE+6)
#define PROF_VEE		(PROF_OPEN_BASE+7)
/* closed profiles */
#define PROF_CLOSED_CIRC	(PROF_CLOSE_BASE+1) 	/* 8+ is closed */
#define PROF_CLOSED_NGON	(PROF_CLOSE_BASE+2)
#define PROF_CLOSED_GENERAL	(PROF_CLOSE_BASE+3)
#define PROF_CLOSED_RECT	(PROF_CLOSE_BASE+4)

/* profiles */
#define PATH_BASE		0x0300	/* max 255 paths */
#define PATH_UNKNOWN		(PATH_BASE)
#define PATH_LINEAR		(PATH_BASE+1)
#define PATH_OPEN_CIRC		(PATH_BASE+2)
#define PATH_CLOSED_CIRC	(PATH_BASE+3)
#define PATH_GENERAL		(PATH_BASE+4)

/* other components */
#define COMP_MASK		0x8000	/* many component subtypes */
#define COMP_BASE		0x8000	/* hi bit means component */
#define COMP_UNKNOWN		(COMP_BASE)
#define COMP_APPLIED_AREA	(COMP_BASE+1)
#define COMP_BOSS_TOP		(COMP_BASE+2)
#define COMP_CHAMFER_OFFSET	(COMP_BASE+3)
#define COMP_PATTERN_MOD	(COMP_BASE+5)
#define COMP_RIB_TOP_FLOOR	(COMP_BASE+8)

/* hole bottom types */
#define HBOT_BASE		0x8100	/* max 255 hole bottoms */
#define HBOT_UNKNOWN		(HBOT_BASE)
#define HBOT_FLAT		(HBOT_BASE+1)  //  flat_hole_bottom
#define HBOT_FLAT_RADIUS	(HBOT_BASE+2)  //  flat_with_radius_hole_bottom
#define HBOT_SPHERICAL		(HBOT_BASE+3)  //  spherical_hole_bottom
#define HBOT_CONICAL		(HBOT_BASE+4)  //  conical_hole_bottom
#define HBOT_THROUGH		(HBOT_BASE+5)  //  through_bottom_condition

/* pocket bottom types */
#define PBOT_BASE		0x8200	/* max 255 pocket bottoms */
#define PBOT_UNKNOWN		(PBOT_BASE)
#define PBOT_THROUGH		(PBOT_BASE+1)  // through bottom
#define PBOT_FLAT		(PBOT_BASE+2)  // planar bottom
#define PBOT_RADIUSED		(PBOT_BASE+3)  // radiused bottom
#define PBOT_GENERAL		(PBOT_BASE+4)  // general bottom

/* slot end types */
#define SLOTEND_BASE		0x8300	/* max 255 slot ends */
#define SLOTEND_UNKNOWN		(SLOTEND_BASE)
#define SLOTEND_WOODRUFF	(SLOTEND_BASE+1)
#define SLOTEND_RADIUSED	(SLOTEND_BASE+2)
#define SLOTEND_FLAT		(SLOTEND_BASE+3)
#define SLOTEND_OPEN		(SLOTEND_BASE+4)
#define SLOTEND_LOOP		(SLOTEND_BASE+5)

/* taper types */
#define TAPER_BASE		0x8400	/* max 255 taper */
#define TAPER_UNKNOWN		(TAPER_BASE)
#define TAPER_DIAMETER		(TAPER_BASE+1)
#define TAPER_ANGLE		(TAPER_BASE+2)

/* profile floors */
#define PROFLOOR_BASE		0x8500	/* max 255 profile floor */
#define PROFLOOR_UNKNOWN	(PROFLOOR_BASE)
#define PROFLOOR_THROUGH	(PROFLOOR_BASE+1)  //  through_profile_floor
#define PROFLOOR_PLANAR		(PROFLOOR_BASE+2)  //  planar_profile_floor
#define PROFLOOR_GENERAL	(PROFLOOR_BASE+3)  //  general_profile_floor 

/* tolerance based things */
//      composite_shape_aspect --> datums
//      derived_shape_aspect   --> tols
//      symmetric_shape_aspect

//      tolerance_zone

#define ASPECT_CAT(sa)		((sa)->m_type & ASPECT_CAT_MASK)

#define IS_FEATURE(sa)		(ASPECT_CAT(sa) == FEAT_BASE)
#define IS_PROFILE(sa)		(ASPECT_CAT(sa) == PROF_BASE)
#define IS_OPEN_PROFILE(sa)	(((sa)->m_type & PROF_OC_MASK) == PROF_OPEN_BASE)
#define IS_CLOSED_PROFILE(sa)	(((sa)->m_type & PROF_OC_MASK) == PROF_CLOSE_BASE)
#define IS_PATH(sa)		(ASPECT_CAT(sa) == PATH_BASE)

#define IS_COMPONENT(sa)	(((sa)->m_type & COMP_MASK) == COMP_BASE)
#define IS_SLOT_END(sa)		(ASPECT_CAT(sa) == SLOTEND_BASE)
#define IS_HOLE_BOTTOM(sa)	(ASPECT_CAT(sa) == HBOT_BASE)
#define IS_POCKET_BOTTOM(sa)	(ASPECT_CAT(sa) == PBOT_BASE)
#define IS_TAPER(sa)		(ASPECT_CAT(sa) == TAPER_BASE)
#define IS_PROFILE_FLOOR(sa)	(ASPECT_CAT(sa) == PROFLOOR_BASE)




/* ----------------------------------------
 * Machining Process Executable Types
 */
#define EXEC_NONE		0
#define EXEC_BAD		1  /* ERROR */

/* control flow */
#define EXEC_CAT_MASK		0xff00	/* type mask */

#define EXEC_CTRL_BASE		0x0100	/* no more than 255 control flow */
#define EXEC_ASSIGN		(EXEC_CTRL_BASE+1)
#define EXEC_IF			(EXEC_CTRL_BASE+2)
#define EXEC_NONSEQ		(EXEC_CTRL_BASE+3)
#define EXEC_PARALLEL		(EXEC_CTRL_BASE+4)
#define EXEC_RAPID		(EXEC_CTRL_BASE+5)
#define EXEC_SELECT		(EXEC_CTRL_BASE+6)
#define EXEC_WHILE		(EXEC_CTRL_BASE+7)
#define EXEC_WORKPLAN		(EXEC_CTRL_BASE+8)

#define EXEC_MILL_BASE		0x0200	/* no more than 255 mill ops */
#define EXEC_MILL_BOTSIDE	(EXEC_MILL_BASE+1)
#define EXEC_MILL_BOTSIDE_ROUGH	(EXEC_MILL_BASE+2)
#define EXEC_MILL_FREEFORM	(EXEC_MILL_BASE+3) 
#define EXEC_MILL_PLANE		(EXEC_MILL_BASE+4)
#define EXEC_MILL_PLANE_ROUGH	(EXEC_MILL_BASE+5)
#define EXEC_MILL_SIDE		(EXEC_MILL_BASE+6)
#define EXEC_MILL_SIDE_ROUGH	(EXEC_MILL_BASE+7)

#define EXEC_DRILL_BASE		0x0300	/* no more than 255 drill ops */
#define EXEC_BORE		(EXEC_DRILL_BASE+1)
#define EXEC_BACK_BORE		(EXEC_DRILL_BASE+2)
#define EXEC_DRILL		(EXEC_DRILL_BASE+3)
#define EXEC_DRILL_CENTER	(EXEC_DRILL_BASE+4)
#define EXEC_DRILL_CSINK	(EXEC_DRILL_BASE+5)
#define EXEC_DRILL_MULTISTEP	(EXEC_DRILL_BASE+6)
#define EXEC_REAM		(EXEC_DRILL_BASE+7)
#define EXEC_TAP		(EXEC_DRILL_BASE+8)
#define EXEC_THREAD_DRILL	(EXEC_DRILL_BASE+9)

#define EXEC_FUNC_BASE		0x0400	/* no more than 255 nc functions */
#define EXEC_MARK_SET		(EXEC_FUNC_BASE+1)
#define EXEC_MARK_WAIT		(EXEC_FUNC_BASE+2)
#define EXEC_MESSAGE		(EXEC_FUNC_BASE+3)
#define EXEC_PALLET_EXCHANGE	(EXEC_FUNC_BASE+4)
#define EXEC_PALLET_INDEX	(EXEC_FUNC_BASE+5)
#define EXEC_STOP		(EXEC_FUNC_BASE+6)
#define EXEC_STOP_OPTIONAL	(EXEC_FUNC_BASE+7)
#define EXEC_TOOL_LOAD		(EXEC_FUNC_BASE+8)
#define EXEC_TOOL_UNLOAD	(EXEC_FUNC_BASE+9)

#define EXEC_PROBE_BASE		0x0500	/* no more than 255 probe types */
#define EXEC_PROBE		(EXEC_PROBE_BASE+1)
#define EXEC_PROBE_COMPLETE	(EXEC_PROBE_BASE+2)
#define EXEC_PROBE_TLEN		(EXEC_PROBE_BASE+3)
#define EXEC_PROBE_TRAD		(EXEC_PROBE_BASE+4)

#define EXEC_TURN_BASE		0x0600	/* turning not yet defined */

#define EXEC_CAT(mpe)		((mpe)->m_type & EXEC_CAT_MASK)

#define IS_CONTROL(mpe)		(EXEC_CAT(mpe) == EXEC_CTRL_BASE)
#define IS_MILL(mpe)		(EXEC_CAT(mpe) == EXEC_MILL_BASE)
#define IS_DRILL(mpe)		(EXEC_CAT(mpe) == EXEC_DRILL_BASE)
#define IS_PROBING(mpe)		(EXEC_CAT(mpe) == EXEC_PROBE_CAT)
#define IS_NC_FUNCTION(mpe)	(EXEC_CAT(mpe) == EXEC_FUNC_BASE)



/* Common units for easy application use */
typedef int StixErr;

#define STIX_OPTIONAL_UNIT	StixUnit u = stixunit_default

enum StixUnit {
    stixunit_default=0,		/* convert to default unit */
    stixunit_as_is,		/* no conversion */
    stixunit_unknown,		/* unknown unit */

    /* length */
    stixunit_mm,		/* default */
    stixunit_cm,
    stixunit_m,
    stixunit_in,
    stixunit_ft,

    /* angle */
    stixunit_deg,		/* default */
    stixunit_rad, 

    /* time */
    stixunit_sec,		/* default */

    /* linear speed */
    stixunit_mmps,
    stixunit_cmps,
    stixunit_mps,		/* default */
    stixunit_ips,
    stixunit_fps,

    /* rotational speed */
    stixunit_hertz,		/* default */
    stixunit_rpm,

    /* pressure */
    stixunit_pa,		/* default */
    stixunit_kpa,
    stixunit_mpa,
    stixunit_psi
};



/* in units.cxx */
char * get_unit_name (unit * u);
char * get_unit_name (StixUnit u);

unit * get_length_unit (shape_representation * rep);
void print_measure_value (measure_value * m);
void print_conversion_factors();

int get_boolean_value (
    const char * true_sym,
    const char * false_sym,
    representation * rep, 
    const char * targ = 0
    );

/* these are common functions, and only appear once */
/* these know about a default unit that should be used for gets/puts */

representation_item * get_representation_item (
    representation * rep, 
    RoseDomain * targ_type, 
    const char * targ_name = 0
    );

descriptive_representation_item * 
get_descriptive_rep_item (representation * rep, const char * targ=0);

/* ratio measures */
double	 get_ratio_value (measure_with_unit *);  // UNITLESS?
double	 get_ratio_value (
    representation * 	rep, 
    const char * 	targ=0
    );

StixErr	 put_ratio_value (measure_with_unit *, double);
measure_with_unit * get_ratio_rep_item (
    representation * 	rep, 
    const char * 	targ=0
    );


/* count measures */
double	 get_count_value (measure_with_unit *);  // UNITLESS
double	 get_count_value (
    representation * 	rep, 
    const char * 	targ=0
    );

StixErr	 put_count_value (measure_with_unit *, double);
measure_with_unit * get_count_rep_item (
    representation * 	rep, 
    const char * 	targ=0
    );


/* length measures */
double	 get_length_value (measure_with_unit *, STIX_OPTIONAL_UNIT);
double	 get_length_value (
    representation * 	rep, 
    const char * 	targ=0,
    STIX_OPTIONAL_UNIT);

StixErr	 put_length_value (measure_with_unit *, double, STIX_OPTIONAL_UNIT);
StixUnit get_length_default_unit ();
StixErr  put_length_default_unit (StixUnit u);
measure_with_unit * get_length_rep_item (
    representation * 	rep, 
    const char * 	targ=0
    );

/* plane angle measures */
double	 get_angle_value (measure_with_unit *, STIX_OPTIONAL_UNIT);
double	 get_angle_value (
    representation * 	rep, 
    const char * 	targ=0,
    STIX_OPTIONAL_UNIT);

StixErr	 put_angle_value (measure_with_unit *, double, STIX_OPTIONAL_UNIT);
StixUnit get_angle_default_unit ();
StixErr  put_angle_default_unit (StixUnit u);
measure_with_unit * get_angle_rep_item (
    representation * 	rep, 
    const char * 	targ=0);

/* duration measures */
double	 get_time_value (measure_with_unit *, STIX_OPTIONAL_UNIT);
double	 get_time_value (
    representation * 	rep, 
    const char * 	targ=0,
    STIX_OPTIONAL_UNIT);

StixErr	 put_time_value (measure_with_unit *, double, STIX_OPTIONAL_UNIT);
StixUnit get_time_default_unit ();
StixErr  put_time_default_unit (StixUnit u);
measure_with_unit * get_time_rep_item (
    representation * 	rep, 
    const char * 	targ=0);

/* linear speed measures */
double	 get_speed_value (measure_with_unit *, STIX_OPTIONAL_UNIT);
double	 get_speed_value (
    representation * 	rep, 
    const char * 	targ=0,
    STIX_OPTIONAL_UNIT);

StixErr	 put_speed_value (measure_with_unit *, double, STIX_OPTIONAL_UNIT);
StixUnit get_speed_default_unit ();
StixErr  put_speed_default_unit (StixUnit u);
measure_with_unit * get_speed_rep_item (
    representation * 	rep, 
    const char * 	targ=0);

/* rotational speed measures */
double	 get_rotspeed_value (measure_with_unit *, STIX_OPTIONAL_UNIT);
double	 get_rotspeed_value (
    representation * 	rep, 
    const char * 	targ=0,
    STIX_OPTIONAL_UNIT);

StixErr	 put_rotspeed_value (measure_with_unit *, double, STIX_OPTIONAL_UNIT);
StixUnit get_rotspeed_default_unit ();
StixErr  put_rotspeed_default_unit (StixUnit u);
measure_with_unit * get_rotspeed_rep_item (
    representation * 	rep, 
    const char * 	targ=0);

/* pressure measures */
double   get_pressure_value (measure_with_unit *, STIX_OPTIONAL_UNIT);
double	 get_pressure_value (
    representation * 	rep, 
    const char * 	targ=0,
    STIX_OPTIONAL_UNIT);

StixErr	 put_pressure_value (measure_with_unit *, double, STIX_OPTIONAL_UNIT);
StixUnit get_pressure_default_unit ();
StixErr  put_pressure_default_unit (StixUnit u);
measure_with_unit * get_pressure_rep_item (
    representation * 	rep, 
    const char * 	targ = 0);


/* other types of rep items */
axis2_placement_3d * 	get_placement (representation*, const char* nm=0);
direction * 		get_direction (representation*, const char* nm=0);


face_shape_representation * get_face_shape_property (
    shape_aspect *, const char* rep_nm=0
    );
planar_shape_representation * get_planar_shape_property (
    shape_aspect *, const char* rep_nm=0
    );
direction_shape_representation * get_direction_shape_property (
    shape_aspect *, const char* rep_nm=0
    );

#ifdef BUG_PATH_SHAPE_REP_NOT_USED
shape_representation * 
#else
path_shape_representation * 
#endif
get_path_shape_property (
    shape_aspect *, const char* rep_nm=0
    );


RoseObject * get_select_object (RoseUnion * sel);


product_definition * property_get_pdef (
    property_definition_representation * pdr
    );

product_definition * property_get_pdef (
    property_definition * pd
    );

RoseObject * get_property_owner (
    property_definition_representation * pdr
    );

representation * get_action_property_rep (
    action_method * am,
    const char * prop_name
    );


/* ----------------------------------------
 * DUMP Functions -- Print out basic details about
 * the objects.  The StixIndex class is a little helper
 * class for indenting the output of the dump functions
 */
class StixIndent {
private:
    char m_tabs[255];
public:
    StixIndent();

    void init();
    void inc();
    void dec();

    /* virtual by request to make it easier to
     * redirect the output to other places 
     */
    virtual void print();
    virtual char* string();
};

char* printable_name (action_method *);
char* printable_name (product *);
char* printable_name (product_definition_formation *);
char* printable_name (product_definition *);

void dump_workpiece_detail (
    product_definition *, StixIndent* tabs=0
    );
void dump_workplan_detail (
    machining_workplan *, StixIndent* tabs=0
    );
void dump_workplan_all_tools (
    machining_workplan *, StixIndent* tabs=0
    );
void dump_workplan_all_feats (
    machining_workplan *, StixIndent* tabs=0
    );
void dump_workingstep_detail (
    machining_workingstep *, StixIndent* tabs=0
    );
void dump_feature_detail (
    shape_aspect *, StixIndent* tabs=0
    );
void dump_operation_detail (
    machining_operation *, StixIndent* tabs=0
    );
void dump_tool_detail (
    machining_tool *, StixIndent* tabs=0
    );
void dump_action_props (
    action_method *, StixIndent* tabs=0
    );



/* --------------------------------------------------
 * Project functions
 */
unsigned 		get_project_exid (product *);
StixProject * 		get_project_idx (product *);
product_definition * 	get_project_workpiece (product *);
machining_workplan * 	get_project_workplan (product *);


/* --------------------------------------------------
 * Setup functions
 */
unsigned 		get_setup_exid (product *);
StixSetup * 		get_setup_idx  (product *);


/* --------------------------------------------------
 * Workpiece functions 
 */
unsigned 		get_workpiece_exid (product_definition *);
StixWorkpiece * 	get_workpiece_idx (product_definition *);
shape_representation * 	get_workpiece_shape (product_definition *);

product_definition * 	get_workpiece_stock (product_definition *);
shape_representation * 	get_workpiece_stock_shape (product_definition *);

/* Stock limits handled from the workpiece, is the stock limit id
 * actually useful to anyone?  In the mappings it just returns the 
 * entity id of the product
 */
unsigned get_workpiece_stock_limit_id (product_definition *);
double 	get_workpiece_stock_min_x (product_definition *);
double 	get_workpiece_stock_max_x (product_definition *);
double 	get_workpiece_stock_min_y (product_definition *);
double 	get_workpiece_stock_max_y (product_definition *);
double 	get_workpiece_stock_min_z (product_definition *);
double 	get_workpiece_stock_max_z (product_definition *);



/* --------------------------------------------------
 * Features
 */
unsigned   	get_aspect_type (shape_aspect *);
char *   	get_aspect_typename (shape_aspect *);

direction * 	get_aspect_direction_property (shape_aspect *);

unsigned 	get_feature_exid (shape_aspect *);
StixFeature * 	get_feature_idx  (shape_aspect *);

product_definition * get_feature_workpiece(shape_aspect *);
axis2_placement_3d * get_feature_placement(shape_aspect *);

shape_aspect * 	get_feature_final (shape_aspect *);

shape_aspect * 	get_feature_component(
    shape_aspect * feat,
    const char * sar_desc,	/* often 'foo usage' */
    const char * sa_ed_desc=0,	/* often 'something foo occurrence' */
    const char * sar_name=0	/* rare, used for slot ends, others */
    );


double calc_feature_depth (shape_aspect *, STIX_OPTIONAL_UNIT);

shape_aspect *  get_hole_profile (shape_aspect *);
shape_aspect *  get_hole_path (shape_aspect *);

shape_aspect *  get_hole_taper (shape_aspect *);
shape_aspect *  get_hole_bottom_condition (shape_aspect *);
double 		get_hole_diameter (shape_aspect *, STIX_OPTIONAL_UNIT);
double 		get_hole_final_diameter (shape_aspect *, STIX_OPTIONAL_UNIT);

shape_aspect * 	get_pocket_profile (shape_aspect *);  // boundary??
shape_aspect * 	get_pocket_path (shape_aspect *);
shape_aspect * 	get_pocket_open_wall_profile (shape_aspect *);
shape_aspect * 	get_pocket_taper (shape_aspect *);
shape_aspect * 	get_pocket_bottom_condition (shape_aspect *);
/* StixFeatureBossCursor -- traverses a pockets bosses */

double get_pocket_planar_radius (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );

double get_pocket_orthogonal_radius (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );

double get_pocket_slope_angle (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );


shape_aspect * 	get_planar_face_course_of_travel (shape_aspect *); 
shape_aspect * 	get_planar_face_removal_boundary (shape_aspect *);
shape_aspect * 	get_planar_face_face_boundary (shape_aspect *);
/* StixFeatureBossCursor -- traverses a planar face bosses */


shape_aspect * get_boss_taper (shape_aspect *);
shape_aspect * get_boss_boundary (shape_aspect *);


/* Edge Round */
face_shape_representation * get_edge_round_face_shape (shape_aspect *);
face_shape_representation * get_edge_round_first_face (shape_aspect *);
face_shape_representation * get_edge_round_second_face (shape_aspect *);

double get_edge_round_radius (shape_aspect *, STIX_OPTIONAL_UNIT);
double get_edge_round_first_offset (shape_aspect *, STIX_OPTIONAL_UNIT);
double get_edge_round_second_offset (shape_aspect *, STIX_OPTIONAL_UNIT);


/* Slot ends */
shape_aspect * get_slot_end_at_start (shape_aspect * sa);
shape_aspect * get_slot_end_at_end (shape_aspect * sa);
shape_aspect * get_slot_course_of_travel (shape_aspect *); 
shape_aspect * get_slot_swept_shape (shape_aspect *); 


/* Outside profiles */
shape_aspect * get_outside_profile_feature_boundary (shape_aspect *); 
shape_aspect * get_outside_profile_sweep_path (shape_aspect *); 


/* shape profiles */
shape_aspect * get_shape_profile_feature_boundary (shape_aspect *); 
shape_aspect * get_shape_profile_sweep_path (shape_aspect *); 
shape_aspect * get_shape_profile_floor (shape_aspect *); 
direction * get_shape_profile_direction (shape_aspect * sa);


/* step */
shape_aspect * 	get_step_course_of_travel (shape_aspect *); 
shape_aspect * 	get_step_wall_boundary (shape_aspect *);
/* StixFeatureBossCursor -- traverses a step bosses */



/* --------------------------------------------------
 * Feature Component functions
 */
unsigned	get_component_exid (shape_aspect *);
StixComponent * get_component_idx  (shape_aspect *);




/* PATH ATTRIBUTES ============================== */

axis2_placement_3d * get_path_placement (shape_aspect *);

direction * get_path_linear_direction (
    shape_aspect *
    );
double get_path_linear_distance (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );


double get_path_circle_radius (		/* both closed and partial */
    shape_aspect *, STIX_OPTIONAL_UNIT
    );

double get_path_circle_sweep_angle (	/* partial only */
    shape_aspect *, STIX_OPTIONAL_UNIT
    );

#ifdef BUG_PATH_SHAPE_REP_NOT_USED
shape_representation * 
#else
path_shape_representation * 
#endif
get_path_general_shape (
    shape_aspect * sa 	
    );


/* PROFILE ATTRIBUTES ============================== */

axis2_placement_3d * get_profile_placement (shape_aspect *);

/* circular closed profile */
double get_profile_circle_diameter (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );



/* rectangular closed profile */
double get_profile_rectangular_width  (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );
double get_profile_rectangular_length (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );
double get_profile_rectangular_corner_radius (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );


double get_profile_ngon_diameter (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );
int get_profile_ngon_sides (
    shape_aspect * sa
    );
int get_profile_ngon_circumscribed_or_flats (
    shape_aspect * sa 	    /* true=flats false=circumscribed */
    );


/* good for general open or closed profiles */
#ifdef BUG_PATH_SHAPE_REP_NOT_USED
shape_representation * 
#else
path_shape_representation * 
#endif
get_profile_general_shape (
    shape_aspect * sa 	
    );


double get_profile_linear_length (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );


double get_profile_arc_radius (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );
double get_profile_arc_sweep_angle (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );


double get_profile_square_u_width (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );
double get_profile_square_u_first_radius (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );
double get_profile_square_u_first_angle (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );
double get_profile_square_u_second_radius (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );
double get_profile_square_u_second_angle (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );


double get_profile_rounded_u_width (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );



double get_profile_tee_first_angle (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );
double get_profile_tee_second_angle (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );
double get_profile_tee_cross_bar_width (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );
double get_profile_tee_cross_bar_depth (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );
double get_profile_tee_radius (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );
double get_profile_tee_width (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );
double get_profile_tee_first_offset (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );
double get_profile_tee_second_offset (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );


double get_profile_vee_radius (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );
double get_profile_vee_angle (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );
double get_profile_vee_tilt (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );




/* tapers */
double get_taper_angle (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );
double get_taper_final_diameter (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );


/* hole bottoms */
double get_holebottom_conical_tip_angle (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );
double get_holebottom_conical_tip_radius (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );


/* pocket bottoms */
double get_pocketbottom_radius (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );
double get_pocketbottom_radiused_bottom_radius (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );
face_shape_representation * get_pocketbottom_general_face (
    shape_aspect *
    );



/* slot ends */
double get_slotend_flat_first_radius (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );
double get_slotend_flat_second_radius (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );
double get_slotend_woodruff_radius (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );



/* profile floor conditions */
double get_profloor_radius (
    shape_aspect *, STIX_OPTIONAL_UNIT
    );

int get_profloor_start_or_end (	
    shape_aspect * 	/* true=end, false=start */
    );

planar_shape_representation * get_profloor_planar_face (	
    shape_aspect *
    );
face_shape_representation * get_profloor_general_face (	
    shape_aspect *
    );



/* --------------------------------------------------
 * Executable information, workplans and workingsteps
 */
unsigned get_exec_exid (machining_process_executable *);
unsigned get_exec_type (machining_process_executable *);
char *   get_exec_typename (machining_process_executable *);

machining_process_executable * get_exec_parent (
    machining_process_executable *
    );

/* StixWorkplanElementCursor -- traverses a workplans substeps */


/* is it more convenient to make these take MPE? */
shape_aspect *	get_workingstep_feature (
    machining_workingstep *	
    );
shape_aspect *	get_workingstep_inproc_feature (
    machining_workingstep *
    );
shape_aspect *	get_workingstep_final_feature (
    machining_workingstep *
    );

machining_operation * get_workingstep_operation (
    machining_process_executable * mpe
    );



/* ----------------------------------------
 * Milling Operations
 */
machining_tool * get_operation_tool (
    machining_operation * op
    );
machining_technology * get_operation_technology (
    machining_operation * op
    );
machining_functions * get_operation_functions (
    machining_operation * op
    );
machining_strategy * get_operation_strategy (
    machining_operation * op
    );
double get_operation_overcut_length (
    machining_operation * op, STIX_OPTIONAL_UNIT
    );



machining_approach_retract_strategy * get_milling_operation_approach_strategy (
    machining_operation * op
    );
machining_approach_retract_strategy * get_milling_operation_retract_strategy (
    machining_operation * op
    );
double get_milling_operation_axial_cutting_depth (
    machining_operation * op, STIX_OPTIONAL_UNIT
    );
double get_milling_operation_radial_cutting_depth (
    machining_operation * op, STIX_OPTIONAL_UNIT
    );
double get_milling_operation_allowance_side (
    machining_operation * op, STIX_OPTIONAL_UNIT
    );
double get_milling_operation_allowance_bottom (
    machining_operation * op, STIX_OPTIONAL_UNIT
    );



double get_drilling_operation_cutting_depth (
    machining_operation * op, STIX_OPTIONAL_UNIT
    );
double get_drilling_operation_previous_diameter (
    machining_operation * op, STIX_OPTIONAL_UNIT
    );
double get_drilling_operation_dwell_time_bottom (
    machining_operation * op, STIX_OPTIONAL_UNIT
    );
double get_drilling_operation_feed_on_retract (
    machining_operation * op
    );



/* --------------------------------------------------
 * Machining Technology Functions
 */
double get_technology_cutspeed (
    machining_technology * mt, STIX_OPTIONAL_UNIT
    );
double get_technology_spindle (
    machining_technology * mt, STIX_OPTIONAL_UNIT
    );
double get_technology_feedrate_per_tooth (
    machining_technology * mt, STIX_OPTIONAL_UNIT
    );
int get_technology_synchronize_spindle_with_feed (
    machining_technology * mt
    );
int get_technology_inhibit_feedrate_override (
    machining_technology * mt
    );
int get_technology_inhibit_spindle_override (
    machining_technology * mt
    );


/* --------------------------------------------------
 * Machine Functions
 */
int get_machine_functions_coolant (
    machining_functions * mf
    );
double get_machine_functions_coolant_pressure (
    machining_functions * mf, STIX_OPTIONAL_UNIT
    );
int get_machine_functions_mist (
    machining_functions * mf
    );
int get_machine_functions_through_spindle_coolant (
    machining_functions * mf
    );
double get_machine_functions_through_pressure (
    machining_functions * mf, STIX_OPTIONAL_UNIT
    );
int get_machine_functions_chip_removal (
    machining_functions * mf
    );
direction * get_machine_functions_oriented_spindle_stop (
    machining_functions * mf
    );


/* --------------------------------------------------
 * Tools functions
 */
unsigned get_tool_exid (machining_tool *);

double 	get_tool_diameter    (
    machining_tool *, STIX_OPTIONAL_UNIT
    );

double 	get_tool_edge_length (
    machining_tool *, STIX_OPTIONAL_UNIT
    );

double 	get_tool_edge_radius (
    machining_tool *, STIX_OPTIONAL_UNIT
    );

double 	get_tool_circumference_angle (
    machining_tool *, STIX_OPTIONAL_UNIT
    );

machining_tool_body_representation * get_tool_body (
    machining_tool *
    );

