/* $RCSfile: tolerance.cxx,v $
 * $Revision: 1.2 $ $Date: 2003/08/12 16:05:17 $
 * Auth: Dave Loffredo (loffredo@steptools.com)
 * 
 * Copyright (c) 1991-2003 by STEP Tools Inc. 
 * All Rights Reserved.
 * 
 * This file is part of the STIX Library
 * 
 * This file may be distributed and/or modified under the terms of 
 * the GNU General Public License version 2 as published by the Free
 * Software Foundation and appearing in the file LICENSE.GPL included
 * with this file.
 * 
 * THIS FILE IS PROVIDED "AS IS" WITH NO WARRANTY OF ANY KIND,
 * INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * STEP Tools Inc. also offers licenses for commercial use of this
 * software in non-GPL applications.  Contact info@steptools.com for
 * terms and pricing.
 * 
 * 		----------------------------------------
 */

#include <rose.h>
#include <integrated_cnc_schema.h>
#include "stix_imp.h"

#define IMPLEMENT_CURSOR_BODY(cls,rtntyp) \
cls::cls (StixNC * nc) { \
    m_nc = nc;		\
    if (!nc || !nc-> design()) return; \
    m_objs.traverse (nc-> design());	\
    m_objs.domain (ROSE_DOMAIN(rtntyp)); \
} \
rtntyp * cls::next() 	{ return ROSE_CAST(rtntyp,m_objs.next()); } \
void cls::rewind() 	{ m_objs.rewind(); } \
unsigned cls::size() 	{ return m_objs.size(); } 


IMPLEMENT_CURSOR_BODY(StixGeomToleranceCursor,geometric_tolerance)
IMPLEMENT_CURSOR_BODY(StixDimToleranceCursor,plus_minus_tolerance)
IMPLEMENT_CURSOR_BODY(StixDatumCursor,datum);


/* overload the function so that it works with both dimensional 
 * and geometric tolerances 
 */

unsigned get_tolerance_type (plus_minus_tolerance * tol) {
    return (tol? tol-> m_type: 0);
}
    
unsigned get_tolerance_type (geometric_tolerance * tol) {
    return (tol? tol-> m_type: 0);
}

char * get_tolerance_typename (plus_minus_tolerance * tol)
{
    switch (tol? tol->m_type: 0) {
    case DTOL_SIZE:		
	return "size dimension tolerance";
    case DTOL_SIZE_PATH:	
	return "size along path dimension tolerance";
    case DTOL_SIZE_ANGULAR:	
	return "angular size dimension tolerance";
    case DTOL_LOC:		
	return "location dimension tolerance";
    case DTOL_LOC_PATH:		
	return "location along path dimension tolerance";
    case DTOL_LOC_ANGULAR:	
	return "angular location dimension tolerance";

    case DTOL_UNKNOWN:		
	return "unknown dimension tolerance";
    default:
	return "bad dimension tolerance";
    }
}

char * get_tolerance_typename (geometric_tolerance * tol)
{
    switch (tol? tol->m_type: 0) {

    case GTOL_CYLINDRICITY: 	return "cylindricity tolerance";
    case GTOL_FLATNESS: 	return "flatness tolerance";
    case GTOL_LINE_PROFILE: 	return "line profile tolerance";
    case GTOL_MODIFIED: 	return "modified tolerance";
    case GTOL_POSITION: 	return "position tolerance";
    case GTOL_ROUNDNESS: 	return "roundness tolerance";
    case GTOL_STRAIGHTNESS: 	return "straightness tolerance";
    case GTOL_SURFACE_PROFILE: 	return "surface profile tolerance";

	/* All tolerances below have datum refs */
    case GTOL_ANGULARITY: 	return "angularity tolerance";
    case GTOL_CIRCULAR_RUNOUT: 	return "circular runout tolerance";
    case GTOL_COAXIALITY: 	return "coaxiality tolerance";
    case GTOL_CONCENTRICITY: 	return "concentricity tolerance";
    case GTOL_PARALLELISM: 	return "parallelism tolerance";
    case GTOL_PERPENDICULARITY: return "perpendicularity tolerance";
    case GTOL_SYMMETRY: 	return "symmetry tolerance";
    case GTOL_TOTAL_RUNOUT: 	return "total runout tolerance";

    case GTOL_UNKNOWN_DATUM: 	return "unknown tolerance with datums";
    case GTOL_UNKNOWN: 		return "unknown geometric tolerance";
    default:
	return "bad geometric tolerance";
    }
}


char * get_tolerance_name (plus_minus_tolerance * tol)
{
    RoseObject * obj = (tol? get_select_object 
			(tol-> toleranced_dimension()) : 0);

    /* just use late-bound code to avoid a bunch of casts */
    char * nm = (obj? obj-> getString("name"): 0);
    return (nm? nm: "");
}

char * get_tolerance_name (geometric_tolerance * tol)
{
    char * nm = (tol? tol-> name(): 0);
    return (nm? nm: "");
}


shape_representation * get_aspect_shape_rep (
    shape_aspect * sa
    )
{
    property_definition_representation * pdr = (sa? sa-> m_props: 0);

    /* go through the properties for a shape def */
    while (pdr) {

	/* Only these describe the shape of the aspect */
	if (!pdr-> isa(ROSE_DOMAIN(shape_definition_representation))) {
	    pdr = pdr-> m_next_pdr;
	    continue;
	}

	representation * rep = pdr-> used_representation();
	if (!rep || !rep-> items()) {
	    pdr = pdr-> m_next_pdr;
	    continue;
	}

	if (!rep->isa (ROSE_DOMAIN(shape_representation))) {
	    fprintf (stdout, "WARN: #%d should be a shape representation\n",
		     rep-> entity_id());
	    pdr = pdr-> m_next_pdr;
	    continue;
	}
	return ROSE_CAST(shape_representation,rep);
    }
    return 0;
}


/* 
	if (rep->items()->size() > 1) {
	    fprintf (stdout, "WARN: #%d tolerance should have only one item\n",
		     rep-> entity_id());
	    pdr = pdr-> m_next_pdr;
	    continue;
	}

	
	representation_item * ri = get_representation_item
	    (rep, ROSE_DOMAIN(geometric_representation_item));

	return ROSE_CAST(geometric_representation_item,ri);
    }
    return 0;
*/


/* eventually break these out into dimension-only functions */
topological_representation_item * get_aspect_shape_face (
    shape_aspect * sa
    )
{
    shape_representation * rep = get_aspect_shape_rep (sa);
    if (!rep || !rep-> items()) 
	return 0;

    if (rep->items()->size() > 1) {
	fprintf (stdout, "WARN: #%d tolerance should have only one item\n",
		 rep-> entity_id());
	return 0;
    }
    representation_item * ri = get_representation_item
	(rep, ROSE_DOMAIN(topological_representation_item));

    return ROSE_CAST(topological_representation_item,ri);
}


 
/* eventually break these out into dimension-only functions */
geometric_representation_item * get_aspect_shape_geometry (
    shape_aspect * sa
    )
{
    shape_representation * rep = get_aspect_shape_rep (sa);
    if (!rep || !rep-> items()) 
	return 0;

    if (rep->items()->size() > 1) {
	fprintf (stdout, "WARN: #%d tolerance should have only one item\n",
		 rep-> entity_id());
	return 0;
    }
    representation_item * ri = get_representation_item
	(rep, ROSE_DOMAIN(geometric_representation_item));

    return ROSE_CAST(geometric_representation_item,ri);
}


/* eventually break these out into dimension-only functions */
topological_representation_item * get_tolerance_face (
    plus_minus_tolerance * tol
    )
{
    shape_aspect * sa = 0;
    RoseObject * obj = (tol? get_select_object 
			(tol-> toleranced_dimension()) : 0);

    if (!obj) return 0;
    if (obj-> isa(ROSE_DOMAIN(dimensional_size)))
    {
	dimensional_size * ds = ROSE_CAST(dimensional_size,obj);
	sa = ds-> applies_to();
    }
    if (obj-> isa(ROSE_DOMAIN(dimensional_location)))
    {
	dimensional_location * dl = ROSE_CAST(dimensional_location,obj);
	sa = dl-> relating_shape_aspect();
    }

    return get_aspect_shape_face (sa);
}



topological_representation_item * get_tolerance_face (
    geometric_tolerance * tol
    )
{
    shape_aspect * sa = (tol? (tol-> toleranced_shape_aspect()) : 0);
    return get_aspect_shape_face (sa);
}

topological_representation_item * get_tolerance_other_face (
    plus_minus_tolerance * tol
    )
{
    shape_aspect * sa = 0;
    RoseObject * obj = (tol? get_select_object 
			(tol-> toleranced_dimension()) : 0);

    /* only present in location tolerances */
    if (!obj) return 0;
    if (obj-> isa(ROSE_DOMAIN(dimensional_location)))
    {
	dimensional_location * dl = ROSE_CAST(dimensional_location,obj);
	sa = dl-> related_shape_aspect();
    }
    return get_aspect_shape_face (sa);
}


////////////////////////////////////////

measure_with_unit * get_measure_rep_item (
    representation * rep, 
    const char *     targ)
{
    representation_item * it = get_representation_item 
	(rep, ROSE_DOMAIN(measure_with_unit), targ);

    return (it? ROSE_CAST(measure_with_unit,it): 0);
}


double get_measure_value (measure_with_unit * mwu, StixUnit u)
{
    /* just return any double value, dont fret units */
    measure_value * mv = (mwu? mwu-> value_component(): 0);
    unit * 	    uc = (mwu? mwu-> unit_component(): 0);

    if (!mv) return 0;

    /* Validation code could check to make sure that this is one of
     * the proper select types 
     */
    double val = mv-> getDouble();

    /* do conversions to the desired units. */
    // return convert_measure_units (val, uc, (u? u: dflt_unit_pressure));
    return val;
}

double get_measure_value (representation *r, const char *t, StixUnit u)
{
    return get_measure_value(get_measure_rep_item (r, t), u);
}


//  double get_upper_limit_value (measure_with_unit *, STIX_OPTIONAL_UNIT);
//  double get_lower_limit_value (measure_with_unit *, STIX_OPTIONAL_UNIT);
//  double get_precision_value (measure_with_unit *);

double get_measure_upper_limit_value (measure_with_unit * mwu, StixUnit u)
{
    /* Look for a qualified rep item.  Also could do the non-repitem
     * way. Put that in later
     */
    if (!mwu || !mwu-> isa(ROSE_DOMAIN(qualified_representation_item)))
	return 0;

    /* Use the late bound access in case we have a new qual rep item
     * AND/OR that doesn't have a class compiled in 
     */
    SetOfvalue_qualifier * quals = 
	ROSE_CAST(SetOfvalue_qualifier, mwu-> getObject("qualifiers"));
    if (!quals) return 0;

    /* search the list for a standard uncertainty with measure_name */
    unsigned i,sz;

    for (i=0, sz=quals->size(); i<sz; i++)
    {
	RoseObject * obj = get_select_object (quals->get(i));
	if (!obj-> isa(ROSE_DOMAIN(standard_uncertainty)))
	    continue;

	standard_uncertainty * su = ROSE_CAST(standard_uncertainty,obj);
	if (!su-> measure_name()) continue;
	if (strcmp(su-> measure_name(), "upper limit")) 
	    continue;

	return su-> uncertainty_value();
    }
    return 0;
}

double get_measure_lower_limit_value (measure_with_unit * mwu, StixUnit u)
{
    if (!mwu || !mwu-> isa(ROSE_DOMAIN(qualified_representation_item)))
	return 0;

    /* Use the late bound access in case we have a new qual rep item
     * AND/OR that doesn't have a class compiled in 
     */
    SetOfvalue_qualifier * quals = 
	ROSE_CAST(SetOfvalue_qualifier, mwu-> getObject("qualifiers"));
    if (!quals) return 0;

    /* search the list for a standard uncertainty with measure_name */
    unsigned i,sz;

    for (i=0, sz=quals->size(); i<sz; i++)
    {
	RoseObject * obj = get_select_object (quals->get(i));
	if (!obj-> isa(ROSE_DOMAIN(standard_uncertainty)))
	    continue;

	standard_uncertainty * su = ROSE_CAST(standard_uncertainty,obj);
	if (!su-> measure_name()) continue;
	if (strcmp(su-> measure_name(), "lower limit")) 
	    continue;

	return su-> uncertainty_value();
    }
    return 0;
}


double get_measure_precision_value (measure_with_unit * mwu)
{
    if (!mwu || !mwu-> isa(ROSE_DOMAIN(qualified_representation_item)))
	return 0;

    /* Use the late bound access in case we have a new qual rep item
     * AND/OR that doesn't have a class compiled in 
     */
    SetOfvalue_qualifier * quals = 
	ROSE_CAST(SetOfvalue_qualifier, mwu-> getObject("qualifiers"));
    if (!quals) return 0;

    /* search the list for a standard uncertainty with measure_name */
    unsigned i,sz;

    for (i=0, sz=quals->size(); i<sz; i++)
    {
	RoseObject * obj = get_select_object (quals->get(i));
	if (!obj-> isa(ROSE_DOMAIN(precision_qualifier)))
	    continue;

	precision_qualifier * pc = ROSE_CAST(precision_qualifier,obj);
	return pc-> precision_value();
    }
    return 0;
}




////////////////////////////////////////
double get_tolerance_nominal (plus_minus_tolerance * tol, StixUnit u)
{
    dimensional_characteristic_representation * dcr = 0; 
    RoseObject * obj = (tol? get_select_object 
			(tol-> toleranced_dimension()) : 0);

    /* is there ever going to be more than one */
    if (!obj) return 0;
    if (obj-> isa(ROSE_DOMAIN(dimensional_size)))
    {
	dimensional_size * ds = ROSE_CAST(dimensional_size,obj);
	dcr = ds-> m_dcrs;
    }
    if (obj-> isa(ROSE_DOMAIN(dimensional_location)))
    {
	dimensional_location * dl = ROSE_CAST(dimensional_location,obj);
	dcr = dl-> m_dcrs;
    }

    /* special rep subtype used by this guy */
    shape_dimension_representation * rep = (dcr? dcr-> representation(): 0);
    return get_measure_value (rep, 0, u);
}


double get_tolerance_nominal (geometric_tolerance * gt, StixUnit u)
{
    measure_with_unit * mwu = (gt? gt-> magnitude(): 0);
    return get_measure_value (mwu, u);
}



double get_tolerance_upper (plus_minus_tolerance * tol, StixUnit u)
{
    RoseObject * obj = (tol? get_select_object (tol-> range()) : 0);
    if (obj && obj-> isa(ROSE_DOMAIN(tolerance_value)))
    {
	tolerance_value * tv = ROSE_CAST(tolerance_value,obj);
	return get_measure_value (tv-> upper_bound(), u);
    }

    /* could also be a limits and fits, ignore */
    return 0;
}

double get_tolerance_upper (geometric_tolerance * gt,  StixUnit u)
{
    measure_with_unit * mwu = (gt? gt-> magnitude(): 0);
    return get_measure_upper_limit_value (mwu, u);
}


double get_tolerance_lower (plus_minus_tolerance * tol, StixUnit u)
{
    RoseObject * obj = (tol? get_select_object (tol-> range()) : 0);
    if (obj && obj-> isa(ROSE_DOMAIN(tolerance_value)))
    {
	tolerance_value * tv = ROSE_CAST(tolerance_value,obj);
	return get_measure_value (tv-> lower_bound(), u);
    }

    /* could also be a limits and fits, ignore */
    return 0;
}

double get_tolerance_lower (geometric_tolerance * gt,  StixUnit u)
{
    measure_with_unit * mwu = (gt? gt-> magnitude(): 0);
    return get_measure_lower_limit_value (mwu, u);
}


double get_tolerance_precision (geometric_tolerance * gt)
{
    measure_with_unit * mwu = (gt? gt-> magnitude(): 0);
    return get_measure_precision_value (mwu);
}


datum * get_tolerance_datum (geometric_tolerance * gt, int i)
{

    // datums are kept by all of the things that are direct subtypes
    // of gtol_with_datum, plus a few extra types that are and/ored in
    // as needed.  In particular, position, line profile, and surface
    // profile tolerances.

    if (!gt) return 0;
    if (!gt-> isa(ROSE_DOMAIN(geometric_tolerance_with_datum_reference)))
	return 0;
	
    geometric_tolerance_with_datum_reference * gtd = 
	ROSE_CAST(geometric_tolerance_with_datum_reference,gt);

    SetOfdatum_reference * refs = gtd->datum_system();
    if (!refs || !refs->size()) 
	return 0;

    if ((i < 0) || (i >= refs->size()))
	return 0;

    return refs-> get(i)-> referenced_datum();
}


limit_condition get_tolerance_limit_condition (
    geometric_tolerance * gt
    )
{
    if (!gt-> isa(ROSE_DOMAIN(modified_geometric_tolerance)))
	return limit_condition_NULL;

    modified_geometric_tolerance * mt = 
	ROSE_CAST(modified_geometric_tolerance, gt);

    return mt-> modifier();
}



/* gets the plane or unbounded surface */
geometric_representation_item * get_datum_geometry (datum * d)
{
    return get_aspect_shape_geometry (d);
}

/* gets the face on the part that defines the datum */
topological_representation_item * get_datum_face (datum * d)
{
    /* look for datum feature associated with this, 
       grab face from that 
    */
    /* FIXME -- Extend the datum class to hold the sars that relate
     * this to the datum_feature.  For now, just brute force it. 
     */

    RoseCursor objs;
    RoseObject * obj;

    if (!d) return 0;

    /* Traverse the shape defining relationships and put in
     * backpointers to the feature that they apply. 
     */
    objs.traverse (d->design());
    objs.domain (ROSE_DOMAIN(shape_aspect_relationship));
    while (obj=objs.next()) {
	shape_aspect_relationship * sar =
	    ROSE_CAST(shape_aspect_relationship,obj);

	/* datum is in the -ed side */
	/* datum_feature is in the -ing side */
	if (sar->related_shape_aspect() != d) 
	    continue;

	if (!sar->relating_shape_aspect() || 
	    !sar->relating_shape_aspect()-> isa (ROSE_DOMAIN(datum_feature)))
	    continue;

	return get_aspect_shape_face (sar->relating_shape_aspect());
    }
    return 0;
}



/* ============================================================
 * SETUP FUNCTIONS
 */

/* Some complex instance utility functions */
extern "C" int stix_datum_reference_compare (const void * a, const void * b)
{
    datum_reference * ent_a = (*(datum_reference **) a);
    datum_reference * ent_b = (*(datum_reference **) b);

    int prec_a = (ent_a? ent_a-> precedence(): 99);
    int prec_b = (ent_b? ent_b-> precedence(): 99);

    /* sort smallest to largest */
    return (prec_a - prec_b);
}

// void tolerance_calc_extra() if needed
void StixNC::classify_tolerances()
{
    RoseCursor objs;
    RoseObject * obj;

    /* Like the product definitions, shape aspects are used for many
     * different purposes.
     */
    if (m_verbose) fprintf (stdout, "Classifying tolerances\n");


    /* Start with the dimensional tolerances */
    objs.traverse (m_design);
    objs.domain (ROSE_DOMAIN(plus_minus_tolerance));
    while (obj=objs.next()) {
	plus_minus_tolerance * tol = 
	    ROSE_CAST(plus_minus_tolerance,obj);

	/* reduce types to an enum for simplicity */
	tol-> m_type = calc_tolerance_type(tol);
    }

    /* Then do the geometric tolerances */
    objs.domain (ROSE_DOMAIN(geometric_tolerance));
    while (obj=objs.next()) {
	geometric_tolerance * tol = 
	    ROSE_CAST(geometric_tolerance,obj);

	/* reduce types to an enum for simplicity */
	tol-> m_type = calc_tolerance_type(tol);
    }

    /* Hook up the characteristic representations for the nominal
     * values.  These point to one of the two dimension types, so 
     * we must hook them to the appropriate type.
     */
    objs.domain (ROSE_DOMAIN(dimensional_characteristic_representation));
    while (obj=objs.next()) {
	dimensional_characteristic_representation * dcr = 
	    ROSE_CAST(dimensional_characteristic_representation,obj);

	/* hook up property to correct type or object */
	RoseObject * targ = get_select_object(dcr-> dimension());

	if (!targ) continue;  /* bad shape def */
	if (targ-> isa(ROSE_DOMAIN(dimensional_size)))
	{
	    dimensional_size * ds = 
		ROSE_CAST(dimensional_size,targ);

	    dcr-> m_next_dcr = ds->m_dcrs;
	    ds->m_dcrs = dcr;
	}
	if (targ-> isa(ROSE_DOMAIN(dimensional_location)))
	{
	    dimensional_location * dl = 
		ROSE_CAST(dimensional_location,targ);

	    dcr-> m_next_dcr = dl->m_dcrs;
	    dl->m_dcrs = dcr;
	}
    }

    /* for convenience sake, we want to sort all of the datum
     * references in order of precedence.  It makes it easy to 
     * get the correct ones afterwards
     */
    objs.domain (ROSE_DOMAIN(geometric_tolerance_with_datum_reference));
    while (obj=objs.next()) 
    {
	geometric_tolerance_with_datum_reference * gt = 
	    ROSE_CAST(geometric_tolerance_with_datum_reference,obj);

	SetOfdatum_reference * refs = gt->datum_system();
	if (!refs || !refs->size()) continue;

	qsort (refs->_data(), refs->size(), sizeof(datum_reference*),
	       stix_datum_reference_compare);
    }
}




/* overload the function so that it works with both dimensional 
 * and geometric tolerances 
 */
unsigned calc_tolerance_type (plus_minus_tolerance * tol)
{
    if (!tol) return 0;

    RoseObject * obj = get_select_object 
	(tol-> toleranced_dimension());

    if (!obj) return 0;
    if (obj-> isa(ROSE_DOMAIN(dimensional_size)))
    {
	/* check for the more specific types */
	if (obj-> isa(ROSE_DOMAIN(angular_size)))
	    return DTOL_SIZE_ANGULAR;

	if (obj-> isa(ROSE_DOMAIN(dimensional_size_with_path)))
	    return DTOL_SIZE_PATH;

	return DTOL_SIZE;  /* not a recognized subtype */
    }

    if (obj-> isa(ROSE_DOMAIN(dimensional_location)))
    {
	/* check for the more specific types */
	if (obj-> isa(ROSE_DOMAIN(angular_location)))
	    return DTOL_LOC_ANGULAR;

	if (obj-> isa(ROSE_DOMAIN(dimensional_location_with_path)))
	    return DTOL_LOC_PATH;

	return DTOL_LOC;  /* not a recognized subtype */
    }
    return 0;
}


unsigned calc_tolerance_type (geometric_tolerance * tol)
{
    if (!tol) return 0;
    char * nm = (tol->name()? tol-> name(): "");

    /* The modified and with_datum_refence types are sometimes used as
     * complex instance mix-in type so we check for them last.  Also, a 
     * name string is sometimes used instead of one of the subtypes so 
     * we check for that too..
     */

    /* check for the more specific types */
    if (tol-> isa(ROSE_DOMAIN(cylindricity_tolerance)) ||
	!strcmp (nm, "cylindricity"))
	return GTOL_CYLINDRICITY;

    if (tol-> isa(ROSE_DOMAIN(flatness_tolerance)) ||
	!strcmp (nm, "flatness"))
	return GTOL_FLATNESS;

    if (tol-> isa(ROSE_DOMAIN(line_profile_tolerance)) ||
	!strcmp (nm, "line profile"))
	return GTOL_LINE_PROFILE;

    if (tol-> isa(ROSE_DOMAIN(position_tolerance)) ||
	!strcmp (nm, "position"))
	return GTOL_POSITION;

    if (tol-> isa(ROSE_DOMAIN(roundness_tolerance)) ||
	!strcmp (nm, "roundness"))
	return GTOL_ROUNDNESS;

    if (tol-> isa(ROSE_DOMAIN(straightness_tolerance)) ||
	!strcmp (nm, "straightness"))
	return GTOL_STRAIGHTNESS;

    if (tol-> isa(ROSE_DOMAIN(surface_profile_tolerance)) ||
	!strcmp (nm, "surface profile"))
	return GTOL_SURFACE_PROFILE;

    /* -------------------- */
    /* DATUM TOLERANCES -- All tolerances below have datum refs */
    if (tol-> isa(ROSE_DOMAIN(angularity_tolerance)) ||
	!strcmp (nm, "angularity"))
	return GTOL_ANGULARITY;

    if (tol-> isa(ROSE_DOMAIN(circular_runout_tolerance)) ||
	!strcmp (nm, "circular runout"))
	return GTOL_CIRCULAR_RUNOUT;

    if (tol-> isa(ROSE_DOMAIN(coaxiality_tolerance)) ||
	!strcmp (nm, "coaxiality"))
	return GTOL_COAXIALITY;

    if (tol-> isa(ROSE_DOMAIN(concentricity_tolerance)) ||
	!strcmp (nm, "concentricity"))
	return GTOL_CONCENTRICITY;

    if (tol-> isa(ROSE_DOMAIN(parallelism_tolerance)) ||
	!strcmp (nm, "parallelism"))
	return GTOL_PARALLELISM;

    if (tol-> isa(ROSE_DOMAIN(perpendicularity_tolerance)) ||
	!strcmp (nm, "perpendicularity"))
	return GTOL_PERPENDICULARITY;

    if (tol-> isa(ROSE_DOMAIN(symmetry_tolerance)) ||
	!strcmp (nm, "symmetry"))
	return GTOL_SYMMETRY;

    if (tol-> isa(ROSE_DOMAIN(total_runout_tolerance)) ||
	!strcmp (nm, "total runout"))
	return GTOL_TOTAL_RUNOUT;

    /* -------------------- */
    /* unknown type with datums */
    if (tol-> isa(ROSE_DOMAIN(geometric_tolerance_with_datum_reference)))
	return GTOL_UNKNOWN_DATUM;

    // What do we do with
    // geometric_tolerance_with_defined_unit

    /* check for this last, in case we haven't seen anything else */
    if (tol-> isa(ROSE_DOMAIN(modified_geometric_tolerance)))
	return GTOL_MODIFIED;

    return GTOL_UNKNOWN;
}








