/* $RCSfile: tolerance.h,v $
 * $Revision: 1.4 $ $Date: 2003/08/20 20:04:55 $
 * Auth: Dave Loffredo (loffredo@steptools.com)
 * 
 * Copyright (c) 1991-2003 by STEP Tools Inc. 
 * All Rights Reserved.
 * 
 * This file is part of the STIX Library
 * 
 * This file may be distributed and/or modified under the terms of 
 * the GNU General Public License version 2 as published by the Free
 * Software Foundation and appearing in the file LICENSE.GPL included
 * with this file.
 * 
 * THIS FILE IS PROVIDED "AS IS" WITH NO WARRANTY OF ANY KIND,
 * INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * STEP Tools Inc. also offers licenses for commercial use of this
 * software in non-GPL applications.  Contact info@steptools.com for
 * terms and pricing.
 * 
 * 		----------------------------------------
 */

class StixGeomToleranceCursor {
 private:
    StixNC * 		m_nc;
    RoseDesignCursor	m_objs;

 public:
    StixGeomToleranceCursor (StixNC * nc);

    /* Returns null when no more objects */
    geometric_tolerance * next();

    /* reset traversal to beginning */
    void rewind();

    /* This is not for loop guards.  Use null return from next() to
     * signal no more objects.  This computes the size, it does not
     * just return a stored value. 
     */
    unsigned size();
};


class StixDimToleranceCursor {
 private:
    StixNC * 		m_nc;
    RoseDesignCursor	m_objs;

 public:
    StixDimToleranceCursor (StixNC * nc);

    /* Returns null when no more objects */
    plus_minus_tolerance * next();

    /* reset traversal to beginning */
    void rewind();

    /* This is not for loop guards.  Use null return from next() to
     * signal no more objects.  This computes the size, it does not
     * just return a stored value. 
     */
    unsigned size();
};


class StixDatumCursor {
 private:
    StixNC * 		m_nc;
    RoseDesignCursor	m_objs;

 public:
    StixDatumCursor (StixNC * nc);

    /* Returns null when no more objects */
    datum * next();

    /* reset traversal to beginning */
    void rewind();

    /* This is not for loop guards.  Use null return from next() to
     * signal no more objects.  This computes the size, it does not
     * just return a stored value. 
     */
    unsigned size();
};



/* ========================================
 * Tolerance Types - This returns a code to make processing the
 * different tolerances easier.
 */
#define DTOL_BASE		0x0100	/* no more than 255 */
#define DTOL_UNKNOWN		(DTOL_BASE)
#define DTOL_SIZE		(DTOL_BASE+1)
#define DTOL_SIZE_PATH		(DTOL_BASE+2)
#define DTOL_SIZE_ANGULAR	(DTOL_BASE+3)
#define DTOL_LOC		(DTOL_BASE+4)
#define DTOL_LOC_PATH		(DTOL_BASE+5)
#define DTOL_LOC_ANGULAR	(DTOL_BASE+6)

#define GTOL_BASE		0x0200	/* no more than 255 */
#define GTOL_UNKNOWN		(GTOL_BASE)
#define GTOL_CYLINDRICITY	(GTOL_BASE+1)
#define GTOL_FLATNESS		(GTOL_BASE+2)
#define GTOL_LINE_PROFILE	(GTOL_BASE+3)
#define GTOL_MODIFIED		(GTOL_BASE+4)
#define GTOL_POSITION		(GTOL_BASE+5)
#define GTOL_ROUNDNESS		(GTOL_BASE+6)
#define GTOL_STRAIGHTNESS	(GTOL_BASE+7)
#define GTOL_SURFACE_PROFILE	(GTOL_BASE+8)

/* All tolerances below have datum refs */
#define GTOL_UNKNOWN_DATUM	(GTOL_BASE+9)
#define GTOL_ANGULARITY		(GTOL_BASE+10)
#define GTOL_CIRCULAR_RUNOUT	(GTOL_BASE+11)
#define GTOL_COAXIALITY		(GTOL_BASE+12)
#define GTOL_CONCENTRICITY	(GTOL_BASE+13)
#define GTOL_PARALLELISM	(GTOL_BASE+14)
#define GTOL_PERPENDICULARITY	(GTOL_BASE+15)
#define GTOL_SYMMETRY		(GTOL_BASE+16)
#define GTOL_TOTAL_RUNOUT	(GTOL_BASE+17)

/* overload the function so that it works with both dimensional 
 * and geometric tolerances 
 */
unsigned get_tolerance_type (plus_minus_tolerance *);
unsigned get_tolerance_type (geometric_tolerance *);

char * get_tolerance_typename (plus_minus_tolerance *);
char * get_tolerance_typename (geometric_tolerance *);

char * get_tolerance_name (plus_minus_tolerance *);
char * get_tolerance_name (geometric_tolerance *);


/* Only used for initial classification */
unsigned calc_tolerance_type (plus_minus_tolerance *);
unsigned calc_tolerance_type (geometric_tolerance *);


double get_tolerance_nominal (
    plus_minus_tolerance *, STIX_OPTIONAL_UNIT
    );
double get_tolerance_nominal (
    geometric_tolerance *, STIX_OPTIONAL_UNIT
    );


double get_tolerance_upper (
    plus_minus_tolerance *, STIX_OPTIONAL_UNIT
    );
double get_tolerance_upper (
    geometric_tolerance *, STIX_OPTIONAL_UNIT
    );


double get_tolerance_lower (
    plus_minus_tolerance *, STIX_OPTIONAL_UNIT
    );
double get_tolerance_lower (
    geometric_tolerance *, STIX_OPTIONAL_UNIT
    );

double get_tolerance_precision (geometric_tolerance * gt);


limit_condition get_tolerance_limit_condition (geometric_tolerance * gt);


topological_representation_item * get_tolerance_face (
    plus_minus_tolerance *
    );
topological_representation_item * get_tolerance_face (
    geometric_tolerance * tol
    );

topological_representation_item * get_tolerance_other_face (
    plus_minus_tolerance *
    );


datum * get_tolerance_datum (
    geometric_tolerance * tol, int prec
    );


/* eventually break these out */
shape_representation * get_aspect_shape_rep (
    shape_aspect * sa
    );
topological_representation_item * get_aspect_shape_face (
    shape_aspect * sa
    );
geometric_representation_item * get_aspect_shape_geometry (
    shape_aspect * sa
    );



/* gets the plane or unbounded surface for the datum */
geometric_representation_item * get_datum_geometry (
    datum * d
    );

/* gets the face on the part that defines the datum */
topological_representation_item * get_datum_face (
    datum * d
    );






//  representation * get_tolerance_shape (
//      geometric_tolerance * tol
//      );

//  representation * get_tolerance_representation (
//      geometric_tolerance * tol
//      );

