/* $RCSfile: tool.cxx,v $
 * $Revision: 1.1 $ $Date: 2003/03/25 19:31:12 $
 * Auth: Dave Loffredo (loffredo@steptools.com)
 * 
 * Copyright (c) 1991-2003 by STEP Tools Inc. 
 * All Rights Reserved.
 * 
 * This file is part of the STIX Library
 * 
 * This file may be distributed and/or modified under the terms of 
 * the GNU General Public License version 2 as published by the Free
 * Software Foundation and appearing in the file LICENSE.GPL included
 * with this file.
 * 
 * THIS FILE IS PROVIDED "AS IS" WITH NO WARRANTY OF ANY KIND,
 * INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * STEP Tools Inc. also offers licenses for commercial use of this
 * software in non-GPL applications.  Contact info@steptools.com for
 * terms and pricing.
 * 
 * 		----------------------------------------
 */

#include <rose.h>
#include <integrated_cnc_schema.h>
#include "stix_imp.h"

/* Tools functions */
machining_tool_body_representation * get_tool_body (machining_tool * mt)
{
    resource_property_representation * rpr = (mt? mt->m_props: 0);
    representation * rep;			    
    char * pname;

    while (rpr) {
	pname = (rpr-> property()? rpr-> property()-> name(): 0);

	/* This should be "tool body" */
	if (pname && !strcmp(pname, "tool_body")) {
	    rep = rpr-> att_representation();
	    if (!rep-> isa (ROSE_DOMAIN(machining_tool_body_representation)))
		/* print a warning too */
		return 0;

	    return ROSE_CAST(machining_tool_body_representation, rep);
	}
	rpr = rpr-> m_next_rpr;
    }
    return 0;
}


/* Tools functions */
double 	get_tool_diameter (
    machining_tool * mt, StixUnit u
    )
{
    return get_length_value(get_tool_body (mt), "diameter", u);
}


double 	get_tool_edge_length (
    machining_tool * mt, StixUnit u
    )
{
    /* this is the incorrect string.  Fix when the ST-Plan 
     * output is corrected 
     */
    return get_length_value(get_tool_body (mt), "flute_length", u);
}


double 	get_tool_edge_radius (
    machining_tool * mt, StixUnit u
    ) 
{
    /* this is the incorrect string.  Fix when the ST-Plan 
     * output is corrected 
     */
    return get_length_value(get_tool_body (mt), "corner_radius", u);
}


double 	get_tool_circumference_angle (
    machining_tool * mt, StixUnit u
    ) 
{
    /* this is the incorrect string.  Fix when the ST-Plan 
     * output is corrected 
     */
    return get_angle_value(get_tool_body (mt), "tool_circumference_angle", u);
}
