/* $RCSfile: utils.cxx,v $
 * $Revision: 1.22 $ $Date: 2003/05/21 14:56:53 $
 * Auth: Dave Loffredo (loffredo@steptools.com)
 * 
 * Copyright (c) 1991-2003 by STEP Tools Inc. 
 * All Rights Reserved.
 * 
 * This file is part of the STIX Library
 * 
 * This file may be distributed and/or modified under the terms of 
 * the GNU General Public License version 2 as published by the Free
 * Software Foundation and appearing in the file LICENSE.GPL included
 * with this file.
 * 
 * THIS FILE IS PROVIDED "AS IS" WITH NO WARRANTY OF ANY KIND,
 * INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * STEP Tools Inc. also offers licenses for commercial use of this
 * software in non-GPL applications.  Contact info@steptools.com for
 * terms and pricing.
 * 
 * 		----------------------------------------
 */

#include <rose.h>
#include <integrated_cnc_schema.h>
#include "stix_imp.h"


/* little helper class for indenting the output 
 * of the dump functions
 */

StixIndent::StixIndent() {
    init();
}
void StixIndent::init() {
    m_tabs[0] = 0;
}
void StixIndent::inc() {
    int sz = strlen(m_tabs);
    if (sz > 253) return;
    m_tabs[sz] = '\t';
    m_tabs[sz+1] = 0;
}
void StixIndent::dec() {
    int sz = strlen(m_tabs);
    if (!sz) return;
    m_tabs[sz-1] = 0;
}
void StixIndent::print() {
    printf(m_tabs);
}

char* StixIndent::string() { 
    return m_tabs; 
}


/* ==================================================
 * UTILITIES
 */
/* need some sort of adb type thing */
RoseObject * get_select_object (RoseUnion * sel)
{
    RoseAttribute * att;
    RoseObject * obj;

    if (!sel) return 0;

    att = sel-> getAttribute();
    if (!att || !att->isObject()) return 0;

    obj = sel-> getObject();
    if (!obj) return 0;
    if (obj-> domain()->typeIsSelect())
	return get_select_object ((RoseUnion*)obj);

    return obj;
}


axis2_placement_3d * get_placement (
    representation * 	rep, 
    const char * 	nm
    )
{
    representation_item * it = get_representation_item 
	(rep, ROSE_DOMAIN(axis2_placement_3d), nm);

    return (it? ROSE_CAST(axis2_placement_3d,it): 0);
}



direction * get_direction (
    representation * 	rep, 
    const char * 	nm
    )
{
    representation_item * it = get_representation_item 
	(rep, ROSE_DOMAIN(direction), nm);

    return (it? ROSE_CAST(direction,it): 0);
}


product_definition * property_get_pdef (
    property_definition_representation * pdr
    )
{
    /* the represented_definition selects can be a pdef,
     * shape aspect, or shape aspect relationship.
     */

    if (!pdr->definition()) return 0;
    if (pdr->definition()-> is_property_definition()) {
	return property_get_pdef (
	    pdr->definition()-> _property_definition());
    }
    return 0;
}


product_definition * property_get_pdef (
    property_definition * pd
    )
{

    /* a characterized definition could be 
       characterized_object,
       product_definition,
       product_definition_relationship
       product_definition_shape	  (type of property def)
       shape_aspect, 
       shape_aspect_relationship

    */
    RoseObject * def = get_select_object (pd->definition());
    if (!def) return 0;

    if (def-> isa (ROSE_DOMAIN(product_definition)))
	return ROSE_CAST (product_definition, def);

    return 0;
}



RoseObject * get_property_owner (
    property_definition_representation * pdr
    )
{
    RoseObject * obj;

    /* We are probably looking for a property definition, but they
     * have recently made this a select, so it could also be a shape 
     * aspect or SAR.
     */
    if (!pdr || !pdr->definition()) return 0;

    obj = get_select_object (pdr->definition());
    if (obj && obj-> isa (ROSE_DOMAIN(property_definition)))
    {
	/* one more level down */
	property_definition * pd = ROSE_CAST(property_definition,obj);
	obj = get_select_object (pd->definition());
    }
    return obj;
}




representation * get_action_property_rep (
    action_method * am,
    const char * prop_name
    )
{
    action_property_representation * apr = (am? am->m_props: 0);
    action_property * ap;
    char * ap_name;

    while (apr) {
	ap = apr-> property();
	ap_name = (ap? ap-> name(): 0);

	/* This should be "tool body" */
	if (ap_name && !strcmp(ap_name, prop_name)) {
	    return apr-> att_representation();
	}
	apr = apr-> m_next_apr;
    }
    return 0;
}


face_shape_representation * get_face_shape_property (
    shape_aspect * sa, const char* rep_nm
    )
{
    property_definition_representation * pdr = (sa? sa-> m_props: 0);
    char * s;

    while (pdr) {
	representation * rep = pdr-> used_representation();

	/* search the properties for a direction shape rep */
	if (!rep || !rep-> isa(ROSE_DOMAIN(face_shape_representation))) {
	    pdr = pdr-> m_next_pdr;
	    continue;
	}

	/* test against the name */
	s = rep-> name();
	if (rep_nm && (!s || strcmp (rep_nm, s))) {
	    pdr = pdr-> m_next_pdr;
	    continue;
	}

	return ROSE_CAST(face_shape_representation,rep);
    }
    return 0;
}


planar_shape_representation * get_planar_shape_property (
    shape_aspect * sa, const char* rep_nm
    )
{
    property_definition_representation * pdr = (sa? sa-> m_props: 0);
    char * s;

    while (pdr) {
	representation * rep = pdr-> used_representation();

	/* search the properties for a direction shape rep */
	if (!rep || !rep-> isa(ROSE_DOMAIN(planar_shape_representation))) {
	    pdr = pdr-> m_next_pdr;
	    continue;
	}

	/* test against the name */
	s = rep-> name();
	if (rep_nm && (!s || strcmp (rep_nm, s))) {
	    pdr = pdr-> m_next_pdr;
	    continue;
	}

	return ROSE_CAST(planar_shape_representation,rep);
    }
    return 0;
}


direction_shape_representation * get_direction_shape_property (
    shape_aspect * sa, const char* rep_nm
    )
{
    property_definition_representation * pdr = (sa? sa-> m_props: 0);
    char * s;

    while (pdr) {
	representation * rep = pdr-> used_representation();

	/* search the properties for a direction shape rep */
	if (!rep || !rep-> isa(ROSE_DOMAIN(direction_shape_representation))) {
	    pdr = pdr-> m_next_pdr;
	    continue;
	}

	/* test against the name */
	s = rep-> name();
	if (rep_nm && (!s || strcmp (rep_nm, s))) {
	    pdr = pdr-> m_next_pdr;
	    continue;
	}

	return ROSE_CAST(direction_shape_representation,rep);
    }
    return 0;
}

#ifdef BUG_PATH_SHAPE_REP_NOT_USED
shape_representation * 
#else
path_shape_representation * 
#endif
get_path_shape_property (
    shape_aspect * sa, const char* rep_nm
    )
{
    property_definition_representation * pdr = (sa? sa-> m_props: 0);
    char * s;

    while (pdr) {
	representation * rep = pdr-> used_representation();

	/* search the properties for a direction shape rep */
	if (!rep || !rep-> isa(ROSE_DOMAIN(path_shape_representation))) {
	    pdr = pdr-> m_next_pdr;
	    continue;
	}

	/* test against the name */
	s = rep-> name();
	if (rep_nm && (!s || strcmp (rep_nm, s))) {
	    pdr = pdr-> m_next_pdr;
	    continue;
	}

	return ROSE_CAST(path_shape_representation,rep);
    }

#ifdef BUG_PATH_SHAPE_REP_NOT_USED
    /* The shape for general profiles are supposed to be written out
     * as a path_shape_representation.  For some strange reason, they
     * are being written as a shape_representation_with_parameters 
     * instead.  
     * 
     * Not only does this not match the spec, it breaks the where rules 
     * on the SRWP, since it is not allowed to contain a path. 
     */

    /* Look through the properties again and see if we have one that
     * contains an instance of path
     */

    pdr = (sa? sa-> m_props: 0);
    while (pdr) {
	representation * rep = pdr-> used_representation();

	if (!rep || !rep-> isa(ROSE_DOMAIN(shape_representation))) {
	    pdr = pdr-> m_next_pdr;
	    continue;
	}

	/* test against the name */
	s = rep-> name();
	if (rep_nm && (!s || strcmp (rep_nm, s))) {
	    pdr = pdr-> m_next_pdr;
	    continue;
	}

	/* look for a path instance */
	if (!get_representation_item (rep, ROSE_DOMAIN(path))) {
	    pdr = pdr-> m_next_pdr;
	    continue;
	}

	return ROSE_CAST(shape_representation,rep);
    }
#endif
    return 0;
}





/* ---------------------------------------- */
/* FIND EXTERNAL IDENTIFIER */
/* ---------------------------------------- */
unsigned get_project_exid (product * p)
{
    return ((p && p->m_idx)? p->m_idx->m_exid: 0);
}
unsigned get_setup_exid (product * p)
{
    return ((p && p->m_idx)? p->m_idx->m_exid: 0);
}

unsigned get_workpiece_exid (product_definition * pdef)
{
    return ((pdef && pdef->m_idx)? pdef->m_idx->m_exid: 0);
}
unsigned get_feature_exid (shape_aspect * sa)
{	
    return ((sa && sa->m_idx)? sa->m_idx->m_exid: 0);
}
unsigned get_component_exid (shape_aspect * sa)
{	
    return ((sa && sa->m_idx)? sa->m_idx->m_exid: 0);
}
unsigned get_exec_exid (machining_process_executable * mpe)
{	
    return (mpe? mpe-> m_exid: 0);
}
unsigned get_tool_exid (machining_tool * mt)
{	
    return (mt? mt-> m_exid: 0);
}



/* have different functions for casting and typing */
StixProject * get_project_idx (product * p)
{
    /* could put in test code to verify type based on exid */
    return (StixProject*) (p? p->m_idx: 0);
}
StixSetup * get_setup_idx   (product * p)
{
    /* could put in test code to verify type based on exid */
    return (StixSetup*) (p? p->m_idx: 0);
}
StixWorkpiece * get_workpiece_idx (product_definition * pdef)
{
    /* no need for a test since only workpieces set this */
    return (StixWorkpiece*) (pdef? pdef->m_idx: 0);
}
StixFeature * 	get_feature_idx   (shape_aspect * sa)
{
    /* could put in test code to verify type based on exid */
    return (StixFeature*) (sa? sa->m_idx: 0);
}
StixComponent * get_component_idx (shape_aspect * sa)
{
    /* could put in test code to verify type based on exid */
    return (StixComponent*) (sa? sa->m_idx: 0);
}

