/* $RCSfile: workpiece.cxx,v $
 * $Revision: 1.1 $ $Date: 2003/03/25 19:31:12 $
 * Auth: Dave Loffredo (loffredo@steptools.com)
 * 
 * Copyright (c) 1991-2003 by STEP Tools Inc. 
 * All Rights Reserved.
 * 
 * This file is part of the STIX Library
 * 
 * This file may be distributed and/or modified under the terms of 
 * the GNU General Public License version 2 as published by the Free
 * Software Foundation and appearing in the file LICENSE.GPL included
 * with this file.
 * 
 * THIS FILE IS PROVIDED "AS IS" WITH NO WARRANTY OF ANY KIND,
 * INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * STEP Tools Inc. also offers licenses for commercial use of this
 * software in non-GPL applications.  Contact info@steptools.com for
 * terms and pricing.
 * 
 * 		----------------------------------------
 */

#include <rose.h>
#include <integrated_cnc_schema.h>
#include "stix_imp.h"

/* ==================================================
 * Workpiece Functions
 */
shape_representation * 	get_workpiece_shape (
    product_definition * pdef
    )
{
    StixWorkpiece * idx = get_workpiece_idx(pdef);
    return (idx? idx->m_shape: 0);
}

product_definition * get_workpiece_stock (
    product_definition * pdef
    )
{
    StixWorkpiece * idx = get_workpiece_idx(pdef);
    return (idx? idx-> m_stock: 0);
}

shape_representation * 	get_workpiece_stock_shape (
    product_definition * pdef
    )
{
    StixWorkpiece * idx = get_workpiece_idx(pdef);
    return (idx? get_workpiece_shape (idx-> m_stock): 0);
}

unsigned get_workpiece_stock_limit_id (product_definition * pd)
{
    /* is this useful?  In the mappings it just returns the 
     * entity id of the product
     */
    product * p = (pd->formation()? pd->formation()->of_product(): 0);
    return (p? p-> entity_id(): 0);
}

double 	get_workpiece_stock_min_x (product_definition * pd) {
    StixWorkpiece * idx = get_workpiece_idx(pd);
    return ((idx && idx->m_limits)? idx->m_limits->m_min.m_x: 0);
}

double 	get_workpiece_stock_max_x (product_definition * pd) {
    StixWorkpiece * idx = get_workpiece_idx(pd);
    return ((idx && idx->m_limits)? idx->m_limits->m_max.m_x: 0);
}


double 	get_workpiece_stock_min_y (product_definition * pd) {
    StixWorkpiece * idx = get_workpiece_idx(pd);
    return ((idx && idx->m_limits)? idx->m_limits->m_min.m_y: 0);
}

double 	get_workpiece_stock_max_y (product_definition * pd) {
    StixWorkpiece * idx = get_workpiece_idx(pd);
    return ((idx && idx->m_limits)? idx->m_limits->m_max.m_y: 0);
}


double 	get_workpiece_stock_min_z (product_definition * pd) {
    StixWorkpiece * idx = get_workpiece_idx(pd);
    return ((idx && idx->m_limits)? idx->m_limits->m_min.m_z: 0);
}

double 	get_workpiece_stock_max_z (product_definition * pd) {
    StixWorkpiece * idx = get_workpiece_idx(pd);
    return ((idx && idx->m_limits)? idx->m_limits->m_max.m_z: 0);
}



/* this is totally hacked up and did not work in the original
 * mappings, so we just keep the same constants.  When this is set to
 * work properly, we will point to the material designation and others.
 */
/* calulation functions */
StixMaterial *   calc_material (product_definition * pd)
{
    StixWorkpiece * idx = get_workpiece_idx(pd);
    StixMaterial * sm = new StixMaterial;
    StixProperty * prop;

    material_designation * md = (idx? idx-> m_md: 0);

    if (!md) {
	/* hard coded */
	sm-> m_std_id = "ST-Machine";
	sm-> m_mtl_id = "Aluminum";

	prop = new StixProperty;
	prop-> m_name = "alloy";
	prop-> m_desc = "6061 T6";
	sm-> m_mtl_props.append (prop);

	prop = new StixProperty;
	prop-> m_name = "family";
	prop-> m_desc = "6000 Series Alloys";
	sm-> m_mtl_props.append (prop);
    }
    else {
	document * doc = (md-> m_doc? md-> m_doc-> assigned_document(): 0);
	
	if (doc && doc-> id())
	    sm-> m_std_id = doc-> id();
	else
	    sm-> m_std_id = "ST-Machine";

	sm-> m_mtl_id = md-> name();
	/* no properties done */
    }
    return sm;
}

