/* $RCSfile: workplan.cxx,v $
 * $Revision: 1.12 $ $Date: 2003/05/26 20:03:24 $
 * Auth: Dave Loffredo (loffredo@steptools.com)
 * 
 * Copyright (c) 1991-2003 by STEP Tools Inc. 
 * All Rights Reserved.
 * 
 * This file is part of the STIX Library
 * 
 * This file may be distributed and/or modified under the terms of 
 * the GNU General Public License version 2 as published by the Free
 * Software Foundation and appearing in the file LICENSE.GPL included
 * with this file.
 * 
 * THIS FILE IS PROVIDED "AS IS" WITH NO WARRANTY OF ANY KIND,
 * INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * STEP Tools Inc. also offers licenses for commercial use of this
 * software in non-GPL applications.  Contact info@steptools.com for
 * terms and pricing.
 * 
 * 		----------------------------------------
 */

#include <rose.h>
#include <integrated_cnc_schema.h>
#include "stix_imp.h"


unsigned get_exec_type (machining_process_executable * mpe)
{
    return (mpe? mpe-> m_type: 0);
}

char * get_exec_typename (machining_process_executable * mpe)
{
    switch (get_exec_type(mpe)) {
    case EXEC_NONE:		return "unrecongized executable";
    case EXEC_BAD:		return "bad executable";

    case EXEC_ASSIGN:		return "assignment";
    case EXEC_IF:		return "if statement";
    case EXEC_NONSEQ:		return "non-sequential block";
    case EXEC_PARALLEL:		return "parallel block";
    case EXEC_RAPID:		return "rapid motion";
    case EXEC_SELECT:		return "selective branch";
    case EXEC_WHILE:		return "while statement";
    case EXEC_WORKPLAN:		return "workplan";

    case EXEC_MILL_BOTSIDE: 	  return "bottom and side finish milling";
    case EXEC_MILL_BOTSIDE_ROUGH: return "bottom and side rough milling";
    case EXEC_MILL_FREEFORM:	return "freeform surface milling";
    case EXEC_MILL_PLANE:	return "plane finish milling";
    case EXEC_MILL_PLANE_ROUGH:	return "plane rough milling";
    case EXEC_MILL_SIDE:	return "side finish milling";
    case EXEC_MILL_SIDE_ROUGH:	return "side rough milling";

    case EXEC_BORE:		return "boring";
    case EXEC_BACK_BORE:	return "back boring";
    case EXEC_DRILL:		return "drilling";
    case EXEC_DRILL_CENTER:	return "center drilling";
    case EXEC_DRILL_CSINK:	return "counter sinking";
    case EXEC_DRILL_MULTISTEP:	return "multistep drilling";
    case EXEC_REAM:		return "reaming";
    case EXEC_TAP:		return "tapping";
    case EXEC_THREAD_DRILL:	return "thread drilling";

    case EXEC_MARK_SET:		return "set mark";
    case EXEC_MARK_WAIT:	return "wait for mark";
    case EXEC_MESSAGE:		return "display message";
    case EXEC_PALLET_EXCHANGE:	return "exchange pallet";
    case EXEC_PALLET_INDEX:	return "index pallet";
    case EXEC_STOP:		return "program stop";
    case EXEC_STOP_OPTIONAL:	return "optional stop";
    case EXEC_TOOL_LOAD:	return "load tool";
    case EXEC_TOOL_UNLOAD:	return "unload tool";

    case EXEC_PROBE:		return "workpiece probing";
    case EXEC_PROBE_COMPLETE:	return "workpiece complete probing";
    case EXEC_PROBE_TLEN:	return "tool length probing";
    case EXEC_PROBE_TRAD:	return "tool radius probing";

    default:
	return "unknown executable";
    }
}

unsigned calc_exec_base_type (machining_process_executable * mpe)
{
    /* This expects the operations and other associated information to
     * be present, so we can only do this in the later calculation stage.
     */
    char * desc;
    if (!mpe) return EXEC_NONE;

    if (mpe-> isa(ROSE_DOMAIN(machining_workingstep)))
    {
	machining_operation * op = get_workingstep_operation(mpe);
	desc = (op && op-> description()? op-> description(): "");

	if (!op) {
	    fprintf (stdout, "WARN: workingstep #%d has no operation\n",
		     mpe-> entity_id());
	    return EXEC_BAD;  /* ERROR */
	}
	// milling_type_operation
	if (op-> isa(ROSE_DOMAIN(bottom_and_side_milling_operation))) {
	    if (!strcmp(desc,"rough milling"))  return EXEC_MILL_BOTSIDE_ROUGH;
	    if (!strcmp(desc,"finish milling")) return EXEC_MILL_BOTSIDE;
	    return EXEC_BAD;  /* ERROR */
	}

	if (op-> isa(ROSE_DOMAIN(plane_milling_operation))) {
	    if (!strcmp(desc,"rough milling"))  return EXEC_MILL_PLANE_ROUGH;
	    if (!strcmp(desc,"finish milling")) return EXEC_MILL_PLANE;
	    return EXEC_BAD;  /* ERROR */
	}

	if (op-> isa(ROSE_DOMAIN(side_milling_operation))) {
	    if (!strcmp(desc,"rough milling"))  return EXEC_MILL_SIDE_ROUGH;
	    if (!strcmp(desc,"finish milling")) return EXEC_MILL_SIDE;
	    return EXEC_BAD;  /* ERROR */
	}

	if (op-> isa(ROSE_DOMAIN(freeform_milling_operation))) 
	    return EXEC_MILL_FREEFORM; 

	// drilling_type_operation
	if (op-> isa(ROSE_DOMAIN(drilling_operation))) {
	    if (!strcmp(desc,"drilling")) 	 	return EXEC_DRILL;
	    if (!strcmp(desc,"center drilling"))  	return EXEC_DRILL_CENTER;
	    if (!strcmp(desc,"counter sinking")) 	return EXEC_DRILL_CSINK;
	    if (!strcmp(desc,"multistep drilling")) 	return EXEC_DRILL_MULTISTEP;
	    return EXEC_BAD;  /* ERROR */
	}

	if (op-> isa(ROSE_DOMAIN(boring_operation)))  {
	    if (!strcmp(desc,"boring"))  	return EXEC_BORE;
	    if (!strcmp(desc,"reaming"))  	return EXEC_REAM;
	    return EXEC_BAD;  /* ERROR */
	}

	if (op-> isa(ROSE_DOMAIN(back_boring_operation)))
	    return EXEC_BACK_BORE;

	if (op-> isa(ROSE_DOMAIN(tapping_operation))) {
	    if (!strcmp(desc,"tapping")) 	  return EXEC_TAP;
	    if (!strcmp(desc,"thread drilling"))  return EXEC_THREAD_DRILL;
	    return EXEC_BAD;  /* ERROR */
	}

	/* technically, these can be standalone, but it makes more
	 * sense to have at least "workpiece probing" as the operation
	 * of a machining workingstep so you can point to a feature to
	 * be probed.
	 */
	if (op-> isa(ROSE_DOMAIN(machining_touch_probing)))  {
	    if (!strcmp(desc,"tool length probing"))  	return EXEC_PROBE_TLEN;
	    if (!strcmp(desc,"tool radius probing"))  	return EXEC_PROBE_TRAD;
	    if (!strcmp(desc,"workpiece probing"))  	return EXEC_PROBE;
	    if (!strcmp(desc,"workpiece complete probing")) 
		return EXEC_PROBE_COMPLETE;
	    return EXEC_BAD;  /* ERROR */
	}
    }

    /* ================================================== */
    /* Workplan */
    if (mpe-> isa(ROSE_DOMAIN(machining_workplan))) 
	return EXEC_WORKPLAN;

    /* ================================================== */
    /* Control Flow Stuff -- Exactly an MPE
     */
    desc = (mpe-> description()? mpe-> description(): "");
    if (mpe->domain() == ROSE_DOMAIN(machining_process_executable)) {
	if (!strcmp(desc,"assignment")) 	return EXEC_ASSIGN;
	if (!strcmp(desc,"if statement"))  	return EXEC_IF;
	if (!strcmp(desc,"non-sequential"))  	return EXEC_NONSEQ;
	if (!strcmp(desc,"parallel")) 		return EXEC_PARALLEL;
	if (!strcmp(desc,"selective"))  	return EXEC_SELECT;
	if (!strcmp(desc,"while statement"))  	return EXEC_WHILE;
	return EXEC_BAD;  /* ERROR */
    }

    if (mpe-> isa(ROSE_DOMAIN(machining_nc_function)))  {
	if (!strcmp(desc,"display message"))  	return EXEC_MESSAGE;
	if (!strcmp(desc,"optional stop"))  	return EXEC_STOP_OPTIONAL;
	if (!strcmp(desc,"program stop"))  	return EXEC_STOP;
	if (!strcmp(desc,"set mark"))  		return EXEC_MARK_SET;
	if (!strcmp(desc,"wait for mark"))  	return EXEC_MARK_WAIT;
	if (!strcmp(desc,"exchange pallet"))  	return EXEC_PALLET_EXCHANGE;
	if (!strcmp(desc,"index pallet"))  	return EXEC_PALLET_INDEX;
	if (!strcmp(desc,"load tool"))  	return EXEC_TOOL_LOAD;
	if (!strcmp(desc,"unload tool"))  	return EXEC_TOOL_UNLOAD;
	return EXEC_BAD;  /* ERROR */
    }

    if (mpe-> isa(ROSE_DOMAIN(machining_rapid_movement)))  {
	return EXEC_RAPID;
    }

    if (mpe-> isa(ROSE_DOMAIN(machining_touch_probing)))  {
	if (!strcmp(desc,"tool length probing"))  	return EXEC_PROBE_TLEN;
	if (!strcmp(desc,"tool radius probing"))  	return EXEC_PROBE_TRAD;
	if (!strcmp(desc,"workpiece probing"))  	return EXEC_PROBE;
	if (!strcmp(desc,"workpiece complete probing"))	return EXEC_PROBE_COMPLETE;
	return EXEC_BAD;  /* ERROR */
    }

    /* miss on all */
    return FEAT_NONE;
}



machining_process_executable * get_exec_substep (
    machining_process_executable * mpe, unsigned i
    )
{
    /* this may or may not be used, but it is nice to know how to do it */
    if (!mpe || !mpe-> isa (ROSE_DOMAIN(machining_workplan)))
	return 0;

    machining_workplan * wp = ROSE_CAST(machining_workplan,mpe);

    sequential_method * sm = (i<wp->m_steps.size())? 
	((sequential_method *) wp->m_steps[i]) : 0;

    action_method * ed = (sm? sm->related_method(): 0);

    return (ed && ed-> isa(ROSE_DOMAIN(machining_process_executable)))?
	(ROSE_CAST(machining_process_executable,ed)): 0;
}

machining_process_executable * get_exec_parent (
    machining_process_executable * mpe)
{
    return (mpe? mpe-> m_parent: 0);
}




/* Calculate a depth-first flattened list of all nested workingsteps.
 * This is here only to calculate a complete list of tools present and
 * missing for all of the worksteps in a workplan.   This caches the 
 * list for each nested workplan for efficiency.
 * 
 * Returns nonzero on error, such as a cycle in the workplan.
 */

int calc_exec_all_worksteps (
    machining_workplan * wp, 	/* wp to check */
    rose_stack * nested_wps	/* use to detect cycles */
    )
{
    unsigned i,sz;
    unsigned j,szz;

    /* not present or already done */
    if (!wp) return 0;

    /* already been computed */
    if (wp-> m_calc_all_steps) {
	/* reached from within a traversal, check for loops */
	if (nested_wps) for (i=0, sz=nested_wps->size(); i<sz; i++) {
	    if ((void*)wp == nested_wps->get(i)) {
		fprintf (stdout, "WARN: Workplan #%d refers to itself\n",
			 wp-> entity_id());
		return 1;
	    }
	}
	return 0;
    }

    /* first time in */
    if (!nested_wps) {
	nested_wps = new rose_stack;
    }
    nested_wps-> push(wp);
    wp-> m_calc_all_steps = new rose_vector;

    for (i=0, sz=wp->m_steps.size(); i<sz; i++) {
	sequential_method * sm = (sequential_method *) 
	    wp->m_steps[i];

	action_method * ed = sm->related_method();
	
	if (ed-> isa(ROSE_DOMAIN(machining_workingstep)))
	    wp-> m_calc_all_steps-> append 
		(ROSE_CAST(machining_workingstep,ed));
	
	if (ed-> isa(ROSE_DOMAIN(machining_workplan))) {
	    machining_workplan * nest = ROSE_CAST(machining_workplan,ed);

	    /* test for cyclic errors */
	    if (calc_exec_all_worksteps (nest, nested_wps))
		continue;

	    /* append the computed list */
	    rose_vector * wp_all = wp-> m_calc_all_steps;
	    rose_vector * nest_all = nest-> m_calc_all_steps;
	    for (j=0, szz=nest_all->size(); j<szz; j++)
		wp_all-> append (nest_all->get(j));
	}	
    }

    nested_wps-> pop();
    return 0;
}


int calc_exec_unique_features (
    machining_workplan * wp
    )
{
    /* not present or already done */
    if (!wp || wp-> m_calc_unique_feats) return 0;

    unsigned i,sz;
    rose_vector * all_steps = wp->m_calc_all_steps;
    rose_vector * all_feats = new rose_vector;

    if (!all_steps) {
	/* build DFS list of all steps first */ 
	calc_exec_all_worksteps (wp);
	all_steps = wp->m_calc_all_steps;
    }

    wp-> m_calc_unique_feats = all_feats;

    /* avoid doing traversal unless needed */
    if (!all_steps->size()) return 0;

    ROSE.beginTraversal();
    for (i=0, sz=all_steps->size(); i<sz; i++) {
	machining_workingstep * ws = 
	    (machining_workingstep *) all_steps->get(i);

	shape_aspect * sa = get_workingstep_feature(ws);

	/* this should also filter out bosses, but I don't think
	 *  bosses will show up here.
	 */
	if (!sa || sa-> wasVisited()) continue;

	sa-> visit();
	all_feats-> append (sa);
    }
    ROSE.endTraversal();
    return 0;
}

int calc_exec_unique_tools (
    machining_workplan * wp
    )
{
    /* not present or already done */
    if (!wp || wp-> m_calc_unique_tools) return 0;

    unsigned i,sz;
    rose_vector * all_steps = wp->m_calc_all_steps;
    rose_vector * all_tools = new rose_vector;

    if (!all_steps) {
	/* build DFS list of all steps first */ 
	calc_exec_all_worksteps (wp);
	all_steps = wp->m_calc_all_steps;
    }

    wp-> m_calc_unique_tools = all_tools;

    /* avoid doing traversal unless needed */
    if (!all_steps->size()) return 0;

    ROSE.beginTraversal();
    for (i=0, sz=all_steps->size(); i<sz; i++) {
	machining_workingstep * ws = 
	    (machining_workingstep *) all_steps->get(i);

	machining_tool * mt = (ws-> m_op? ws-> m_op-> m_tool: 0);

	if (!mt || mt-> wasVisited()) continue;

	mt-> visit();
	all_tools-> append (mt);
    }
    ROSE.endTraversal();
    return 0;
}




shape_aspect * get_workingstep_feature (machining_workingstep * ws)
{
    if (!ws) return 0;
    machining_feature_process * mfp =
	(ws-> m_inproc? 
	 ws-> m_inproc: 
	 ws-> m_final);

    return (mfp? mfp-> m_feature: 0);
}

shape_aspect * get_workingstep_inproc_feature (machining_workingstep * ws)
{
    machining_feature_process * mfp = (ws? ws-> m_inproc: 0);
    return (mfp? mfp-> m_feature: 0);
}

shape_aspect * get_workingstep_final_feature (machining_workingstep * ws)
{
    machining_feature_process * mfp = (ws? ws-> m_final: 0);
    return (mfp? mfp-> m_feature: 0);
}




machining_operation * get_workingstep_operation (
    machining_process_executable * mpe
    )
{
    if (mpe && mpe-> isa(ROSE_DOMAIN(machining_workingstep))) {
	machining_workingstep * ws = ROSE_CAST (machining_workingstep,mpe);
	return ws-> m_op;
    }
    return 0;
}



/* ----------------------------------------
 * Operations
 */

machining_tool * get_operation_tool (
    machining_operation * op
    )
{
    return (op? op-> m_tool: 0);
}

machining_technology * get_operation_technology (
    machining_operation * op
    )
{
    return (op? op-> m_tech: 0);
}

machining_functions * get_operation_functions (
    machining_operation * op
    )
{
    return (op? op-> m_funs: 0);
}

machining_strategy * get_operation_strategy (
    machining_operation * op
    )
{
    return (op? op-> m_strategy: 0);
}


double get_operation_overcut_length (
    machining_operation * op, StixUnit u
    )
{
    /* valid on all milling and drilling steps.  Thats all
     * we have right now
     */
    return get_length_value (
	get_action_property_rep (op, "overcut length"), 0, u
	);
}


machining_approach_retract_strategy * 
get_milling_operation_approach_strategy (
    machining_operation * op
    )
{
    /* could check IS_MILL, but we would catch it earlier */
    return (op? op-> m_approach: 0);
}

machining_approach_retract_strategy * 
get_milling_operation_retract_strategy (
    machining_operation * op
    )
{
    /* could check IS_MILL, but we would catch it earlier */
    return (op? op-> m_retract: 0);
}




double get_milling_operation_axial_cutting_depth (
    machining_operation * op, StixUnit u
    )
{
    /* Valid on PLANE, SIDE, bottom and side */

    return get_length_value (
	get_action_property_rep (op, "axial cutting depth"), 0, u
	);
}

double get_milling_operation_radial_cutting_depth (
    machining_operation * op, StixUnit u
    )
{
    /* Valid on SIDE, bottom and side */
    return get_length_value (
	get_action_property_rep (op, "radial cutting depth"), 0, u
	);
}

double get_milling_operation_allowance_side (
    machining_operation * op, StixUnit u
    )
{
    /* Valid on SIDE, bottom and side */

    return get_length_value (
	get_action_property_rep (op, "allowance bottom"), 0, u
	);
}

double get_milling_operation_allowance_bottom (
    machining_operation * op, StixUnit u
    )
{
    /* Valid on EXEC_MILL_PLANE, bottom and side*/

    return get_length_value (
	get_action_property_rep (op, "allowance bottom"), 0, u
	);
}





double get_drilling_operation_cutting_depth (
    machining_operation * op, StixUnit u
    )
{
    /* check needed on put, gratuitous on get */
//    if (!op || !IS_DRILL(op)) return 0;

    return get_length_value (
	get_action_property_rep (op, "cutting depth"), 0, u
	);
}

double get_drilling_operation_previous_diameter (
    machining_operation * op, StixUnit u
    )
{
    /* check needed on put, gratuitous on get */
//    if (!op || !IS_DRILL(op)) return 0;

    return get_length_value (
	get_action_property_rep (op, "previous diameter"), 0, u
	);
}

double get_drilling_operation_dwell_time_bottom (
    machining_operation * op, StixUnit u
    )
{
    /* check needed on put, gratuitous on get */
//    if (!op || !IS_DRILL(op)) return 0;

    return get_time_value (
	get_action_property_rep (op, "dwell time bottom"), 0, u
	);
}

double get_drilling_operation_feed_on_retract (
    machining_operation * op
    )
{
    /* check needed on put, gratuitous on get */
//    if (!op || !IS_DRILL(op)) return 0;

    return get_ratio_value (
	get_action_property_rep (op, "feed on retract")
	);
}














/* ----------------------------------------
 * Milling Technology Functions
 */
double get_technology_cutspeed (
    machining_technology * mt, StixUnit u
    )
{
    return get_speed_value (
	get_action_property_rep (mt, "cutspeed"),0,u
	);
}

double get_technology_spindle (
    machining_technology * mt, StixUnit u
    )
{
    return get_rotspeed_value (
	get_action_property_rep (mt, "spindle"),0,u
	);
}

double get_technology_feedrate_per_tooth (
    machining_technology * mt, StixUnit u
    )
{
    return get_length_value (
	get_action_property_rep (mt, "feedrate per tooth"),0,u
	);
}


int get_technology_synchronize_spindle_with_feed (
    machining_technology * mt
    )
{
    return get_boolean_value (
	"synchronized", 
	"not synchronized",
	get_action_property_rep (mt, "synchronize spindle with feed")
	);
}

int get_technology_inhibit_feedrate_override (
    machining_technology * mt
    )
{
    /* IMPORTANT -- we are checking for the NOT case here
     * because the attribute was defined as a negative.
     */
    return get_boolean_value (
	"override not allowed", 
	"override allowed", 
	get_action_property_rep (mt, "inhibit feedrate override")
	);
}

int get_technology_inhibit_spindle_override (
    machining_technology * mt
    )
{
    /* IMPORTANT -- we are checking for the NOT case here
     * because the attribute was defined as a negative.
     */
    return get_boolean_value (
	"override not allowed", 
	"override allowed",
	get_action_property_rep (mt, "inhibit spindle override")
	);
}



/* ----------------------------------------
 * Milling Machine Functions
 */
int get_machine_functions_coolant (
    machining_functions * mf
    )
{
#ifdef BUG_BOOLEAN_HAS_WRONG_VALUES
    descriptive_representation_item * it = get_descriptive_rep_item (
	get_action_property_rep (mf, "coolant")
	);

    char * desc = (it? it->description(): 0);

    if (!desc || 
	!strcmp(desc, "coolant off") ||   // proper symbol
	!strcmp(desc, "off"))		  // bogus symbol
	return 0;

    if (!strcmp(desc, "coolant on") ||    // proper symbol
	!strcmp(desc, "on"))		  // bogus symbol
	return 1;

    return 0;     /* error case, null or unknown string */
#else
    return get_boolean_value (
	"coolant on", 
	"coolant off", 
	get_action_property_rep (mf, "coolant")
	);
#endif
}



int get_machine_functions_mist (
    machining_functions * mf
    )
{
    return get_boolean_value (
	"mist on", 
	"mist off", 
	get_action_property_rep (mf, "mist")
	);
}

int get_machine_functions_through_spindle_coolant (
    machining_functions * mf
    )
{
    return get_boolean_value (
	"through spindle coolant on", 
	"through spindle coolant off", 
	get_action_property_rep (mf, "through spindle coolant")
	);
}

int get_machine_functions_chip_removal (
    machining_functions * mf
    )
{
    return get_boolean_value (
	"chip removal on", 
	"chip removal off",
	get_action_property_rep (mf, "chip removal")
	);
}


direction * get_machine_functions_oriented_spindle_stop (
    machining_functions * mf
    )
{
    return get_direction (
	get_action_property_rep (mf, "oriented spindle stop")
	);
}

double get_machine_functions_coolant_pressure (
    machining_functions * mf, StixUnit u
    )
{
    return get_pressure_value (
	get_action_property_rep(mf, "coolant pressure"),0,u
	);
}

double get_machine_functions_through_pressure (
    machining_functions * mf, StixUnit u
    )
{
    return get_pressure_value (
	get_action_property_rep(mf, "through spindle pressure"),0,u
	);
}


