/* $RCSfile: xform.cxx,v $
 * $Revision: 1.4 $ $Date: 2003/05/27 18:30:36 $
 * Auth: Ford Oxaal (ford@steptools.com)
 * Auth: Dave Loffredo (loffredo@steptools.com)
 * 
 * Copyright (c) 1991-2003 by STEP Tools Inc. 
 * All Rights Reserved.
 * 
 * This file is part of the STIX Library
 * 
 * This file may be distributed and/or modified under the terms of 
 * the GNU General Public License version 2 as published by the Free
 * Software Foundation and appearing in the file LICENSE.GPL included
 * with this file.
 * 
 * THIS FILE IS PROVIDED "AS IS" WITH NO WARRANTY OF ANY KIND,
 * INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * STEP Tools Inc. also offers licenses for commercial use of this
 * software in non-GPL applications.  Contact info@steptools.com for
 * terms and pricing.
 * 
 * 		----------------------------------------
 */

/* Code originally taken from the expander package and updated
 * for use exclusively in a STEP context.   It appears that Ford
 * or Gene was using a 3x3 matrix for the rotational transform to
 * which they added another row for the offset transform.
 */


#include <rose.h>
#include <math.h>
#include <integrated_cnc_schema.h>
#include "stix_imp.h"
#include "xform.h"

#define PI 3.14159265358979323846264338327950288419716939937511
#define PI2 (2.0 * 3.14159265358979323846264338327950288419716939937511)
#define pyth3d_old(x,y,z) sqrt((((x)*(x))+((y)*(y))+((z)*(z))))

/* step translation functions */

/* These need a bit of work to clean up.  I don't know why they
 * were written as friends, plus they assume that an object is 
 * brand new and will break if used to update an existing object.
 */
void StixMtrx_from_cto (RoseObject * cto, StixMtrx & M)
{
    if (!cto) {
	fprintf(stderr, "StixMtrxCto3d: ERROR: NULL arg\n");
	return;
    }
    if (cto->isa("cartesian_transformation_operator_2d")) {
	for (unsigned i = 0; i < 2; i++)
	{
	    /* build x row */
	    M.m[i][0] = cto->getObject("axis1")->getObject("direction_ratios")->getDouble(i);
	    /* build y row */
	    M.m[i][1] = cto->getObject("axis2")->getObject("direction_ratios")->getDouble(i);
	    /* build translation row */
	    M.m[i][3] = cto->getObject("local_origin")->getObject("coordinates")->getDouble(i);
	}
	M.m[2][0] = 0; 	/* dummy element of x row */
	M.m[2][1] = 0;	/* dummy element of y row */
	M.m[2][3] = 0; 	/* dummy element of translation row */
	/* build z row */
	M.m[0][2] = 0;
	M.m[1][2] = 0;
	M.m[2][2] = 1;
    }
    else if (cto->isa("cartesian_transformation_operator_3d"))
    {
	for (unsigned i = 0; i < 3; i++)
	{
	    /* build x row */
	    M.m[i][0] = cto->getObject("axis1")->getObject("direction_ratios")->getDouble(i);
	    /* build y row */
	    M.m[i][1] = cto->getObject("axis2")->getObject("direction_ratios")->getDouble(i);
	    /* build z row */
	    M.m[i][2] = cto->getObject("axis3")->getObject("direction_ratios")->getDouble(i);
	    /* build translation row */
	    M.m[i][3] = cto->getObject("local_origin")->getObject("coordinates")->getDouble(i);
	}
    }
    else
	fprintf(stderr, "StixMtrxCto3d: ERROR: cannot handle %s\n",
		 cto->domain()->name());
}

void StixMtrx_to_cto (StixMtrx * M, RoseObject & cto)
{
    for (unsigned i = 0; i < 3; i++)
    {  
	/* build x row */
	cto.getObject("axis1")->getObject("direction_ratios")->putDouble(M->m[i][0], i);
	/* build y row */
	cto.getObject("axis2")->getObject("direction_ratios")->putDouble(M->m[i][1], i);
	/* build z row */
	cto.getObject("axis3")->getObject("direction_ratios")->putDouble(M->m[i][2], i);
	/* build translation row */
	cto.getObject("local_origin")->getObject("coordinates")->putDouble(M->m[i][3], i);
    }
}

/* input axis2_placement_3d and output StixMtrx with x, y and z and
 * location filled in 
 */
void StixMtrx_from_ap (RoseObject * ax, StixMtrx & M)
{
    if (ax == NULL)
    {
	fprintf(stderr, "StixMtrx: passed in NULL axis2_placement_3d, exiting\n");
	exit(2);
    }

    StixVctr x(1,0,0), y, z(0,0,1), w(0,0,0);

    for (unsigned i = 0; i < 3; i++)
    {
	if (ax->isa("axis2_placement_3d")) {
	    if (ax->getObject("ref_direction")) {
		x.put(ax->getObject("ref_direction")->getObject("direction_ratios")->getDouble(i), i);
	    }
	    if (ax->getObject("axis"))
		z.put(ax->getObject("axis")->getObject("direction_ratios")->getDouble(i), i);
	    /* w is the translation row */
	    if (ax->getObject("location")) {
		if (ax->getObject("location")->getObject("coordinates")) {
		    if (ax->getObject("location")->getObject("coordinates")->getDouble(i)) {
			w.put(ax->getObject("location")->getObject("coordinates")->getDouble(i), i);
		    }
		}
	    }
	}
	if (ax->isa("axis1_placement")) {
	    if (ax->getObject("axis"))
		z.put(ax->getObject("axis")->getObject("direction_ratios")->getDouble(i), i);
	    /* w is the translation row */
	    if (ax->getObject("location")) {
		if (ax->getObject("location")->getObject("coordinates")) {
		    if (ax->getObject("location")->getObject("coordinates")->getDouble(i)) {
			w.put(ax->getObject("location")->getObject("coordinates")->getDouble(i), i);
		    }
		}
	    }
	    x = z.rightAngleTo();
	}
    }
    x.normalizeInPlace();
    z.normalizeInPlace();
    y = z * x;
    /* x, y, and z are the rotation rows, w is the translation row */
    M.populate(x, y, z, w);
}  

/* pass in pointer to aP3d that has been pnewed in some design */
void StixMtrx_to_ap (StixMtrx & M, RoseObject * aP3d)
{
    RoseDesign * rD = aP3d->design();
    RoseObject * ax = rD->pnewInstance("direction"); ax->putObject(rD->pnewInstance("ListOfDouble"), "direction_ratios");
    RoseObject * ref = rD->pnewInstance("direction"); ref->putObject(rD->pnewInstance("ListOfDouble"), "direction_ratios");
    RoseObject * loc = rD->pnewInstance("cartesian_point"); loc->putObject(rD->pnewInstance("ListOfDouble"), "coordinates");
    for (unsigned i = 0; i < 3; i++)
    {
	ref->getObject("direction_ratios")->putDouble(M.m[i][0], i);
	ax->getObject("direction_ratios")->putDouble(M.m[i][2], i);
	loc->getObject("coordinates")->putDouble(M.m[i][3], i);
    }
    aP3d->putObject(ax, "axis");
    aP3d->putObject(ref, "ref_direction");
    aP3d->putObject(loc, "location");
}

void StixVctr_from_vec (RoseObject * V, StixVctr & v)
{
    if (!V->isa("vector")){
	fprintf(stderr, "StixVctrV: ERROR: arg not a STEP vector\n");
	return;
    }
    RoseObject * or = V->getObject("orientation");
    RoseObject * dir = or->getObject("direction_ratios");
    for (unsigned i = 0; i < 3; i++)
    {
	v.put(dir->getDouble(i), i);
    }
}

void StixVctr_from_dir (RoseObject * D, StixVctr & v)
{
    if (!D->isa("direction")){
	fprintf(stderr, "StixVctrD: ERROR: arg not a STEP direction\n");
	return;
    }
    ListOfDouble * lod = 
	ROSE_CAST(ListOfDouble, D->getObject("direction_ratios"));
    for (unsigned i = 0; i < 3; i++) {
	v.put(lod->getDouble(i), i);
    }
}

/* pass in pointer to direction that has been pnewed in some design */
void StixVctr_to_dir (StixVctr & v, RoseObject * D)
{
    if (!D->isa("direction")){
	fprintf(stderr, "StixVctrD: ERROR: arg not a STEP direction\n");
	return;
    }
    for (unsigned i = 0; i < 3; i++)
    {
	((ListOfDouble *) D)->add(v.get(i));
    }
}


/* ==================================================
 * Matrix Class
 */

/* Constructors */
StixMtrx::StixMtrx()
{
    for (int i = 0; i < 4; i++)
	for (int j = 0; j < 4; j++)
	    if (i == j) m[i][j] = 1;
	    else m[i][j] = 0;
}

StixMtrx::StixMtrx(
    double a, double b, double c, double d,
    double e, double f, double g, double h,
    double i, double j, double k, double l,
    double mm, double n, double o, double p)
{
    m[0][0] = a;  m[1][0] = b;  m[2][0] = c;  m[3][0] = d;
    m[0][1] = e;  m[1][1] = f;  m[2][1] = g;  m[3][1] = h;
    m[0][2] = i;  m[1][2] = j;  m[2][2] = k;  m[3][2] = l;
    m[0][3] = mm; m[1][3] = n;  m[2][3] = o;  m[3][3] = p;
}

/* Operators */
StixVctr StixMtrx::operator * (StixVctr & v1)
{
    StixVctr ret_v;
    double val = 0;
    for (int i = 0; i < 4; i++)
    {
	for (int j = 0; j < 4; j++)
	{

	    // CHANGE THIS LATER
	    /* Get column of matrix, and dot with "row" of vector.  This
	       implements case 3. given in StixMtrx.h. */
	    //val = val + get(i, j) * v1.get(j);
	    val = val + get(j, i) * v1.get(j);
	}
	ret_v.put(val, i);
	val = 0;
    }
    return ret_v;
}

StixMtrx StixMtrx::operator * (StixMtrx & m1)
{
    StixMtrx ret_m;
    double val = 0;
    for (int i = 0; i < 4; i++)
    {
	for (int j = 0; j < 4; j++)
	{
	    for (int k = 0; k < 4; k++)
	    {
		/* get row of first, and dot with column of second */
		val = val + get(k, i) * m1.get(j, k);
	    }
	    ret_m.put(val, j, i);
	    val = 0;
	}
    }
    return ret_m;
}

StixMtrx StixMtrx::operator * (double s)
{
    StixMtrx ret_m;
    for (int j = 0; j < 4; j++)
    {
	for (int i = 0; i < 4; i++)
	{
	    m[i][j] = m[i][j] * s;
	}
    }
    return ret_m;
}

/* Members */
double StixMtrx::get(int i, int j)
{
    if (i < 0 || i > 3 || j < 0 || j > 3)
    {
	fprintf(stderr, "StixMtrx: ERROR: indices out of bounds: %d %d.\n", i, j);
	return 0;
    }
    return m[i][j];
}

StixVctr StixMtrx::x()
{
    StixVctr ret;
    for (int i = 0; i < 3; i++)
	ret.put(get(i, 0), i);
    return ret;
}
StixVctr StixMtrx::y()
{
    StixVctr ret;
    for (int i = 0; i < 3; i++)
	ret.put(get(i, 1), i);
    return ret;
}
StixVctr StixMtrx::z()
{
    StixVctr ret;
    for (int i = 0; i < 3; i++)
	ret.put(get(i, 2), i);
    return ret;
}
StixVctr StixMtrx::w()
{
    StixVctr ret;
    for (int i = 0; i < 3; i++)
	ret.put(get(i, 3), i);
    return ret;
}

void StixMtrx::x(StixVctr & v)
{
    for (int i = 0; i < 3; i++)
	put(v.get(i), i, 0);
}
void StixMtrx::y(StixVctr & v)
{
    for (int i = 0; i < 3; i++)
	put(v.get(i), i, 1);
}
void StixMtrx::z(StixVctr & v)
{
    for (int i = 0; i < 3; i++)
	put(v.get(i), i, 2);
}
void StixMtrx::w(StixVctr & v)
{
    for (int i = 0; i < 3; i++)
	put(v.get(i), i, 3);
}

void StixMtrx::put(double value, int i, int j)
{
    if (i < 0 || i > 3 || j < 0 || j > 3)
    {
	fprintf(stderr, "StixMtrx: ERROR: indices out of bounds: %d %d.\n", i, j);
	return;
    }
    m[i][j] = value;
}

/* useful for calculating inverse of orthogonal 3 x 3 portion */
StixMtrx StixMtrx::t3(void)
{
    StixMtrx ret_m;
    for (int i = 0; i < 3; i++)
    {
	for (int j = 0; j < 3; j++)
	{
	    ret_m.put(get(i, j), j, i);
	}
    }
    return ret_m;  
}

/* useful for screwing around */
StixMtrx StixMtrx::t4(void)
{
    StixMtrx ret_m;
    for (int i = 0; i < 4; i++)
    {
	for (int j = 0; j < 4; j++)
	{
	    ret_m.put(get(i, j), j, i);
	}
    }
    return ret_m;  
}

void StixMtrx::normalizeInPlace()
{
    double denomX = sqrt(m[0][0] * m[0][0] + 
			 m[0][1] * m[0][1] + 
			 m[0][2] * m[0][2]);
    double denomY = sqrt(m[1][0] * m[1][0] + 
			 m[1][1] * m[1][1] + 
			 m[1][2] * m[1][2]);
    double denomZ = sqrt(m[2][0] * m[2][0] + 
			 m[2][1] * m[2][1] + 
			 m[2][2] * m[2][2]);
    int j;
    for (j = 0; j < 3; j++)
	m[0][j] = m[0][j] / denomX;
    for (j = 0; j < 3; j++)
	m[1][j] = m[1][j] / denomY;
    for (j = 0; j < 3; j++)
	m[2][j] = m[2][j] / denomZ;
}

void StixMtrx::leftHanded()
{
    //StixMtrx ret_m;
    //ret_m = t4().t4(); // copy
    //ret_m[3][2] = -ret_m[3][2];
    m[3][2] = -m[3][2];
}

/* for the 2d portion of matrices only */
double StixMtrx::det2d()
{
    double ret = 0;
    ret = get(0,0) * get(1,1) - get(0,1) * get(1,0);
    return ret;
}

/* for matrices with orthonormal bases only */
StixMtrx StixMtrx::inverse()
{
    StixMtrx ret_m;
    double val = 0;
    /* Inverse of rotation part is just the transpose */
    ret_m = t3();
    /* Inverse of the translation part zeros out translation vector */
    /* scan column i */
    for (int i = 0; i < 3; i++)
    {
	for (int k = 0; k < 3; k++)
	{
	    val = val + get(k, 3) * ret_m.get(i, k);
	}
	ret_m.put(-val, i, 3);
	val = 0;
    }
    return ret_m;
}

StixMtrx StixMtrx::genTransformTo(StixMtrx & m1)
{
    StixMtrx ret_m;
    /* transform to origin, then to m1 */
    ret_m = inverse() * m1;
    return ret_m;
}

void StixMtrx::genRotation(StixVctr & v, double t)
{
    double u1, u2, u3;
    u1 = v.get(0); u2 = v.get(1); u3 = v.get(2);
    put(u1 * u1 + cos(t) * (1.0 - (u1 * u1)), 0, 0);
    put(u1 * u2 * (1.0 - cos(t)) - u3 * sin(t), 1, 0);
    put(u3 * u1 * (1.0 - cos(t)) + u2 * sin(t), 2, 0);
    put(u1 * u2 * (1.0 - cos(t)) + u3 * sin(t), 0, 1);
    put(u2 * u2 + cos(t) * (1.0 - (u2 * u2)), 1, 1);
    put(u2 * u3 * (1.0 - cos(t)) - u1 * sin(t), 2, 1);
    put(u3 * u1 * (1.0 - cos(t)) - u2 * sin(t), 0, 2);
    put(u2 * u3 * (1.0 - cos(t)) + u1 * sin(t), 1, 2);
    put(u3 * u3 + cos(t) * (1.0 - (u3 * u3)), 2, 2);
}

void StixMtrx::putTranslation(double a, double b, double c)
{
    m[3][0] = a; m[3][1] = b; m[3][2] = c;
}

void StixMtrx::copy3(StixMtrx & M)
{
    for (int i = 0; i < 3; i++)
	for (int j = 0; j < 3; j++)
	    put(M.get(i, j), i, j);
}

int StixMtrx::isIdentity3()
{
    for (int i = 0; i < 3; i++)
	for (int j = 0; j < 3; j++)
	    if (j == i) {
		if (m[i][j] != 1) return 0;
	    }
	    else {
		if (m[i][j] != 0) return 0;
	    }
     
    return 1;
}

int StixMtrx::isIdentity4()
{
    for (int i = 0; i < 4; i++)
	for (int j = 0; j < 4; j++)
	    if (j == i) {
		if (m[i][j] != 1) return 0;
	    }
	    else {
		if (m[i][j] != 0) return 0;
	    }
     
    return 1;
}


int StixMtrx::populate(StixVctr & x, StixVctr & y, StixVctr & z)
{
    for (int i = 0; i < 4; i++)
    {
	m[i][0] = x.get(i);
	m[i][1] = y.get(i);
	m[i][2] = z.get(i);
    }
    return 0;
}

/* x, y, z and w are row vectors */
int StixMtrx::populate(StixVctr & x, StixVctr & y, StixVctr & z, StixVctr & w)
{
    for (int i = 0; i < 3; i++)
    {
	m[i][0] = x.get(i);
	m[i][1] = y.get(i);
	m[i][2] = z.get(i);
	m[i][3] = w.get(i);
    }
    return 0;
}

void StixMtrx::print(FILE * out, char * indent)
{
    if (!indent) indent = "";
    fprintf(out,
	    "%s%f %f %f %f\n",
	    indent, m[0][0], m[1][0], m[2][0], m[3][0]);
    fprintf(out, 
	    "%s%f %f %f %f\n"
	    "%s%f %f %f %f\n"
	    "%s%f %f %f %f\n"
	    "%s%s\n",
	    indent, m[0][1], m[1][1], m[2][1], m[3][1],
	    indent, m[0][2], m[1][2], m[2][2], m[3][2],
	    indent, m[0][3], m[1][3], m[2][3], m[3][3],
	    indent, "-------------------------------"
	);
}

void StixMtrx::print_axes(FILE * out, char * indent)
{
    /* print as a cartesian placement and direction */ 
    if (!indent) indent = "";
    fprintf (out, 
	     "%sx-axis: %.15g %.15g %.15g\n"
	     "%sy-axis: %.15g %.15g %.15g\n"
	     "%sz-axis: %.15g %.15g %.15g\n"
	     "%sorigin: %.15g %.15g %.15g\n",
	     indent, m[0][0], m[1][0], m[2][0],
	     indent, m[0][1], m[1][1], m[2][1],
	     indent, m[0][2], m[1][2], m[2][2],
	     indent, m[0][3], m[1][3], m[2][3]
	     );
}



  

/* ==================================================
 * Vector Class
 */

/* Constructors */
StixVctr::StixVctr(void)
{
    v[0] = 0.0; v[1] = 0.0; v[2] = 0.0; v[3] = 0.0;
}

StixVctr::StixVctr(double x, double y, double z, double w)
{
    v[0] = x; v[1] = y; v[2] = z; v[3] = w;  
}

/* Operators */
StixVctr StixVctr::operator * (StixVctr & w) // cross
{
    StixVctr V;
    V.put(v[1] * w.get(2) - v[2] * w.get(1), 0);
    V.put(v[2] * w.get(0) - v[0] * w.get(2), 1);
    V.put(v[0] * w.get(1) - v[1] * w.get(0), 2);
    return V;
}

StixVctr StixVctr::operator - () // vector addition
{
    StixVctr V;
    for (int i = 0; i < 3; i++)
	V.put(-v[i], i);
    return V;
}

StixVctr StixVctr::operator + (StixVctr & w) // vector addition
{
    StixVctr V;
    for (int i = 0; i < 3; i++)
	V.put(v[i] + w.get(i), i);
    return V;
}

/* This operation implies a row vector V, because it is the first
   operand.  This then implies a row matrix as the second operand,
   which gets transposed to a column matrix.  The operation here
   implicitly does the transposition on said row matrix by the call
   m.get(i,j).  i is the row, and j is the column (See StixMtrx.h).
   Because j is in the inner loop, we are traversing a row matrix as
   if it were a column matrix, thus achieving the necessary
   transposition. */
StixVctr StixVctr::operator * (StixMtrx & m)
{
    int verbose = 0;
    StixVctr ret_v;
    double val = 0;
    if (verbose) printf("StixVctr.c:\n");
    for (int i = 0; i < 4; i++)
    {
	for (int j = 0; j < 4; j++)
	{
	    val = val + m.get(i, j) * get(j);
	    if (verbose) printf("%2.0f * %2.0f + %2.0f  ", m.get(i,j), get(j), val);
	}
	ret_v.put(val, i);
	if (verbose) printf("\n");
	val = 0;
    }
    return ret_v;
}

StixVctr StixVctr::operator * (double s)
{
    StixVctr V;
    for (int i = 0; i < 4; i++)
	V.v[i] = v[i] * s;
    return V;
}

double StixVctr::get(int i)
{
    if (i <= 3 && i >= 0)
    {
	return v[i];
    }
    else
    {
	fprintf(stderr, "StixVctr: get: ERROR: index out of bounds\n");
	return 0;
    }
}
void StixVctr::put(double val, int i)
{
    if (i <= 3 && i >= 0)
    {
	v[i] = val;
    }
    else
    {
	fprintf(stderr, "StixVctr: put: ERROR: index out of bounds\n");
	exit(1);
    }
}
void StixVctr::normalizeInPlace(void) // in place normalization
{
    double denom = sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2]);
    for (int i = 0; i < 3; i++)
	v[i] = v[i] / denom;
}
StixVctr StixVctr::normalize(void)
{
    StixVctr V;
    double denom = sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2]);
    for (int i = 0; i < 3; i++)
	V.v[i] = v[i] / denom;
    V.v[3] = v[3];
    return V;
}
int isSame(double a, double b, double tol)
{
    if (fabs(a - b) > tol) return 0;
    else return 1;
}
StixVctr StixVctr::getDiffVec(void)
{
    StixVctr V;
    double a = get(0);
    double b = get(1);
    double c = get(2);
    /* if vector components are similar -- flip sign on one of them */
    if (isSame(a, b, 0.1) && isSame(b, c, 0.1)) {
	a = -a;
	V.put(a, 0);
	return V;
    }
    /* else rotate the components position-wise */
    double tmp = a;
    a = b; 
    b = c;
    c = tmp;
    V.put(a,0);
    V.put(b,1);
    V.put(c,2);
    return V;
}
StixVctr StixVctr::rightAngleTo(void)
{
    StixVctr V;
    normalizeInPlace();
    V = getDiffVec();
    V = *this * V;
    return V;
}
double StixVctr::length(void)
{
    return sqrt((v[0] * v[0] + v[1] * v[1] + v[2] * v[2]));
}
int StixVctr::isZeroVec(double tol)
{
    if (fabs(v[0]) < tol &&
	fabs(v[1]) < tol &&
	fabs(v[2]) < tol)
	return 1;
    else
	return 0;
}
void StixVctr::print(FILE * out, char * indent)
{
    if (indent)
	fprintf(out, "%s%f %f %f %f\n",
		indent, v[0], v[1], v[2], v[3]);
    else
	fprintf(out, "%f %f %f %f\n",
		v[0], v[1], v[2], v[3]);
}
